{-----------------------------------------------------------------------------

	Haskell ATerm Library
		
	Joost Visser
	CWI, Amsterdam

  This module is part of the ATerm library for Haskell. This module
  provides wrapper functions that take care of IO.
  
------------------------------------------------------------------------------}

module ATermIO where

import System
import ATermConversion 
import ATermReadWrite

--- Transformation wrapper ----------------------------------------------------

{-
atermIOwrap mtransform
  = do [fin,fout] <- getArgs
       sin        <- readFile fin
       hterm_in   <- return .fromATermString . dehyphen $ sin
       hterm_out  <- mtransform $ hterm_in
       sout       <- return . toATermString $ hterm_out
       writeFile fout sout
       
atermIOwrapShared mtransform
  = do [fin,fout] <- getArgs
       sin        <- readFile fin
       hterm_in   <- return .fromATermString . dehyphen $ sin
       hterm_out  <- mtransform $ hterm_in
       sout       <- return . toSharedATermString $ hterm_out
       writeFile fout sout
-}
     
atermIOwrap progName mtransform
  = do args <- getArgs
       opts <- return $ parseOptions progName args
       sin  <- readStream (fin opts)
       tin  <- return . fromATermString . dehyphen $ sin
       tout <- mtransform $ tin
       sout <- return . toString (format opts) $ tout
       writeStream (fout opts) sout 
    where
      readStream "#stdin#"	= getContents
      readStream f		= readFile f
      writeStream "#stdout#"	= putStr
      writeStread f		= writeFile f
      toString format 
        = case format of 
            "TEXT" -> toATermString
	    "TAF"  -> toSharedATermString
	    _      -> error $ "format unknown: "++"\n"++usage progName    
	
--- Helpers ------------------------------------------------------------------

dehyphen str
  = map aux str
    where aux '-' = '_'
          aux c   = c
       
--- Option handling ---------------------------------------------------------

data OptionsATermIO
  = OptionsATermIO { fin :: String, fout :: String, format :: String }
defaultOptionsATermIO
  = OptionsATermIO { fin = "#stdin#", fout = "#stdout#", format = "TEXT" }
  
parseOptions programName args
  = p args
    where 
      p []			= defaultOptionsATermIO
      p ("-t":args)		= (p args){ format = "TEXT" }
      p ("-s":args)		= (p args){ format = "TAF" }
      p ("-b":args)		= err "BAF format not supported!"
      p ("-i":fname:args)	= (p args){ fin = fname }
      p ("-o":fname:args)	= (p args){ fout = fname }
      p args			= err $ "Can't parse options: "++concat args
      err msg			= error $ msg++"\n"++usage programName

usage programName
  = unlines ["Usage","",
             "  "++programName++" [-i <fname>] [-o <fname>] [-t|-s]",
             "",
	     "Options","",
	     "  -i <fname>    name of input file    (default: stdin)",
	     "  -o <fname>    name of output file   (default: stdout)",
	     "  -t            output format is TEXT (plain text)",
	     "  -s            output format is TAF  (textual sharing)"
	    ]
    
-------------------------------------------------------------------------------
