%% gh -- a compiler for Generic Haskell.
%% Copyright (c) 2001 -- 2005  The Generic Haskell Team. Utrecht University
%% 
%% This library is free software; you can redistribute it and/or modify
%% it under the terms of the GNU Lesser General Public License as
%% published by the Free Software Foundation; either version 2.1 of the
%% License, or (at your option) any later version.
%% 
%% This library is distributed in the hope that it will be useful, but
%% WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%% Lesser General Public License for more details.
%% 
%% You should have received a copy of the GNU Lesser General Public
%% License along with this library; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
%%
%% $Id: ErrorUtils.lhs 194 2005-01-12 09:16:08Z andres $
%%
%% author: Andres Loeh (andres@cs.uu.nl)

%***************************************************************************
\section{Error message utilities}
%***************************************************************************

\begin{code}
module ErrorUtils
  ( internalError, unstaticError, errorLayout, paragraph, Exc(..), fromOK, notused )
  where

import Utils
import PPrint
import AbstractSyntax
import Name
\end{code}

The function |internalError| is supposed to be called in occasions
where a resulting error is a programming error of the GH prototype and
should not just be caused by specific user input.

\begin{code}
makeError                      :: Doc -> a
makeError d                    =  error $ nl ++ displayS (renderPretty 1.0 80 d) ""

internalError                  :: String -> a
internalError msg              =  makeError $
                                    errorLayout 
                                      []
                                      "internal error"
                                      (paragraph (msg ++ post))
  where
  post = "\n\nThis might be an error in the input that is not correctly "
           ++ "checked for, or it is a bug in the compiler."
           ++ "\n\nIf you want to issue a bug report, "
           ++ "please send an input file that caused the error together with "
           ++ "the error description to " ++ email ++ ". "
           ++ "Sorry for the inconvenience."

unstaticError                  :: String -> a
unstaticError msg              =  makeError $
                                    errorLayout
                                      []
                                      "error (should have been caught earlier)"
                                      (paragraph msg)

paragraph                      :: String -> Doc
paragraph                      =  vcat . map (fillSep . map text . words) . lines

errorLayout                    :: [Range]  {- list of ranges -}
                               -> String   {- type of error -}
                               -> Doc      {- description -}
                               -> Doc      {- the error message -}

errorLayout rs tp doc          =  hcat (punctuate (text ", ") $ map (text . show) rs) 
                                    <$> indent 4 (text "***" <+> text tp <+> text "***" <> text ":")
                                    <$> indent 4 doc

data Exc a = OK a | Error

instance Monad Exc where
  return  = OK
  m >>= f = case m of
              Error    -> Error
              OK x     -> f x

fromOK (OK x) = x
\end{code}

An error messages used to provide input attributes that should not be 
needed.

\begin{code}
notused                         :: a
notused                         =  internalError "Illegal use of input attribute."
\end{code}

