%% gh -- a compiler for Generic Haskell.
%% Copyright (c) 2001 -- 2005  The Generic Haskell Team. Utrecht University
%% 
%% This library is free software; you can redistribute it and/or modify
%% it under the terms of the GNU Lesser General Public License as
%% published by the Free Software Foundation; either version 2.1 of the
%% License, or (at your option) any later version.
%% 
%% This library is distributed in the hope that it will be useful, but
%% WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%% Lesser General Public License for more details.
%% 
%% You should have received a copy of the GNU Lesser General Public
%% License along with this library; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
%%
%% $Id: UHA_ParseMonad.lhs 194 2005-01-12 09:16:08Z andres $
%%
%% author: Andres Loeh (andres@cs.uu.nl)

> module UHA_ParseMonad           (  Result(..), LexContext(..), P, Lex(..), LexMode(..), unLex, run
>                               ,  parseError, parseErrorD, getPosition, getContext, pushContext, popContext, layoutOn)
> where
>
> import AbstractSyntax    {- for ranges -}
> import ErrorUtils
> import Monad
> -- import Pretty
> import PPrint
> -- import IOExts

A simple exception monad.

> data Result a                 =  Fail Doc | Return a
>                                  deriving (Show)

> instance Monad Result where
>     Fail s   >>= _k           =  Fail s
>     Return a >>= k            =  k a
>     return                    =  Return
>     fail                      =  Fail . paragraph

> failR                         =  Fail

> instance MonadPlus Result where
>     mzero                     =  failR empty
>     Fail _s `mplus` m         =  m
>     Return a `mplus` _m       =  Return a

> data LexContext               =  NoLayout | Layout Int
> 	                           deriving (Eq, Ord, Show)
>
> type Contexts                 =  [LexContext]

> data LexMode                  =  PlainUHA
>                               |  GenericHaskell
>                               |  GenericHaskellInterface
>   deriving (Eq,Show)

A state monad (in CPS style).

> type P a
>     =  String                 -- input string
>     -> LexMode                -- current lexing mode
>     -> Position               -- location of last token read
>     -> Int                    -- current column
>     -> Contexts               -- layout information
>     -> Result a

> newtype Lex a                 =  Lex (forall ans . (a -> P ans) -> P ans)
> unLex (Lex x)                 =  x

> run                           :: Lex a -> P a
> run (Lex p)                   =  p (\a -> \i mode l c s -> Return a)

> instance Monad Lex where
>     return a                  =  Lex (\ cont -> cont a)
>     m >>= k                   =  Lex (\ cont -> unLex m (\ a -> unLex (k a) cont))
>     fail s                    =  Lex (\ cont i mode l c stk -> fail s)

> failL                         :: Doc -> Lex a
> failL d                       =  Lex (\ cont i mode l c stk -> failR d)

> parseError                    :: String -> Lex a
> parseError s                  =  Lex (\ cont r mode pos c stk -> 
>                                         failR (txt pos))
>   where txt pos = errorLayout
>                     [Range_Range pos pos]
>                     "error: syntax error" 
>                     (paragraph s)

> parseErrorD                   :: Doc -> Lex a
> parseErrorD s                 =  Lex (\ cont r mode pos c stk -> 
>                                         failR (txt pos))
>   where txt pos = errorLayout
>                     [Range_Range pos pos]
>                     "error: syntax error" 
>                     s

Accessing the state.

> getMode                       :: Lex LexMode
> getMode                       =  Lex (\cont i mode l c s -> cont mode i mode l c s)

> getPosition                   :: Lex Position
> getPosition                   =  Lex (\cont i mode l c s -> cont l i mode l c s)

> getContext                    :: Lex Contexts
> getContext                    =  Lex (\cont i mode l c s -> cont s i mode l c s)

> setBOL                        :: Lex ()
> setBOL                        =  Lex (\cont i mode l c s -> cont () i mode l 0 s)

> pushContext                   :: LexContext -> Lex ()
> pushContext ctxt              = 
>     Lex (\cont i mode l c s -> cont () i mode l c (ctxt:s))

> popContext                    :: Lex ()
> popContext                    =
>     Lex (\ cont i mode l@(Position_Position f y x) c stk ->
>            case stk of
>                []      -> {- trace ("layout popo on empty " ++ show l) $ -} 
>                           cont () i mode l c 
>                             (fail (f ++ ":" ++ show y ++ ":" ++ show x ++ ": " ++ "Internal error: empty context in popContext"))
>   	         (_ : s) -> {- trace ("layout pop " ++ show l) $ -}
>                           cont () i mode l c s)

> layoutOn                      :: Bool -> Lex ()
> layoutOn strict               =  {-
>     getPosition >>= \(Position_Position _ r c) -> trace ("layout on " ++ show c) $ pushContext (Layout c)
> -}
>     do loc@(Position_Position f y x) <- getPosition
>        stk <- getContext
>        case stk of
>          Layout prev_off : _
>            | if strict then prev_off >= x else prev_off > x
>                 -> do -- trace ("layout hack " ++ show loc ++ " (versus " ++ show prev_off ++ ")") $ return ()
>                       pushContext (Layout (x + 1))
>                       setBOL
>          _      -> do -- trace ("layout on " ++ show loc) $ return ()
>                       pushContext (Layout x)

