-- gh -- a compiler for Generic Haskell.
-- Copyright (c) 2001 -- 2005  The Generic Haskell Team. Utrecht University
-- 
-- This library is free software; you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as
-- published by the Free Software Foundation; either version 2.1 of the
-- License, or (at your option) any later version.
-- 
-- This library is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-- Lesser General Public License for more details.
-- 
-- You should have received a copy of the GNU Lesser General Public
-- License along with this library; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
--
-- $Id: TypeConstruction.hs 194 2005-01-12 09:16:08Z andres $
--
-- author: Andres Loeh (andres@cs.uu.nl)

module TypeConstruction where

import AbstractSyntax
import UHA_Range
import Name

appT      =  Type_Application unknownRange True
varT      =  Type_Variable unknownRange
conT      =  Type_Constructor unknownRange
qualT     =  Type_Qualified unknownRange
faT       =  Type_Forall unknownRange
exT       =  Type_Exists unknownRange
parT      =  Type_Parenthesized unknownRange
concaseT  =  Type_ConstructorCase unknownRange
tappT     =  Type_TypeApplication unknownRange
kappT     =  Type_KindApplication unknownRange

varTA     =  TypeArgument_Variable unknownRange

varNT = varT . localName
conNT = conT . localName
faNT vs ts = faT (map localName vs) ts
exNT vs ts = exT (map localName vs) ts

ctxC  = ContextItem_ContextItem unknownRange
ctxNC n = ctxC (localName n)

funT  = conT hsFunName
funAT l r = Type_Application unknownRange False funT [l,r]

listT = conT hsListName
listAT a = Type_Application unknownRange False listT [a]

tupleT n = conT (hsTupleConName n)
pairT = tupleT 2
tupleAT n = Type_Application unknownRange False (tupleT n)
pairAT = tupleAT 2

unitT = conT hsUnitConName

type LambdaType                  = (Names,Type)

-- Here, the list of names in the first component is a list of type variables
-- that the type in the second component is abstraced over.

type KindIndexedType             = (Kind,             -- "base" kind, always |*| for non-GAs
                                    Maybe (Int,Int),  -- arity, only for EGH style
                                    Kind -> LambdaType)

type GenericTypeSignature        = (Name,KindAnnotatedNames,KindAnnotatedNames,Type)
