module UHA_Range where

import AbstractSyntax

before :: Position -> Position -> Maybe Bool
before (Position_Unknown) _ = Nothing
before _ (Position_Unknown) = Nothing
before (Position_Position f1 l1 c1) (Position_Position f2 l2 c2)
  | f1 == f2 = Just $ (l1,c1) <= (l2,c2)
  | otherwise = Nothing

unknownRange :: Range
unknownRange =  Range_Range Position_Unknown Position_Unknown

class HasRange a where
  getRange :: a -> Range

instance HasRange a => HasRange [a] where
  getRange xs = combineRanges (map getRange xs)

combineRanges :: [Range] -> Range
combineRanges []  = unknownRange
combineRanges [x] = x
combineRanges xs  = rangeFromTo (head xs) (last xs)

rangeFromTo :: Range -> Range -> Range
rangeFromTo (Range_Range f _) (Range_Range _ t) = Range_Range f t

makeRange :: Int -> Position -> Range
makeRange n p = Range_Range p (advance n p)
  where advance :: Int -> Position -> Position
        advance n (Position_Unknown) = Position_Unknown
        advance n (Position_Position f l c) = Position_Position f l (c+n)

instance HasRange InterfaceDeclaration where
  getRange (InterfaceDeclaration_Declarations range _) = range
  getRange (InterfaceDeclaration_Abstract range _ _ _ _) = range
  getRange (InterfaceDeclaration_Consig range _ _ _) = range
  getRange (InterfaceDeclaration_Binding range _) = range
  getRange (InterfaceDeclaration_SetModule range _ _) = range
  getRange (InterfaceDeclaration_Import range _) = range

instance HasRange Module where 
  getRange (Module_Module range _ _ _) = range

instance HasRange MaybeExports where 
  getRange (MaybeExports_Nothing)  = unknownRange
  getRange (MaybeExports_Just exports) = getRange exports

instance HasRange Export where
  getRange (Export_Variable range _) = range
  getRange (Export_TypeOrClass range _ _) = range
  getRange (Export_TypeOrClassComplete range _) = range
  getRange (Export_Module range _) = range

instance HasRange Body where
  getRange (Body_Body range _ _) = range

instance HasRange ImportDeclaration where
  getRange (ImportDeclaration_Import range _ _ _ _) = range
  getRange (ImportDeclaration_Empty range) = range

instance HasRange MaybeImportSpecification where
  getRange (MaybeImportSpecification_Nothing) = unknownRange
  getRange (MaybeImportSpecification_Just spec) = getRange spec

instance HasRange ImportSpecification where
  getRange (ImportSpecification_Import range _ _) = range

instance HasRange Import where
  getRange (Import_Variable range _) = range
  getRange (Import_TypeOrClass range _ _) = range
  getRange (Import_TypeOrClassComplete range _) = range

instance HasRange MaybeDeclarations where
  getRange (MaybeDeclarations_Nothing) = unknownRange
  getRange (MaybeDeclarations_Just declarations) = getRange declarations

instance HasRange Declaration where
  getRange (Declaration_Type range _ _) = range
  getRange (Declaration_Data range _ _ _ _) = range
  getRange (Declaration_Newtype range _ _ _ _) = range
  getRange (Declaration_Class range _ _ _) = range
  getRange (Declaration_Instance range _ _ _ _) = range
  getRange (Declaration_Default range _) = range
  getRange (Declaration_FunctionBindings range _) = range
  getRange (Declaration_PatternBinding range _ _) = range
  getRange (Declaration_TypeSignature range _ _) = range
  getRange (Declaration_Fixity range _ _ _) = range
  getRange (Declaration_Empty range) = range
  {- <generic-haskell> -}
  getRange (Declaration_GenericTypeSignature range _ _ _ _) = range
  getRange (Declaration_KindIndexedType range _) = range
  getRange (Declaration_TypeIndexedValue range _) = range
  getRange (Declaration_Dependency range _ _ _) = range
  getRange (Declaration_TypeIndexedType range _ _ _ _ _) = range
  {- </generic-haskell> -}

{- <generic-haskell> -}
instance HasRange Reassignment where
  getRange (Reassignment_Reassignment range _ _) = range
{- </generic-haskell> -}

instance HasRange Fixity where
  getRange (Fixity_Infixl range) = range
  getRange (Fixity_Infixr range) = range
  getRange (Fixity_Infix range) = range

instance HasRange Type where
  getRange (Type_Application range _ _ _) = range
  getRange (Type_Variable range _) = range
  getRange (Type_Constructor range _) = range
  getRange (Type_Qualified range _ _) = range
  getRange (Type_Forall range _ _) = range
  getRange (Type_Exists range _ _) = range
  getRange (Type_Parenthesized range _) = range
  {- <generic-haskell> -}
  getRange (Type_TypeApplication range _ _) = range
  getRange (Type_KindApplication range _ _) = range
  getRange (Type_ConstructorCase range _) = range
  getRange (Type_KindStarCase range) = range
  {- </generic-haskell> -}

instance HasRange SimpleType where
  getRange (SimpleType_SimpleType range n as) = range

{- <generic-haskell> -}
instance HasRange KindAnnotatedName where
  getRange (KindAnnotatedName_KindAnnotatedName range _ _) = range

instance HasRange MaybeKind where
  getRange (MaybeKind_Nothing) = unknownRange
  getRange (MaybeKind_Just kind) = getRange kind
{- </generic-haskell> -}

instance HasRange ContextItem where
  getRange (ContextItem_ContextItem range _ _) = range
  getRange (ContextItem_DependencyConstraint range _ _ _) = range
  getRange (ContextItem_DependencyItem range _ _ _ _) = range

instance HasRange Constructor where
  getRange (Constructor_Constructor range _ _) = range
  getRange (Constructor_Infix range _ _ _) = range
  getRange (Constructor_Record range _ _) = range

instance HasRange FieldDeclaration where
  getRange (FieldDeclaration_FieldDeclaration range _ _) = range

instance HasRange AnnotatedType where
  getRange (AnnotatedType_AnnotatedType range _ _) = range

instance HasRange MaybeExpression where
  getRange (MaybeExpression_Nothing) = unknownRange
  getRange (MaybeExpression_Just expression) = getRange expression

instance HasRange Expression where
  getRange (Expression_Literal range _) = range
  getRange (Expression_Variable range _) = range
  getRange (Expression_Constructor range _) = range
  getRange (Expression_Parenthesized range _) = range
  getRange (Expression_NormalApplication range _ _) = range
  getRange (Expression_InfixApplication range _ _ _) = range
  getRange (Expression_If range _ _ _) = range
  getRange (Expression_Lambda range _ _) = range
  getRange (Expression_Case range _ _) = range
  getRange (Expression_Let range _ _) = range
  getRange (Expression_Do range _) = range
  getRange (Expression_List range _) = range
  getRange (Expression_Tuple range _) = range
  getRange (Expression_Comprehension range _ _) = range
  getRange (Expression_Typed range _ _) = range
  getRange (Expression_RecordConstruction range _ _) = range
  getRange (Expression_RecordUpdate range _ _) = range
  getRange (Expression_Enum range _ _ _) = range
  getRange (Expression_Negate range _) = range
  {- <generic-haskell> -}
  getRange (Expression_As range _ _) = range
  getRange (Expression_Wildcard range) = range
  getRange (Expression_Irrefutable range _) = range
  getRange (Expression_TypeApplication range _ _) = range
  {- </generic-haskell> -}

instance HasRange Statement where
  getRange (Statement_Expression range _) = range
  getRange (Statement_Let range _) = range
  getRange (Statement_Generator range _ _) = range
  getRange (Statement_Empty range) = range

instance HasRange Qualifier where
  getRange (Qualifier_Guard range _) = range
  getRange (Qualifier_Let range _) = range
  getRange (Qualifier_Generator range _ _) = range
  getRange (Qualifier_Empty range) = range

instance HasRange Alternative where
  getRange (Alternative_Alternative range _ _) = range
  getRange (Alternative_Empty range) = range

instance HasRange GuardedExpression where
  getRange (GuardedExpression_GuardedExpression range _ _) = range

instance HasRange RecordExpressionBinding where
  getRange (RecordExpressionBinding_RecordExpressionBinding range _ _) = range

instance HasRange RecordPatternBinding where
  getRange (RecordPatternBinding_RecordPatternBinding range _ _) = range

instance HasRange FunctionBinding where
  getRange (FunctionBinding_FunctionBinding range _ _) = range

instance HasRange LeftHandSide where
  getRange (LeftHandSide_Function range _ _) = range
  getRange (LeftHandSide_Infix range _ _ _) = range
  getRange (LeftHandSide_Parenthesized range _ _) = range

instance HasRange RightHandSide where
  getRange (RightHandSide_Expression range _ _) = range
  getRange (RightHandSide_Guarded range _ _) = range

instance HasRange Pattern where
  getRange (Pattern_Literal range _) = range
  getRange (Pattern_Variable range _) = range
  getRange (Pattern_Constructor range _ _) = range
  getRange (Pattern_Parenthesized range _) = range
  getRange (Pattern_InfixConstructor range _ _ _) = range
  getRange (Pattern_List range _) = range
  getRange (Pattern_Tuple range _) = range
  getRange (Pattern_Record range _ _) = range
  getRange (Pattern_Negate range _) = range
  getRange (Pattern_As range _ _) = range
  getRange (Pattern_Wildcard range) = range
  getRange (Pattern_Irrefutable range _) = range
  getRange (Pattern_Successor range _ _) = range

instance HasRange Literal where
  getRange (Literal_Int range _) = range
  getRange (Literal_Char range _) = range
  getRange (Literal_Float range _) = range
  getRange (Literal_String range _) = range

instance HasRange MaybeName where
  getRange (MaybeName_Nothing) = unknownRange
  getRange (MaybeName_Just name) = getRange name

instance HasRange Name where
  getRange (Name_Identifier range _ _) = range
  getRange (Name_Operator range _ _) = range
  getRange (Name_Special range _ _) = range
  {- <generic-haskell> -}
  getRange (Name_Entity range _ _) = range
  {- </generic-haskell> -}

instance HasRange ModuleName where
  getRange (ModuleName_ModuleName range _) = range

{- <generic-haskell> -}

instance HasRange Kind where
  getRange (Kind_Star range) = range
  getRange (Kind_Variable range _) = range
  getRange (Kind_Arrow range _ _) = range
  getRange (Kind_Parenthesized range _) = range

instance HasRange KindArgument where
  getRange (KindArgument_Star range) = range
  getRange (KindArgument_Arrow range _ _) = range

instance HasRange DependencyBinding where
  getRange (DependencyBinding_Simple range _) = range
  getRange (DependencyBinding_Rename range _ _) = range

instance HasRange TypeArgument where
  getRange (TypeArgument_NamedType range _) = range
  getRange (TypeArgument_Variable range _) = range
  getRange (TypeArgument_Constructor range _) = range
  getRange (TypeArgument_Label range _) = range
  getRange (TypeArgument_ConstructorCase range _) = range
  getRange (TypeArgument_KindStarCase range) = range

{-
instance HasRange TypeBinding where
  getRange (TypeBinding_TypeBinding range _ _ _) = range
-}

{- </generic-haskell> -}
