






























































































  module UHA_Parser (parseFile,parseImport,parseType) where
  
  import AbstractSyntax 
  import UHA_ParseMonad hiding (Result)
  import UHA_Lexer
  import UHA_ParseUtils
  import UHA_Range
  import Monad
  import Name
  import FileNameUtils
  import OSSpecific
  import ErrorUtils
  import List
  import Char (isAlpha)
  import IO
  import Utils
  import PPrint
  -- import GlaExts

  type Terminal = Token
  type Result   = Lex



  parseFile                      :: Int             -- verbose?
                                 -> [String]        -- search path
                                 -> FilePath        -- filename
                                 -> IO (Maybe Module,FilePath)
  parseFile verbose p fp         
    = catch
        ( do
            (f,fn) <- chaseFile p fp 
            case run xModule f GenericHaskell (Position_Position fp 1 1) 0 [] of
                Fail err ->
                  do c <- getTerminalWidth
                     displayIO stdout (renderPretty 1.0 c err)
                     atVerbosity 0 $ "\n"
                     return (Nothing,fn)
                Return r -> return $ (Just r,fn)
        )
        (\ _ -> do atVerbosity 0 $ "File " ++ fp ++ " not found.\n"
                   return (Nothing,"")
        )
   where 
      atVerbosity n s = if verbose >= n then putStr s else return ()     



  parseImport                    :: Int             -- verbose?
                                 -> [String]        -- search path
                                 -> ModuleName      -- module name
                                 -> IO (ModuleInterface  -- the abstract representation
                                       ,[ModuleName]     -- modules we depend on
                                       )
  parseImport verbose p m = 
    do let (ModuleName_ModuleName _ file_) = m
           file                  =  concat (intersperse [directorySeparator] file_) ++ ".ghi"
           atVerbosity n s       =  if verbose >= n then putStr s 
                                                    else return ()
 
       x <- catch (      do (f,fn) <- chaseFile p file
                            atVerbosity 1 $ "Interface file " ++ fn ++ " read.\n"
                            return $ Just f
                  )
                  (\_ -> do atVerbosity 0 $ "Interface file " ++ file ++ " not found.\n"
                            return Nothing
                  )
       
       case x of
         Nothing -> return (ModuleInterface_Unavailable m,[])
         Just f  -> case run xInterface f
                             GenericHaskellInterface (Position_Position file 1 1) 0 [] of
                      Fail err -> do c <- getTerminalWidth
                                     displayIO stdout (renderPretty 1.0 c err)
                                     atVerbosity 0 $ "\n\nInterface file ignored.\n"
                                     return (ModuleInterface_Unavailable m,[])
                      Return r -> return r



  parseType                      :: String -> Type
  parseType s
    = case run xQualType s GenericHaskell 
               (Position_Position "<input>" 1 1) 0 [] of
        Fail err -> error "parse error"
        Return r -> r



  frown ex t                     
    = parseErrorD ( text "found:" <+> text (errshow t )
                      <$> nest 4 ( text "expected:"
                                     <+> cat (punctuate (text ", ") (form ex))
                                 )
                      <$> empty
                  )
   where form [] = [text "<end of file>"]
         form ex = map (text . frm) ex
         frm n@(x:xs) | isAlpha x || x == '_' = "\"" ++ n ++ "\""
                      | x == '<' || x == '\'' = n
                      | otherwise             = "'" ++ n ++ "'"
  
  {-
  instance MonadPlus Lex where
      mzero                     =  fail ""
      mplus m n                 =  m
  -}
  
  

{-



-}



{- frown :-( -}

  xInterface = get >>= state_1__ (\ v1 _ -> return (v1))
  xModule = get >>= state_4__ (\ v1 _ -> return (v1))
  xQnames = get >>= state_7__ (\ v1 _ -> return (v1))
  xDecls = get >>= state_10__ (\ v1 _ -> return (v1))
  xQualType = get >>= state_13__ (\ v1 _ -> return (v1))
  xExp = get >>= state_16__ (\ v1 _ -> return (v1))
  xConVar = get >>= state_19__ (\ v1 _ -> return (v1))
  xVar = get >>= state_22__ (\ v1 _ -> return (v1))
  xCon = get >>= state_25__ (\ v1 _ -> return (v1))
  xSKind = get >>= state_28__ (\ v1 _ -> return (v1))
  xKind = get >>= state_31__ (\ v1 _ -> return (v1))
  
  state_1__ k_1_0__ t__
      = let {
              goto_xInterface'1__ v1 = state_2__ (k_1_0__ v1)
            }
        in case t__ of {
               KW_Interface v1 -> get >>= state_34__ (reduce_12__ goto_xInterface'1__ v1);
               _ -> frown ["__ghi_interface"] t__ }
  
  state_2__ k_1_1__ t__ = case t__ of { EOF -> get >>= k_1_1__; _ -> frown [] t__ }
  
  state_4__ k_2_0__ t__
      = let {
              goto_xModule'1__ v1 = state_5__ (k_2_0__ v1);
              goto_body'2__ v1 v2 = reduce_14__ goto_xModule'1__ v1 v2;
              goto_layout_on'1__ v1 = state_127__ (reduce_54__ goto_body'2__ v1)
            }
        in case t__ of {
               LeftCurly v1 -> get >>= state_124__ (reduce_53__ goto_body'2__ v1);
               KW_Module v1 -> get >>= state_43__ (reduce_13__ goto_xModule'1__ v1);
               _ -> reduce_290__ goto_layout_on'1__ t__ }
  
  state_5__ k_2_1__ t__ = case t__ of { EOF -> get >>= k_2_1__; _ -> frown [] t__ }
  
  state_7__ k_3_0__ t__
      = let {
              goto_xQnames'1__ v1 = state_8__ (k_3_0__ v1);
              goto_many'1_qcname'1__ v1 = reduce_15__ goto_xQnames'1__ v1;
              goto_many''1_qcname'1__ v1
                  = state_651__ (reduce_358__ goto_many'1_qcname'1__ v1) (reduce_365__ goto_many''1_qcname'1__ v1)
            }
        in reduce_364__ goto_many''1_qcname'1__ t__
  
  state_8__ k_3_1__ t__ = case t__ of { EOF -> get >>= k_3_1__; _ -> frown [] t__ }
  
  state_10__ k_4_0__ t__
      = let {
              goto_xDecls'1__ v1 = state_11__ (k_4_0__ v1);
              goto_fixdecl'1__ v1 = reduce_108__ goto_decl'1__ v1;
              goto_infix'2__ v1 v2 = state_233__ (reduce_110__ goto_fixdecl'1__ v1 v2);
              goto_topdecls'1__ v1 = reduce_16__ goto_xDecls'1__ v1;
              goto_topdecls''1__ v1
                  = state_179__ (reduce_88__ goto_topdecls'1__ v1) (reduce_89__ goto_topdecls''1__ v1)
                        (reduce_90__ goto_topdecls''1__ v1);
              goto_realtopdecl'1__ v1 = reduce_92__ goto_topdecl'1__ v1;
              goto_topdecl'1__ v1 = reduce_91__ goto_topdecls''1__ v1;
              goto_decl'1__ v1 = reduce_93__ goto_topdecl'1__ v1;
              goto_signdecl'1__ v1 = reduce_107__ goto_decl'1__ v1;
              goto_vars'1__ v1 = state_250__ (reduce_121__ goto_vars2'1__ v1) (reduce_122__ goto_vars'1__ v1);
              goto_vars2'1__ v1 = state_244__ (reduce_119__ goto_signdecl'1__ v1);
              goto_valdef'1__ v1 = reduce_109__ goto_decl'1__ v1;
              goto_exp0b'1__ v1
                  = state_247__ (reduce_120__ goto_signdecl'1__ v1) (reduce_195__ goto_valdef'1__ v1)
                            (reduce_196__ goto_valdef'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_257__ (reduce_123__ goto_vars'1__ v1) (reduce_197__ goto_valdef'1__ v1)
                                    (reduce_198__ goto_valdef'1__ v1)
                                (reduce_390__ goto_aexp'1__ v1)
                            (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Class v1 -> get >>= state_199__ (reduce_97__ goto_realtopdecl'1__ v1);
               KW_Data v1 -> get >>= state_189__ (reduce_95__ goto_realtopdecl'1__ v1);
               KW_Default v1 -> get >>= state_205__ (reduce_99__ goto_realtopdecl'1__ v1);
               KW_Dependency v1
                   -> get >>= state_219__ (reduce_102__ goto_realtopdecl'1__ v1) (reduce_103__ goto_realtopdecl'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Infix v1 -> get >>= reduce_116__ goto_infix'2__ v1;
               KW_InfixL v1 -> get >>= reduce_117__ goto_infix'2__ v1;
               KW_InfixR v1 -> get >>= reduce_118__ goto_infix'2__ v1;
               KW_Instance v1 -> get >>= state_202__ (reduce_98__ goto_realtopdecl'1__ v1);
               KW_NewType v1 -> get >>= state_194__ (reduce_96__ goto_realtopdecl'1__ v1);
               KW_Type v1
                   -> get
                          >>=
                          state_185__ (reduce_94__ goto_realtopdecl'1__ v1) (reduce_100__ goto_realtopdecl'1__ v1)
                              (reduce_101__ goto_realtopdecl'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "~",
                               "-",
                               "as",
                               "case",
                               "class",
                               "data",
                               "default",
                               "dependency",
                               "do",
                               "hiding",
                               "infix",
                               "infixl",
                               "infixr",
                               "instance",
                               "newtype",
                               "type",
                               "qualified"]
                          t__ }
  
  state_11__ k_4_1__ t__ = case t__ of { EOF -> get >>= k_4_1__; _ -> frown [] t__ }
  
  state_13__ k_5_0__ t__
      = let {
              goto_xQualType'1__ v1 = state_14__ (k_5_0__ v1);
              goto_xType'1__ v1 = reduce_19__ goto_xQualType'1__ v1;
              goto_btype'1__ v1
                  = state_51__ (reduce_17__ goto_xQualType'1__ v1) (reduce_124__ goto_xType'1__ v1)
                                (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1;
              goto_foralls'2__ v1 v2 = state_54__ (reduce_18__ goto_xQualType'1__ v1 v2)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Forall v1 -> get >>= state_617__ (reduce_333__ goto_foralls'2__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "(",
                               "[",
                               "as",
                               "forall",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_14__ k_5_1__ t__ = case t__ of { EOF -> get >>= k_5_1__; _ -> frown [] t__ }
  
  state_16__ k_6_0__ t__
      = let {
              goto_xExp'1__ v1 = state_17__ (k_6_0__ v1);
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_58__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                            (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_17__ k_6_1__ t__ = case t__ of { EOF -> get >>= k_6_1__; _ -> frown [] t__ }
  
  state_19__ k_7_0__ t__
      = let {
              goto_xConVar'1__ v1 = state_20__ (k_7_0__ v1);
              goto_var'1__ v1 = reduce_22__ goto_xConVar'1__ v1;
              goto_con'1__ v1 = reduce_23__ goto_xConVar'1__ v1;
              goto_varid'2__ v1 v2 = reduce_247__ goto_var'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_251__ goto_con'1__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               LeftParen v1 -> get >>= state_479__ (reduce_248__ goto_var'1__ v1) (reduce_252__ goto_con'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<conid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_20__ k_7_1__ t__ = case t__ of { EOF -> get >>= k_7_1__; _ -> frown [] t__ }
  
  state_22__ k_8_0__ t__
      = let {
              goto_xVar'1__ v1 = state_23__ (k_8_0__ v1);
              goto_var'1__ v1 = reduce_24__ goto_xVar'1__ v1;
              goto_varid'2__ v1 v2 = reduce_247__ goto_var'1__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               LeftParen v1 -> get >>= state_478__ (reduce_248__ goto_var'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_23__ k_8_1__ t__ = case t__ of { EOF -> get >>= k_8_1__; _ -> frown [] t__ }
  
  state_25__ k_9_0__ t__
      = let {
              goto_xCon'1__ v1 = state_26__ (k_9_0__ v1);
              goto_con'1__ v1 = reduce_25__ goto_xCon'1__ v1;
              goto_conid'2__ v1 v2 = reduce_251__ goto_con'1__ v1 v2
            }
        in case t__ of {
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               LeftParen v1 -> get >>= state_492__ (reduce_252__ goto_con'1__ v1);
               _ -> frown ["<conid>", "("] t__ }
  
  state_26__ k_9_1__ t__ = case t__ of { EOF -> get >>= k_9_1__; _ -> frown [] t__ }
  
  state_28__ k_10_0__ t__
      = let {
              goto_xSKind'1__ v1 = state_29__ (k_10_0__ v1);
              goto_varop'1__ v1 = reduce_257__ goto_op'1__ v1;
              goto_conop'1__ v1 = reduce_258__ goto_op'1__ v1;
              goto_op'1__ v1 = reduce_319__ goto_kindarg'1__ v1;
              goto_varid'2__ v1 v2 = reduce_326__ goto_kivar'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_255__ goto_conop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_259__ goto_varop'1__ v1 v2;
              goto_kindarg'1__ v1 = reduce_26__ goto_xSKind'1__ v1;
              goto_kivar'1__ v1 = state_597__ (reduce_320__ goto_kindarg'1__ v1)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               BackQuote v1 -> get >>= state_503__ (reduce_256__ goto_conop'1__ v1) (reduce_260__ goto_varop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<varsym>", "<consym>", "`", ".", "-", "!", "as", "hiding", "qualified"] t__ }
  
  state_29__ k_10_1__ t__ = case t__ of { EOF -> get >>= k_10_1__; _ -> frown [] t__ }
  
  state_31__ k_11_0__ t__
      = let {
              goto_xKind'1__ v1 = state_32__ (k_11_0__ v1);
              goto_varop'1__ v1 = reduce_257__ goto_op'1__ v1;
              goto_conop'1__ v1 = reduce_258__ goto_op'1__ v1;
              goto_op'1__ v1 = reduce_323__ goto_kindterm'1__ v1;
              goto_varid'2__ v1 v2 = reduce_326__ goto_kivar'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_255__ goto_conop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_259__ goto_varop'1__ v1 v2;
              goto_kind'1__ v1 = reduce_27__ goto_xKind'1__ v1;
              goto_kindterm'1__ v1 = state_600__ (reduce_321__ goto_kind'1__ v1) (reduce_322__ goto_kind'1__ v1);
              goto_kivar'1__ v1 = reduce_324__ goto_kindterm'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               LeftParen v1 -> get >>= state_605__ (reduce_325__ goto_kindterm'1__ v1);
               BackQuote v1 -> get >>= state_503__ (reduce_256__ goto_conop'1__ v1) (reduce_260__ goto_varop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<varsym>", "<consym>", "(", "`", ".", "-", "!", "as", "hiding", "qualified"] t__ }
  
  state_32__ k_11_1__ t__ = case t__ of { EOF -> get >>= k_11_1__; _ -> frown [] t__ }
  
  state_34__ k_12_1__ t__
      = case t__ of { IntTok v1 v2 -> get >>= state_35__ (k_12_1__ v1 v2); _ -> frown ["<integer literal>"] t__ }
  
  state_35__ k_12_2__ t__
      = let {
              goto_modid'1__ v1 = state_36__ (k_12_2__ v1)
            }
        in case t__ of {
               ConId v1 v2 -> get >>= reduce_292__ goto_modid'1__ v1 v2;
               QConId v1 v2 -> get >>= reduce_293__ goto_modid'1__ v1 v2;
               _ -> frown ["<conid>", "<qconid>"] t__ }
  
  state_36__ k_12_3__ t__
      = case t__ of { KW_ExportTypes v1 -> get >>= state_37__ (k_12_3__ v1); _ -> frown ["__ghi_export_types"] t__ }
  
  state_37__ k_12_4__ t__
      = let {
              goto_xQnames'1__ v1 = state_38__ (k_12_4__ v1);
              goto_many'1_qcname'1__ v1 = reduce_15__ goto_xQnames'1__ v1;
              goto_many''1_qcname'1__ v1
                  = state_651__ (reduce_358__ goto_many'1_qcname'1__ v1) (reduce_365__ goto_many''1_qcname'1__ v1)
            }
        in reduce_364__ goto_many''1_qcname'1__ t__
  
  state_38__ k_12_5__ t__
      = case t__ of { KW_ExportValues v1 -> get >>= state_39__ (k_12_5__ v1); _ -> frown ["__ghi_export_values"] t__ }
  
  state_39__ k_12_6__ t__
      = let {
              goto_xQnames'1__ v1 = state_40__ (k_12_6__ v1);
              goto_many'1_qcname'1__ v1 = reduce_15__ goto_xQnames'1__ v1;
              goto_many''1_qcname'1__ v1
                  = state_651__ (reduce_358__ goto_many'1_qcname'1__ v1) (reduce_365__ goto_many''1_qcname'1__ v1)
            }
        in reduce_364__ goto_many''1_qcname'1__ t__
  
  state_40__ k_12_7__ t__ = case t__ of { KW_Where v1 -> get >>= state_41__ (k_12_7__ v1); _ -> frown ["where"] t__ }
  
  state_41__ k_12_8__ t__
      = let {
              goto_ibody'2__ v1 v2 = k_12_8__ v1 v2;
              goto_layout_on'1__ v1 = state_74__ (reduce_29__ goto_ibody'2__ v1)
            }
        in case t__ of {
               LeftCurly v1 -> get >>= state_71__ (reduce_28__ goto_ibody'2__ v1);
               _ -> reduce_290__ goto_layout_on'1__ t__ }
  
  state_43__ k_13_1__ t__
      = let {
              goto_modid'1__ v1 = state_44__ (k_13_1__ v1)
            }
        in case t__ of {
               ConId v1 v2 -> get >>= reduce_292__ goto_modid'1__ v1 v2;
               QConId v1 v2 -> get >>= reduce_293__ goto_modid'1__ v1 v2;
               _ -> frown ["<conid>", "<qconid>"] t__ }
  
  state_44__ k_13_2__ t__
      = let {
              goto_exports'1__ v1 = reduce_339__ goto_opt'1_exports'1__ v1;
              goto_opt'1_exports'1__ v1 = state_45__ (k_13_2__ v1)
            }
        in case t__ of {
               LeftParen v1 -> get >>= state_623__ (reduce_340__ goto_exports'1__ v1) (reduce_341__ goto_exports'1__ v1);
               KW_Where v1 -> reduce_338__ goto_opt'1_exports'1__ t__;
               _ -> frown ["(", "where"] t__ }
  
  state_45__ k_13_3__ t__ = case t__ of { KW_Where v1 -> get >>= state_46__ (k_13_3__ v1); _ -> frown ["where"] t__ }
  
  state_46__ k_13_4__ t__
      = let {
              goto_body'2__ v1 v2 = k_13_4__ v1 v2;
              goto_layout_on'1__ v1 = state_127__ (reduce_54__ goto_body'2__ v1)
            }
        in case t__ of {
               LeftCurly v1 -> get >>= state_124__ (reduce_53__ goto_body'2__ v1);
               _ -> reduce_290__ goto_layout_on'1__ t__ }
  
  state_51__ k_17_1__ k_124_1__ k_125_1__ k_126_1__ k_127_1__ t__
      = let {
              goto_atype'1__ v1 = k_127_1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_qconsym'1__ v1 = state_262__ (k_125_1__ v1);
              goto_qconsym2'2__ v1 v2 = reduce_277__ goto_qconsym'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_278__ goto_qconsym2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               QConSym v1 v2 -> get >>= reduce_279__ goto_qconsym2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               RightArrow v1 -> get >>= state_260__ (k_124_1__ v1);
               DoubleArrow v1 -> get >>= state_52__ (k_17_1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> k_126_1__ t__ }
  
  state_52__ k_17_2__ t__
      = let {
              goto_xQualType'1__ v1 = k_17_2__ v1;
              goto_xType'1__ v1 = reduce_19__ goto_xQualType'1__ v1;
              goto_btype'1__ v1
                  = state_51__ (reduce_17__ goto_xQualType'1__ v1) (reduce_124__ goto_xType'1__ v1)
                                (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1;
              goto_foralls'2__ v1 v2 = state_54__ (reduce_18__ goto_xQualType'1__ v1 v2)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Forall v1 -> get >>= state_617__ (reduce_333__ goto_foralls'2__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "(",
                               "[",
                               "as",
                               "forall",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_54__ k_18_1__ t__
      = let {
              goto_xQualType'1__ v1 = k_18_1__ v1;
              goto_xType'1__ v1 = reduce_19__ goto_xQualType'1__ v1;
              goto_btype'1__ v1
                  = state_51__ (reduce_17__ goto_xQualType'1__ v1) (reduce_124__ goto_xType'1__ v1)
                                (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1;
              goto_foralls'2__ v1 v2 = state_54__ (reduce_18__ goto_xQualType'1__ v1 v2)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Forall v1 -> get >>= state_617__ (reduce_333__ goto_foralls'2__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "(",
                               "[",
                               "as",
                               "forall",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_57__ k_19_1__ k_148_1__ t__
      = case t__ of {
            RightParen v1 -> k_19_1__ t__;
            Comma v1 -> get >>= state_307__ (k_148_1__ v1);
            _ -> frown [")", ","] t__ }
  
  state_58__ k_20_1__ k_214_1__ k_216_1__ k_218_1__ t__
      = let {
              goto_qvarop'1__ v1 = reduce_261__ goto_qop'1__ v1;
              goto_qconop'1__ v1 = reduce_262__ goto_qop'1__ v1;
              goto_qop'1__ v1 = state_405__ (k_216_1__ v1) (k_218_1__ v1);
              goto_qconsym2'2__ v1 v2 = reduce_265__ goto_qconop'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_278__ goto_qconsym2'2__ v1 v2;
              goto_qvarsym'2__ v1 v2 = reduce_263__ goto_qvarop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_281__ goto_qvarsym'2__ v1 v2;
              goto_qvarsym1'2__ v1 v2 = reduce_282__ goto_qvarsym'2__ v1 v2
            }
        in case t__ of {
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               QVarSym v1 v2 -> get >>= reduce_287__ goto_qvarsym1'2__ v1 v2;
               QConSym v1 v2 -> get >>= reduce_279__ goto_qconsym2'2__ v1 v2;
               BackQuote v1 -> get >>= state_514__ (reduce_264__ goto_qvarop'1__ v1) (reduce_266__ goto_qconop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               DoubleColon v1 -> get >>= state_62__ (k_20_1__ v1);
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               _ -> k_214_1__ t__ }
  
  state_59__ k_20_1__ k_214_1__ k_216_1__ k_218_1__ k_242_1__ t__
      = let {
              goto_qvarop'1__ v1 = reduce_261__ goto_qop'1__ v1;
              goto_qconop'1__ v1 = reduce_262__ goto_qop'1__ v1;
              goto_qop'1__ v1 = state_405__ (k_216_1__ v1) (k_218_1__ v1);
              goto_qconsym2'2__ v1 v2 = reduce_265__ goto_qconop'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_278__ goto_qconsym2'2__ v1 v2;
              goto_qvarsym'2__ v1 v2 = reduce_263__ goto_qvarop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_281__ goto_qvarsym'2__ v1 v2;
              goto_qvarsym1'2__ v1 v2 = reduce_282__ goto_qvarsym'2__ v1 v2
            }
        in case t__ of {
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               QVarSym v1 v2 -> get >>= reduce_287__ goto_qvarsym1'2__ v1 v2;
               QConSym v1 v2 -> get >>= reduce_279__ goto_qconsym2'2__ v1 v2;
               BackQuote v1 -> get >>= state_514__ (reduce_264__ goto_qvarop'1__ v1) (reduce_266__ goto_qconop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               DoubleColon v1 -> get >>= state_62__ (k_20_1__ v1);
               LeftArrow v1 -> get >>= state_468__ (k_242_1__ v1);
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               _ -> k_214_1__ t__ }
  
  state_60__ k_20_1__ k_214_1__ k_216_1__ k_218_1__ k_405_2__ t__
      = let {
              goto_qvarop'1__ v1 = reduce_261__ goto_qop'1__ v1;
              goto_qconop'1__ v1 = reduce_262__ goto_qop'1__ v1;
              goto_qop'1__ v1 = state_406__ (k_216_1__ v1) (k_218_1__ v1) (k_405_2__ v1);
              goto_qconsym2'2__ v1 v2 = reduce_265__ goto_qconop'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_278__ goto_qconsym2'2__ v1 v2;
              goto_qvarsym'2__ v1 v2 = reduce_263__ goto_qvarop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_281__ goto_qvarsym'2__ v1 v2;
              goto_qvarsym1'2__ v1 v2 = reduce_282__ goto_qvarsym'2__ v1 v2
            }
        in case t__ of {
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               QVarSym v1 v2 -> get >>= reduce_287__ goto_qvarsym1'2__ v1 v2;
               QConSym v1 v2 -> get >>= reduce_279__ goto_qconsym2'2__ v1 v2;
               BackQuote v1 -> get >>= state_514__ (reduce_264__ goto_qvarop'1__ v1) (reduce_266__ goto_qconop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               DoubleColon v1 -> get >>= state_62__ (k_20_1__ v1);
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               _ -> k_214_1__ t__ }
  
  state_61__ k_20_1__ k_214_1__ k_216_1__ k_218_1__ k_423_1__ t__
      = let {
              goto_qvarop'1__ v1 = reduce_261__ goto_qop'1__ v1;
              goto_qconop'1__ v1 = reduce_262__ goto_qop'1__ v1;
              goto_qop'1__ v1 = state_405__ (k_216_1__ v1) (k_218_1__ v1);
              goto_qconsym2'2__ v1 v2 = reduce_265__ goto_qconop'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_278__ goto_qconsym2'2__ v1 v2;
              goto_qvarsym'2__ v1 v2 = reduce_263__ goto_qvarop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_281__ goto_qvarsym'2__ v1 v2;
              goto_qvarsym1'2__ v1 v2 = reduce_282__ goto_qvarsym'2__ v1 v2
            }
        in case t__ of {
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               QVarSym v1 v2 -> get >>= reduce_287__ goto_qvarsym1'2__ v1 v2;
               QConSym v1 v2 -> get >>= reduce_279__ goto_qconsym2'2__ v1 v2;
               BackQuote v1 -> get >>= state_514__ (reduce_264__ goto_qvarop'1__ v1) (reduce_266__ goto_qconop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               DoubleColon v1 -> get >>= state_62__ (k_20_1__ v1);
               LeftArrow v1 -> get >>= state_743__ (k_423_1__ v1);
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               _ -> k_214_1__ t__ }
  
  state_62__ k_20_2__ t__
      = let {
              goto_xQualType'1__ v1 = k_20_2__ v1;
              goto_xType'1__ v1 = reduce_19__ goto_xQualType'1__ v1;
              goto_btype'1__ v1
                  = state_51__ (reduce_17__ goto_xQualType'1__ v1) (reduce_124__ goto_xType'1__ v1)
                                (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1;
              goto_foralls'2__ v1 v2 = state_54__ (reduce_18__ goto_xQualType'1__ v1 v2)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Forall v1 -> get >>= state_617__ (reduce_333__ goto_foralls'2__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "(",
                               "[",
                               "as",
                               "forall",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_71__ k_28_1__ t__
      = let {
              goto_idecls'1__ v1 = state_72__ (k_28_1__ v1);
              goto_semis'1__ v1 = reduce_337__ goto_opt'1_semis'1__ v1;
              goto_opt'1_semis'1__ v1
                  = state_77__ (reduce_30__ goto_idecls'1__ v1) (reduce_31__ goto_idecls'1__ v1)
                        (reduce_58__ goto_semis'1__ v1)
            }
        in reduce_336__ goto_opt'1_semis'1__ t__
  
  state_72__ k_28_2__ t__ = case t__ of { RightCurly v1 -> get >>= k_28_2__ v1; _ -> frown ["}"] t__ }
  
  state_74__ k_29_1__ t__
      = let {
              goto_idecls'1__ v1 = state_75__ (k_29_1__ v1);
              goto_semis'1__ v1 = reduce_337__ goto_opt'1_semis'1__ v1;
              goto_opt'1_semis'1__ v1
                  = state_77__ (reduce_30__ goto_idecls'1__ v1) (reduce_31__ goto_idecls'1__ v1)
                        (reduce_58__ goto_semis'1__ v1)
            }
        in reduce_336__ goto_opt'1_semis'1__ t__
  
  state_75__ k_29_2__ t__
      = let {
              goto_close'1__ v1 = k_29_2__ v1
            }
        in case t__ of { VRightCurly -> get >>= reduce_288__ goto_close'1__; _ -> reduce_289__ goto_close'1__ t__ }
  
  state_77__ k_30_1__ k_31_1__ k_58_1__ t__
      = let {
              goto_mSepBy1''2_idecl'2_semis'1__ v1 v2
                  = state_78__ (k_30_1__ v1 v2) (reduce_33__ goto_mSepBy1''2_idecl'2_semis'1__ v1 v2);
              goto_idecl'2__ v1 v2 = reduce_32__ goto_mSepBy1''2_idecl'2_semis'1__ v1 v2;
              goto_fixdecl'1__ v1 = reduce_108__ goto_decl'1__ v1;
              goto_infix'2__ v1 v2 = state_233__ (reduce_110__ goto_fixdecl'1__ v1 v2);
              goto_realtopdecl'1__ v1 = reduce_92__ goto_topdecl'1__ v1;
              goto_topdecl'1__ v1 = reduce_41__ goto_idecl'2__ v1;
              goto_decl'1__ v1 = reduce_93__ goto_topdecl'1__ v1;
              goto_signdecl'1__ v1 = reduce_107__ goto_decl'1__ v1;
              goto_vars'1__ v1 = state_250__ (reduce_121__ goto_vars2'1__ v1) (reduce_122__ goto_vars'1__ v1);
              goto_vars2'1__ v1 = state_244__ (reduce_119__ goto_signdecl'1__ v1);
              goto_valdef'1__ v1 = reduce_109__ goto_decl'1__ v1;
              goto_exp0b'1__ v1
                  = state_247__ (reduce_120__ goto_signdecl'1__ v1) (reduce_195__ goto_valdef'1__ v1)
                            (reduce_196__ goto_valdef'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_257__ (reduce_123__ goto_vars'1__ v1) (reduce_197__ goto_valdef'1__ v1)
                                    (reduce_198__ goto_valdef'1__ v1)
                                (reduce_390__ goto_aexp'1__ v1)
                            (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               SemiColon v1 -> get >>= k_58_1__ v1;
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Class v1 -> get >>= state_199__ (reduce_97__ goto_realtopdecl'1__ v1);
               KW_Data v1 -> get >>= state_189__ (reduce_95__ goto_realtopdecl'1__ v1);
               KW_Default v1 -> get >>= state_205__ (reduce_99__ goto_realtopdecl'1__ v1);
               KW_Dependency v1
                   -> get >>= state_219__ (reduce_102__ goto_realtopdecl'1__ v1) (reduce_103__ goto_realtopdecl'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Infix v1 -> get >>= reduce_116__ goto_infix'2__ v1;
               KW_InfixL v1 -> get >>= reduce_117__ goto_infix'2__ v1;
               KW_InfixR v1 -> get >>= reduce_118__ goto_infix'2__ v1;
               KW_Instance v1 -> get >>= state_202__ (reduce_98__ goto_realtopdecl'1__ v1);
               KW_NewType v1 -> get >>= state_194__ (reduce_96__ goto_realtopdecl'1__ v1);
               KW_Type v1
                   -> get
                          >>=
                          state_185__ (reduce_94__ goto_realtopdecl'1__ v1) (reduce_100__ goto_realtopdecl'1__ v1)
                              (reduce_101__ goto_realtopdecl'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               KW_Abstract v1 -> get >>= state_83__ (reduce_34__ goto_idecl'2__ v1) (reduce_35__ goto_idecl'2__ v1);
               KW_Binding v1 -> get >>= state_95__ (reduce_37__ goto_idecl'2__ v1);
               KW_Consig v1 -> get >>= state_89__ (reduce_36__ goto_idecl'2__ v1);
               KW_InterfaceImport v1 -> get >>= state_101__ (reduce_39__ goto_idecl'2__ v1);
               KW_SetModule v1 -> get >>= state_97__ (reduce_38__ goto_idecl'2__ v1);
               KW_GADependency v1 -> get >>= state_103__ (reduce_40__ goto_idecl'2__ v1);
               _ -> k_31_1__ t__ }
  
  state_78__ k_30_2__ k_33_1__ t__
      = let {
              goto_semis'1__ v1 = state_81__ (k_33_1__ v1) (reduce_337__ goto_opt'1_semis'1__ v1);
              goto_opt'1_semis'1__ v1 = state_79__ (k_30_2__ v1) (reduce_58__ goto_semis'1__ v1)
            }
        in reduce_336__ goto_opt'1_semis'1__ t__
  
  state_79__ k_30_3__ k_58_1__ t__ = case t__ of { SemiColon v1 -> get >>= k_58_1__ v1; _ -> k_30_3__ t__ }
  
  state_81__ k_33_2__ k_337_1__ t__
      = let {
              goto_idecl'2__ v1 v2 = k_33_2__ v1 v2;
              goto_fixdecl'1__ v1 = reduce_108__ goto_decl'1__ v1;
              goto_infix'2__ v1 v2 = state_233__ (reduce_110__ goto_fixdecl'1__ v1 v2);
              goto_realtopdecl'1__ v1 = reduce_92__ goto_topdecl'1__ v1;
              goto_topdecl'1__ v1 = reduce_41__ goto_idecl'2__ v1;
              goto_decl'1__ v1 = reduce_93__ goto_topdecl'1__ v1;
              goto_signdecl'1__ v1 = reduce_107__ goto_decl'1__ v1;
              goto_vars'1__ v1 = state_250__ (reduce_121__ goto_vars2'1__ v1) (reduce_122__ goto_vars'1__ v1);
              goto_vars2'1__ v1 = state_244__ (reduce_119__ goto_signdecl'1__ v1);
              goto_valdef'1__ v1 = reduce_109__ goto_decl'1__ v1;
              goto_exp0b'1__ v1
                  = state_247__ (reduce_120__ goto_signdecl'1__ v1) (reduce_195__ goto_valdef'1__ v1)
                            (reduce_196__ goto_valdef'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_257__ (reduce_123__ goto_vars'1__ v1) (reduce_197__ goto_valdef'1__ v1)
                                    (reduce_198__ goto_valdef'1__ v1)
                                (reduce_390__ goto_aexp'1__ v1)
                            (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Class v1 -> get >>= state_199__ (reduce_97__ goto_realtopdecl'1__ v1);
               KW_Data v1 -> get >>= state_189__ (reduce_95__ goto_realtopdecl'1__ v1);
               KW_Default v1 -> get >>= state_205__ (reduce_99__ goto_realtopdecl'1__ v1);
               KW_Dependency v1
                   -> get >>= state_219__ (reduce_102__ goto_realtopdecl'1__ v1) (reduce_103__ goto_realtopdecl'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Infix v1 -> get >>= reduce_116__ goto_infix'2__ v1;
               KW_InfixL v1 -> get >>= reduce_117__ goto_infix'2__ v1;
               KW_InfixR v1 -> get >>= reduce_118__ goto_infix'2__ v1;
               KW_Instance v1 -> get >>= state_202__ (reduce_98__ goto_realtopdecl'1__ v1);
               KW_NewType v1 -> get >>= state_194__ (reduce_96__ goto_realtopdecl'1__ v1);
               KW_Type v1
                   -> get
                          >>=
                          state_185__ (reduce_94__ goto_realtopdecl'1__ v1) (reduce_100__ goto_realtopdecl'1__ v1)
                              (reduce_101__ goto_realtopdecl'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               KW_Abstract v1 -> get >>= state_83__ (reduce_34__ goto_idecl'2__ v1) (reduce_35__ goto_idecl'2__ v1);
               KW_Binding v1 -> get >>= state_95__ (reduce_37__ goto_idecl'2__ v1);
               KW_Consig v1 -> get >>= state_89__ (reduce_36__ goto_idecl'2__ v1);
               KW_InterfaceImport v1 -> get >>= state_101__ (reduce_39__ goto_idecl'2__ v1);
               KW_SetModule v1 -> get >>= state_97__ (reduce_38__ goto_idecl'2__ v1);
               KW_GADependency v1 -> get >>= state_103__ (reduce_40__ goto_idecl'2__ v1);
               _ -> k_337_1__ t__ }
  
  state_83__ k_34_1__ k_35_1__ t__
      = let {
              goto_qcname'1__ v1 = state_84__ (k_34_1__ v1) (k_35_1__ v1);
              goto_qvar'1__ v1 = reduce_59__ goto_qcname'1__ v1;
              goto_qcon'1__ v1 = reduce_60__ goto_qcname'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1 -> get >>= state_484__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_84__ k_34_2__ k_35_2__ t__
      = case t__ of { DoubleColon v1 -> get >>= state_85__ (k_34_2__ v1) (k_35_2__ v1); _ -> frown ["::"] t__ }
  
  state_85__ k_34_3__ k_35_3__ t__
      = let {
              goto_varop'1__ v1 = reduce_257__ goto_op'1__ v1;
              goto_conop'1__ v1 = reduce_258__ goto_op'1__ v1;
              goto_op'1__ v1 = reduce_323__ goto_kindterm'1__ v1;
              goto_varid'2__ v1 v2 = reduce_326__ goto_kivar'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_255__ goto_conop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_259__ goto_varop'1__ v1 v2;
              goto_kind'1__ v1 = state_86__ (k_34_3__ v1) (k_35_3__ v1);
              goto_kindterm'1__ v1 = state_600__ (reduce_321__ goto_kind'1__ v1) (reduce_322__ goto_kind'1__ v1);
              goto_kivar'1__ v1 = reduce_324__ goto_kindterm'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               LeftParen v1 -> get >>= state_605__ (reduce_325__ goto_kindterm'1__ v1);
               BackQuote v1 -> get >>= state_503__ (reduce_256__ goto_conop'1__ v1) (reduce_260__ goto_varop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<varsym>", "<consym>", "(", "`", ".", "-", "!", "as", "hiding", "qualified"] t__ }
  
  state_86__ k_34_4__ k_35_4__ t__ = case t__ of { Equals v1 -> get >>= state_87__ (k_34_4__ v1); _ -> k_35_4__ t__ }
  
  state_87__ k_34_5__ t__
      = let {
              goto_mSepBy'2_iconwithtype'2_33__ v1 v2 = k_34_5__ v1 v2;
              goto_mSepBy1'2_iconwithtype'2_33__ v1 v2 = reduce_45__ goto_mSepBy'2_iconwithtype'2_33__ v1 v2;
              goto_mSepBy1''2_iconwithtype'2_33__ v1 v2
                  = state_114__ (reduce_47__ goto_mSepBy1'2_iconwithtype'2_33__ v1 v2)
                        (reduce_51__ goto_mSepBy1''2_iconwithtype'2_33__ v1 v2);
              goto_iconwithtype'2__ v1 v2 = reduce_50__ goto_mSepBy1''2_iconwithtype'2_33__ v1 v2;
              goto_qcname'1__ v1 = state_121__ (reduce_52__ goto_iconwithtype'2__ v1);
              goto_qvar'1__ v1 = reduce_59__ goto_qcname'1__ v1;
              goto_qcon'1__ v1 = reduce_60__ goto_qcname'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1 -> get >>= state_484__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> reduce_44__ goto_mSepBy'2_iconwithtype'2_33__ t__ }
  
  state_89__ k_36_1__ t__
      = let {
              goto_qcname'1__ v1 = state_90__ (k_36_1__ v1);
              goto_qvar'1__ v1 = reduce_59__ goto_qcname'1__ v1;
              goto_qcon'1__ v1 = reduce_60__ goto_qcname'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1 -> get >>= state_484__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_90__ k_36_2__ t__ = case t__ of { Equals v1 -> get >>= state_91__ (k_36_2__ v1); _ -> frown ["="] t__ }
  
  state_91__ k_36_3__ t__ = case t__ of { LeftSquare v1 -> get >>= state_92__ (k_36_3__ v1); _ -> frown ["["] t__ }
  
  state_92__ k_36_4__ t__
      = let {
              goto_mSepBy'1_typeargi'1_21__ v1 = state_93__ (k_36_4__ v1);
              goto_mSepBy1'1_typeargi'1_21__ v1 = reduce_43__ goto_mSepBy'1_typeargi'1_21__ v1;
              goto_mSepBy1''1_typeargi'1_21__ v1
                  = state_113__ (reduce_46__ goto_mSepBy1'1_typeargi'1_21__ v1)
                        (reduce_49__ goto_mSepBy1''1_typeargi'1_21__ v1);
              goto_xType'1__ v1 = reduce_327__ goto_typeorconsym'1__ v1;
              goto_btype'1__ v1
                  = state_258__ (reduce_124__ goto_xType'1__ v1) (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_qconsym'1__ v1 = reduce_328__ goto_typeorconsym'1__ v1;
              goto_qconsym2'2__ v1 v2 = reduce_277__ goto_qconsym'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_278__ goto_qconsym2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1;
              goto_typeorconsym'1__ v1 = reduce_332__ goto_typeargi'1__ v1;
              goto_typeargi'1__ v1 = reduce_48__ goto_mSepBy1''1_typeargi'1_21__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               QConSym v1 v2 -> get >>= reduce_279__ goto_qconsym2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               RightSquare v1 -> reduce_42__ goto_mSepBy'1_typeargi'1_21__ t__;
               DoubleColon v1 -> get >>= state_613__ (reduce_330__ goto_typeorconsym'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_611__ (reduce_329__ goto_typeorconsym'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<consym>",
                               "<qconsym>",
                               "(",
                               "[",
                               "]",
                               "::",
                               "as",
                               "case",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_93__ k_36_5__ t__ = case t__ of { RightSquare v1 -> get >>= k_36_5__ v1; _ -> frown ["]"] t__ }
  
  state_95__ k_37_1__ t__
      = let {
              goto_qcname'1__ v1 = k_37_1__ v1;
              goto_qvar'1__ v1 = reduce_59__ goto_qcname'1__ v1;
              goto_qcon'1__ v1 = reduce_60__ goto_qcname'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1 -> get >>= state_484__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_97__ k_38_1__ t__
      = let {
              goto_modid'1__ v1 = state_98__ (k_38_1__ v1)
            }
        in case t__ of {
               ConId v1 v2 -> get >>= reduce_292__ goto_modid'1__ v1 v2;
               QConId v1 v2 -> get >>= reduce_293__ goto_modid'1__ v1 v2;
               _ -> frown ["<conid>", "<qconid>"] t__ }
  
  state_98__ k_38_2__ t__ = case t__ of { KW_Where v1 -> get >>= state_99__ (k_38_2__ v1); _ -> frown ["where"] t__ }
  
  state_99__ k_38_3__ t__
      = let {
              goto_ibody'2__ v1 v2 = k_38_3__ v1 v2;
              goto_layout_on'1__ v1 = state_74__ (reduce_29__ goto_ibody'2__ v1)
            }
        in case t__ of {
               LeftCurly v1 -> get >>= state_71__ (reduce_28__ goto_ibody'2__ v1);
               _ -> reduce_290__ goto_layout_on'1__ t__ }
  
  state_101__ k_39_1__ t__
      = let {
              goto_modid'1__ v1 = k_39_1__ v1
            }
        in case t__ of {
               ConId v1 v2 -> get >>= reduce_292__ goto_modid'1__ v1 v2;
               QConId v1 v2 -> get >>= reduce_293__ goto_modid'1__ v1 v2;
               _ -> frown ["<conid>", "<qconid>"] t__ }
  
  state_103__ k_40_1__ t__
      = let {
              goto_qcname'1__ v1 = state_104__ (k_40_1__ v1);
              goto_qvar'1__ v1 = reduce_59__ goto_qcname'1__ v1;
              goto_qcon'1__ v1 = reduce_60__ goto_qcname'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1 -> get >>= state_484__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_104__ k_40_2__ t__ = case t__ of { LeftKiParen v1 -> get >>= state_105__ (k_40_2__ v1); _ -> frown ["{["] t__ }
  
  state_105__ k_40_3__ t__
      = let {
              goto_varop'1__ v1 = reduce_257__ goto_op'1__ v1;
              goto_conop'1__ v1 = reduce_258__ goto_op'1__ v1;
              goto_op'1__ v1 = reduce_323__ goto_kindterm'1__ v1;
              goto_varid'2__ v1 v2 = reduce_326__ goto_kivar'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_255__ goto_conop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_259__ goto_varop'1__ v1 v2;
              goto_kind'1__ v1 = state_106__ (k_40_3__ v1);
              goto_kindterm'1__ v1 = state_600__ (reduce_321__ goto_kind'1__ v1) (reduce_322__ goto_kind'1__ v1);
              goto_kivar'1__ v1 = reduce_324__ goto_kindterm'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               LeftParen v1 -> get >>= state_605__ (reduce_325__ goto_kindterm'1__ v1);
               BackQuote v1 -> get >>= state_503__ (reduce_256__ goto_conop'1__ v1) (reduce_260__ goto_varop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<varsym>", "<consym>", "(", "`", ".", "-", "!", "as", "hiding", "qualified"] t__ }
  
  state_106__ k_40_4__ t__ = case t__ of { RightKiParen v1 -> get >>= state_107__ (k_40_4__ v1); _ -> frown ["]}"] t__ }
  
  state_107__ k_40_5__ t__ = case t__ of { LeftArrow v1 -> get >>= state_108__ (k_40_5__ v1); _ -> frown ["<-"] t__ }
  
  state_108__ k_40_6__ t__
      = let {
              goto_many'1_cdepbinding'1__ v1 = k_40_6__ v1;
              goto_many''1_cdepbinding'1__ v1
                  = state_653__ (reduce_360__ goto_many'1_cdepbinding'1__ v1)
                        (reduce_371__ goto_many''1_cdepbinding'1__ v1)
            }
        in reduce_370__ goto_many''1_cdepbinding'1__ t__
  
  state_113__ k_46_1__ k_49_1__ t__
      = case t__ of {
            RightSquare v1 -> k_46_1__ t__;
            Comma v1 -> get >>= state_116__ (k_49_1__ v1);
            _ -> frown ["]", ","] t__ }
  
  state_114__ k_47_1__ k_51_1__ t__ = case t__ of { Bar v1 -> get >>= state_119__ (k_51_1__ v1); _ -> k_47_1__ t__ }
  
  state_116__ k_49_2__ t__
      = let {
              goto_xType'1__ v1 = reduce_327__ goto_typeorconsym'1__ v1;
              goto_btype'1__ v1
                  = state_258__ (reduce_124__ goto_xType'1__ v1) (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_qconsym'1__ v1 = reduce_328__ goto_typeorconsym'1__ v1;
              goto_qconsym2'2__ v1 v2 = reduce_277__ goto_qconsym'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_278__ goto_qconsym2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1;
              goto_typeorconsym'1__ v1 = reduce_332__ goto_typeargi'1__ v1;
              goto_typeargi'1__ v1 = k_49_2__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               QConSym v1 v2 -> get >>= reduce_279__ goto_qconsym2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               DoubleColon v1 -> get >>= state_613__ (reduce_330__ goto_typeorconsym'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_611__ (reduce_329__ goto_typeorconsym'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<consym>",
                               "<qconsym>",
                               "(",
                               "[",
                               "::",
                               "as",
                               "case",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_119__ k_51_2__ t__
      = let {
              goto_iconwithtype'2__ v1 v2 = k_51_2__ v1 v2;
              goto_qcname'1__ v1 = state_121__ (reduce_52__ goto_iconwithtype'2__ v1);
              goto_qvar'1__ v1 = reduce_59__ goto_qcname'1__ v1;
              goto_qcon'1__ v1 = reduce_60__ goto_qcname'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1 -> get >>= state_484__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_121__ k_52_1__ t__ = case t__ of { DoubleColon v1 -> get >>= state_122__ (k_52_1__ v1); _ -> frown ["::"] t__ }
  
  state_122__ k_52_2__ t__
      = let {
              goto_xQualType'1__ v1 = k_52_2__ v1;
              goto_xType'1__ v1 = reduce_19__ goto_xQualType'1__ v1;
              goto_btype'1__ v1
                  = state_51__ (reduce_17__ goto_xQualType'1__ v1) (reduce_124__ goto_xType'1__ v1)
                                (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1;
              goto_foralls'2__ v1 v2 = state_54__ (reduce_18__ goto_xQualType'1__ v1 v2)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Forall v1 -> get >>= state_617__ (reduce_333__ goto_foralls'2__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "(",
                               "[",
                               "as",
                               "forall",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_124__ k_53_1__ t__
      = let {
              goto_bodyaux'1__ v1 = state_125__ (k_53_1__ v1);
              goto_impdecls'1__ v1
                  = state_130__ (reduce_55__ goto_bodyaux'1__ v1) (reduce_57__ goto_bodyaux'1__ v1)
                            (reduce_61__ goto_impdecls'1__ v1)
                        (reduce_62__ goto_impdecls'1__ v1);
              goto_impdecl'1__ v1 = reduce_63__ goto_impdecls'1__ v1;
              goto_fixdecl'1__ v1 = reduce_108__ goto_decl'1__ v1;
              goto_infix'2__ v1 v2 = state_233__ (reduce_110__ goto_fixdecl'1__ v1 v2);
              goto_topdecls'1__ v1 = reduce_56__ goto_bodyaux'1__ v1;
              goto_topdecls''1__ v1
                  = state_179__ (reduce_88__ goto_topdecls'1__ v1) (reduce_89__ goto_topdecls''1__ v1)
                        (reduce_90__ goto_topdecls''1__ v1);
              goto_realtopdecl'1__ v1 = reduce_92__ goto_topdecl'1__ v1;
              goto_topdecl'1__ v1 = reduce_91__ goto_topdecls''1__ v1;
              goto_decl'1__ v1 = reduce_93__ goto_topdecl'1__ v1;
              goto_signdecl'1__ v1 = reduce_107__ goto_decl'1__ v1;
              goto_vars'1__ v1 = state_250__ (reduce_121__ goto_vars2'1__ v1) (reduce_122__ goto_vars'1__ v1);
              goto_vars2'1__ v1 = state_244__ (reduce_119__ goto_signdecl'1__ v1);
              goto_valdef'1__ v1 = reduce_109__ goto_decl'1__ v1;
              goto_exp0b'1__ v1
                  = state_247__ (reduce_120__ goto_signdecl'1__ v1) (reduce_195__ goto_valdef'1__ v1)
                            (reduce_196__ goto_valdef'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_257__ (reduce_123__ goto_vars'1__ v1) (reduce_197__ goto_valdef'1__ v1)
                                    (reduce_198__ goto_valdef'1__ v1)
                                (reduce_390__ goto_aexp'1__ v1)
                            (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Class v1 -> get >>= state_199__ (reduce_97__ goto_realtopdecl'1__ v1);
               KW_Data v1 -> get >>= state_189__ (reduce_95__ goto_realtopdecl'1__ v1);
               KW_Default v1 -> get >>= state_205__ (reduce_99__ goto_realtopdecl'1__ v1);
               KW_Dependency v1
                   -> get >>= state_219__ (reduce_102__ goto_realtopdecl'1__ v1) (reduce_103__ goto_realtopdecl'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Import v1 -> get >>= state_147__ (reduce_65__ goto_impdecl'1__ v1);
               KW_Infix v1 -> get >>= reduce_116__ goto_infix'2__ v1;
               KW_InfixL v1 -> get >>= reduce_117__ goto_infix'2__ v1;
               KW_InfixR v1 -> get >>= reduce_118__ goto_infix'2__ v1;
               KW_Instance v1 -> get >>= state_202__ (reduce_98__ goto_realtopdecl'1__ v1);
               KW_NewType v1 -> get >>= state_194__ (reduce_96__ goto_realtopdecl'1__ v1);
               KW_Type v1
                   -> get
                          >>=
                          state_185__ (reduce_94__ goto_realtopdecl'1__ v1) (reduce_100__ goto_realtopdecl'1__ v1)
                              (reduce_101__ goto_realtopdecl'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> reduce_64__ goto_impdecls'1__ t__ }
  
  state_125__ k_53_2__ t__ = case t__ of { RightCurly v1 -> get >>= k_53_2__ v1; _ -> frown ["}"] t__ }
  
  state_127__ k_54_1__ t__
      = let {
              goto_bodyaux'1__ v1 = state_128__ (k_54_1__ v1);
              goto_impdecls'1__ v1
                  = state_130__ (reduce_55__ goto_bodyaux'1__ v1) (reduce_57__ goto_bodyaux'1__ v1)
                            (reduce_61__ goto_impdecls'1__ v1)
                        (reduce_62__ goto_impdecls'1__ v1);
              goto_impdecl'1__ v1 = reduce_63__ goto_impdecls'1__ v1;
              goto_fixdecl'1__ v1 = reduce_108__ goto_decl'1__ v1;
              goto_infix'2__ v1 v2 = state_233__ (reduce_110__ goto_fixdecl'1__ v1 v2);
              goto_topdecls'1__ v1 = reduce_56__ goto_bodyaux'1__ v1;
              goto_topdecls''1__ v1
                  = state_179__ (reduce_88__ goto_topdecls'1__ v1) (reduce_89__ goto_topdecls''1__ v1)
                        (reduce_90__ goto_topdecls''1__ v1);
              goto_realtopdecl'1__ v1 = reduce_92__ goto_topdecl'1__ v1;
              goto_topdecl'1__ v1 = reduce_91__ goto_topdecls''1__ v1;
              goto_decl'1__ v1 = reduce_93__ goto_topdecl'1__ v1;
              goto_signdecl'1__ v1 = reduce_107__ goto_decl'1__ v1;
              goto_vars'1__ v1 = state_250__ (reduce_121__ goto_vars2'1__ v1) (reduce_122__ goto_vars'1__ v1);
              goto_vars2'1__ v1 = state_244__ (reduce_119__ goto_signdecl'1__ v1);
              goto_valdef'1__ v1 = reduce_109__ goto_decl'1__ v1;
              goto_exp0b'1__ v1
                  = state_247__ (reduce_120__ goto_signdecl'1__ v1) (reduce_195__ goto_valdef'1__ v1)
                            (reduce_196__ goto_valdef'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_257__ (reduce_123__ goto_vars'1__ v1) (reduce_197__ goto_valdef'1__ v1)
                                    (reduce_198__ goto_valdef'1__ v1)
                                (reduce_390__ goto_aexp'1__ v1)
                            (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Class v1 -> get >>= state_199__ (reduce_97__ goto_realtopdecl'1__ v1);
               KW_Data v1 -> get >>= state_189__ (reduce_95__ goto_realtopdecl'1__ v1);
               KW_Default v1 -> get >>= state_205__ (reduce_99__ goto_realtopdecl'1__ v1);
               KW_Dependency v1
                   -> get >>= state_219__ (reduce_102__ goto_realtopdecl'1__ v1) (reduce_103__ goto_realtopdecl'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Import v1 -> get >>= state_147__ (reduce_65__ goto_impdecl'1__ v1);
               KW_Infix v1 -> get >>= reduce_116__ goto_infix'2__ v1;
               KW_InfixL v1 -> get >>= reduce_117__ goto_infix'2__ v1;
               KW_InfixR v1 -> get >>= reduce_118__ goto_infix'2__ v1;
               KW_Instance v1 -> get >>= state_202__ (reduce_98__ goto_realtopdecl'1__ v1);
               KW_NewType v1 -> get >>= state_194__ (reduce_96__ goto_realtopdecl'1__ v1);
               KW_Type v1
                   -> get
                          >>=
                          state_185__ (reduce_94__ goto_realtopdecl'1__ v1) (reduce_100__ goto_realtopdecl'1__ v1)
                              (reduce_101__ goto_realtopdecl'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> reduce_64__ goto_impdecls'1__ t__ }
  
  state_128__ k_54_2__ t__
      = let {
              goto_close'1__ v1 = k_54_2__ v1
            }
        in case t__ of { VRightCurly -> get >>= reduce_288__ goto_close'1__; _ -> reduce_289__ goto_close'1__ t__ }
  
  state_130__ k_55_1__ k_57_1__ k_61_1__ k_62_1__ t__
      = case t__ of { SemiColon v1 -> get >>= state_131__ (k_55_1__ v1) (k_61_1__ v1) (k_62_1__ v1); _ -> k_57_1__ t__ }
  
  state_131__ k_55_2__ k_61_2__ k_62_2__ t__
      = let {
              goto_impdecl'1__ v1 = k_61_2__ v1;
              goto_fixdecl'1__ v1 = reduce_108__ goto_decl'1__ v1;
              goto_infix'2__ v1 v2 = state_233__ (reduce_110__ goto_fixdecl'1__ v1 v2);
              goto_topdecls'1__ v1 = k_55_2__ v1;
              goto_topdecls''1__ v1
                  = state_179__ (reduce_88__ goto_topdecls'1__ v1) (reduce_89__ goto_topdecls''1__ v1)
                        (reduce_90__ goto_topdecls''1__ v1);
              goto_realtopdecl'1__ v1 = reduce_92__ goto_topdecl'1__ v1;
              goto_topdecl'1__ v1 = reduce_91__ goto_topdecls''1__ v1;
              goto_decl'1__ v1 = reduce_93__ goto_topdecl'1__ v1;
              goto_signdecl'1__ v1 = reduce_107__ goto_decl'1__ v1;
              goto_vars'1__ v1 = state_250__ (reduce_121__ goto_vars2'1__ v1) (reduce_122__ goto_vars'1__ v1);
              goto_vars2'1__ v1 = state_244__ (reduce_119__ goto_signdecl'1__ v1);
              goto_valdef'1__ v1 = reduce_109__ goto_decl'1__ v1;
              goto_exp0b'1__ v1
                  = state_247__ (reduce_120__ goto_signdecl'1__ v1) (reduce_195__ goto_valdef'1__ v1)
                            (reduce_196__ goto_valdef'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_257__ (reduce_123__ goto_vars'1__ v1) (reduce_197__ goto_valdef'1__ v1)
                                    (reduce_198__ goto_valdef'1__ v1)
                                (reduce_390__ goto_aexp'1__ v1)
                            (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Class v1 -> get >>= state_199__ (reduce_97__ goto_realtopdecl'1__ v1);
               KW_Data v1 -> get >>= state_189__ (reduce_95__ goto_realtopdecl'1__ v1);
               KW_Default v1 -> get >>= state_205__ (reduce_99__ goto_realtopdecl'1__ v1);
               KW_Dependency v1
                   -> get >>= state_219__ (reduce_102__ goto_realtopdecl'1__ v1) (reduce_103__ goto_realtopdecl'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Import v1 -> get >>= state_147__ (reduce_65__ goto_impdecl'1__ v1);
               KW_Infix v1 -> get >>= reduce_116__ goto_infix'2__ v1;
               KW_InfixL v1 -> get >>= reduce_117__ goto_infix'2__ v1;
               KW_InfixR v1 -> get >>= reduce_118__ goto_infix'2__ v1;
               KW_Instance v1 -> get >>= state_202__ (reduce_98__ goto_realtopdecl'1__ v1);
               KW_NewType v1 -> get >>= state_194__ (reduce_96__ goto_realtopdecl'1__ v1);
               KW_Type v1
                   -> get
                          >>=
                          state_185__ (reduce_94__ goto_realtopdecl'1__ v1) (reduce_100__ goto_realtopdecl'1__ v1)
                              (reduce_101__ goto_realtopdecl'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> k_62_2__ t__ }
  
  state_134__ k_58_1__ k_181_1__ k_182_1__ t__
      = let {
              goto_mSepBy1''1_cdecl'1_semis'1__ v1
                  = state_354__ (k_181_1__ v1) (reduce_184__ goto_mSepBy1''1_cdecl'1_semis'1__ v1);
              goto_fixdecl'1__ v1 = reduce_186__ goto_cdecl'1__ v1;
              goto_infix'2__ v1 v2 = state_233__ (reduce_110__ goto_fixdecl'1__ v1 v2);
              goto_signdecl'1__ v1 = reduce_185__ goto_cdecl'1__ v1;
              goto_vars'1__ v1 = state_250__ (reduce_121__ goto_vars2'1__ v1) (reduce_122__ goto_vars'1__ v1);
              goto_vars2'1__ v1 = state_244__ (reduce_119__ goto_signdecl'1__ v1);
              goto_cdecl'1__ v1 = reduce_183__ goto_mSepBy1''1_cdecl'1_semis'1__ v1;
              goto_valdef'1__ v1 = reduce_187__ goto_cdecl'1__ v1;
              goto_exp0b'1__ v1
                  = state_247__ (reduce_120__ goto_signdecl'1__ v1) (reduce_195__ goto_valdef'1__ v1)
                            (reduce_196__ goto_valdef'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_257__ (reduce_123__ goto_vars'1__ v1) (reduce_197__ goto_valdef'1__ v1)
                                    (reduce_198__ goto_valdef'1__ v1)
                                (reduce_390__ goto_aexp'1__ v1)
                            (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               SemiColon v1 -> get >>= k_58_1__ v1;
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Infix v1 -> get >>= reduce_116__ goto_infix'2__ v1;
               KW_InfixL v1 -> get >>= reduce_117__ goto_infix'2__ v1;
               KW_InfixR v1 -> get >>= reduce_118__ goto_infix'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> k_182_1__ t__ }
  
  state_135__ k_58_1__ k_181_3__ t__ = case t__ of { SemiColon v1 -> get >>= k_58_1__ v1; _ -> k_181_3__ t__ }
  
  state_136__ k_58_1__ k_191_1__ k_192_1__ t__
      = let {
              goto_valdefs1'1__ v1 = state_368__ (k_191_1__ v1) (reduce_193__ goto_valdefs1'1__ v1);
              goto_valdef'1__ v1 = reduce_194__ goto_valdefs1'1__ v1;
              goto_exp0b'1__ v1
                  = state_372__ (reduce_195__ goto_valdef'1__ v1) (reduce_196__ goto_valdef'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_376__ (reduce_197__ goto_valdef'1__ v1) (reduce_198__ goto_valdef'1__ v1)
                                (reduce_390__ goto_aexp'1__ v1)
                            (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               SemiColon v1 -> get >>= k_58_1__ v1;
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> k_192_1__ t__ }
  
  state_137__ k_58_1__ k_191_3__ t__ = case t__ of { SemiColon v1 -> get >>= k_58_1__ v1; _ -> k_191_3__ t__ }
  
  state_138__ k_58_1__ k_206_1__ k_207_1__ t__
      = let {
              goto_mSepBy1''1_decl'1_semis'1__ v1
                  = state_392__ (k_206_1__ v1) (reduce_209__ goto_mSepBy1''1_decl'1_semis'1__ v1);
              goto_fixdecl'1__ v1 = reduce_108__ goto_decl'1__ v1;
              goto_infix'2__ v1 v2 = state_233__ (reduce_110__ goto_fixdecl'1__ v1 v2);
              goto_decl'1__ v1 = reduce_208__ goto_mSepBy1''1_decl'1_semis'1__ v1;
              goto_signdecl'1__ v1 = reduce_107__ goto_decl'1__ v1;
              goto_vars'1__ v1 = state_250__ (reduce_121__ goto_vars2'1__ v1) (reduce_122__ goto_vars'1__ v1);
              goto_vars2'1__ v1 = state_244__ (reduce_119__ goto_signdecl'1__ v1);
              goto_valdef'1__ v1 = reduce_109__ goto_decl'1__ v1;
              goto_exp0b'1__ v1
                  = state_247__ (reduce_120__ goto_signdecl'1__ v1) (reduce_195__ goto_valdef'1__ v1)
                            (reduce_196__ goto_valdef'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_257__ (reduce_123__ goto_vars'1__ v1) (reduce_197__ goto_valdef'1__ v1)
                                    (reduce_198__ goto_valdef'1__ v1)
                                (reduce_390__ goto_aexp'1__ v1)
                            (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               SemiColon v1 -> get >>= k_58_1__ v1;
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Infix v1 -> get >>= reduce_116__ goto_infix'2__ v1;
               KW_InfixL v1 -> get >>= reduce_117__ goto_infix'2__ v1;
               KW_InfixR v1 -> get >>= reduce_118__ goto_infix'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> k_207_1__ t__ }
  
  state_139__ k_58_1__ k_206_3__ t__ = case t__ of { SemiColon v1 -> get >>= k_58_1__ v1; _ -> k_206_3__ t__ }
  
  state_140__ k_58_1__ k_231_1__ k_232_1__ t__
      = let {
              goto_mSepBy1''1_alt'1_semis'1__ v1
                  = state_450__ (k_231_1__ v1) (reduce_234__ goto_mSepBy1''1_alt'1_semis'1__ v1);
              goto_exp0b'1__ v1
                  = state_409__ (reduce_218__ goto_exp0b'1__ v1) (reduce_235__ goto_alt'1__ v1)
                        (reduce_236__ goto_alt'1__ v1);
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_alt'1__ v1 = reduce_233__ goto_mSepBy1''1_alt'1_semis'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               SemiColon v1 -> get >>= k_58_1__ v1;
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> k_232_1__ t__ }
  
  state_141__ k_58_1__ k_231_3__ t__ = case t__ of { SemiColon v1 -> get >>= k_58_1__ v1; _ -> k_231_3__ t__ }
  
  state_147__ k_65_1__ t__
      = let {
              goto_mopt'1_67__ v1 = state_148__ (k_65_1__ v1)
            }
        in case t__ of {
               KW_Qualified v1 -> get >>= reduce_67__ goto_mopt'1_67__ v1;
               _ -> reduce_66__ goto_mopt'1_67__ t__ }
  
  state_148__ k_65_2__ t__
      = let {
              goto_modid'1__ v1 = state_149__ (k_65_2__ v1)
            }
        in case t__ of {
               ConId v1 v2 -> get >>= reduce_292__ goto_modid'1__ v1 v2;
               QConId v1 v2 -> get >>= reduce_293__ goto_modid'1__ v1 v2;
               _ -> frown ["<conid>", "<qconid>"] t__ }
  
  state_149__ k_65_3__ t__
      = let {
              goto_as'1__ v1 = reduce_356__ goto_opt'1_as'1__ v1;
              goto_opt'1_as'1__ v1 = state_150__ (k_65_3__ v1)
            }
        in case t__ of {
               KW_As v1 -> get >>= state_649__ (reduce_357__ goto_as'1__ v1);
               _ -> reduce_355__ goto_opt'1_as'1__ t__ }
  
  state_150__ k_65_4__ t__
      = let {
              goto_mopt'1_52__ v1 = state_154__ (reduce_70__ goto_impspec'1__ v1);
              goto_maybeimpspec'1__ v1 = k_65_4__ v1;
              goto_impspec'1__ v1 = reduce_68__ goto_maybeimpspec'1__ v1
            }
        in case t__ of {
               LeftParen v1 -> reduce_76__ goto_mopt'1_52__ t__;
               KW_Hiding v1 -> get >>= reduce_77__ goto_mopt'1_52__ v1;
               _ -> reduce_69__ goto_maybeimpspec'1__ t__ }
  
  state_154__ k_70_1__ t__ = case t__ of { LeftParen v1 -> get >>= state_155__ (k_70_1__ v1); _ -> frown ["("] t__ }
  
  state_155__ k_70_2__ t__
      = let {
              goto_mSepBy'1_import'1_21__ v1 = state_156__ (k_70_2__ v1);
              goto_mSepBy1'1_import'1_21__ v1 = reduce_72__ goto_mSepBy'1_import'1_21__ v1;
              goto_mSepBy1''1_import'1_21__ v1
                  = state_160__ (reduce_73__ goto_mSepBy1'1_import'1_21__ v1)
                        (reduce_75__ goto_mSepBy1''1_import'1_21__ v1);
              goto_import'1__ v1 = reduce_74__ goto_mSepBy1''1_import'1_21__ v1;
              goto_var'1__ v1 = reduce_78__ goto_import'1__ v1;
              goto_varid'2__ v1 v2 = reduce_247__ goto_var'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_294__ goto_tyconorcls'2__ v1 v2;
              goto_tyconorcls'2__ v1 v2
                  = state_166__ (reduce_79__ goto_import'1__ v1 v2) (reduce_80__ goto_import'1__ v1 v2)
                            (reduce_81__ goto_import'1__ v1 v2)
                        (reduce_82__ goto_import'1__ v1 v2)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               LeftParen v1 -> get >>= state_478__ (reduce_248__ goto_var'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> reduce_71__ goto_mSepBy'1_import'1_21__ t__ }
  
  state_156__ k_70_3__ t__
      = let {
              goto_opt'1_21__ v1 = state_157__ (k_70_3__ v1)
            }
        in case t__ of {
               RightParen v1 -> reduce_334__ goto_opt'1_21__ t__;
               Comma v1 -> get >>= reduce_335__ goto_opt'1_21__ v1;
               _ -> frown [")", ","] t__ }
  
  state_157__ k_70_4__ t__ = case t__ of { RightParen v1 -> get >>= k_70_4__ v1; _ -> frown [")"] t__ }
  
  state_160__ k_73_1__ k_75_1__ t__
      = case t__ of {
            RightParen v1 -> k_73_1__ t__;
            Comma v1 -> get >>= state_162__ (k_75_1__ v1);
            _ -> frown [")", ","] t__ }
  
  state_162__ k_75_2__ t__
      = let {
              goto_import'1__ v1 = k_75_2__ v1;
              goto_var'1__ v1 = reduce_78__ goto_import'1__ v1;
              goto_varid'2__ v1 v2 = reduce_247__ goto_var'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_294__ goto_tyconorcls'2__ v1 v2;
              goto_tyconorcls'2__ v1 v2
                  = state_166__ (reduce_79__ goto_import'1__ v1 v2) (reduce_80__ goto_import'1__ v1 v2)
                            (reduce_81__ goto_import'1__ v1 v2)
                        (reduce_82__ goto_import'1__ v1 v2)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               LeftParen v1 -> get >>= state_478__ (reduce_248__ goto_var'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<conid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_166__ k_79_1__ k_80_1__ k_81_1__ k_82_1__ t__
      = case t__ of { LeftParen v1 -> get >>= state_167__ (k_80_1__ v1) (k_81_1__ v1) (k_82_1__ v1); _ -> k_79_1__ t__ }
  
  state_167__ k_80_2__ k_81_2__ k_82_2__ t__
      = let {
              goto_mSepBy1'1_cname'1_21__ v1 = state_171__ (k_82_2__ v1);
              goto_mSepBy1''1_cname'1_21__ v1
                  = state_173__ (reduce_83__ goto_mSepBy1'1_cname'1_21__ v1)
                        (reduce_85__ goto_mSepBy1''1_cname'1_21__ v1);
              goto_cname'1__ v1 = reduce_84__ goto_mSepBy1''1_cname'1_21__ v1;
              goto_var'1__ v1 = reduce_86__ goto_cname'1__ v1;
              goto_con'1__ v1 = reduce_87__ goto_cname'1__ v1;
              goto_varid'2__ v1 v2 = reduce_247__ goto_var'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_251__ goto_con'1__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               LeftParen v1 -> get >>= state_479__ (reduce_248__ goto_var'1__ v1) (reduce_252__ goto_con'1__ v1);
               RightParen v1 -> get >>= k_81_2__ v1;
               DotDot v1 -> get >>= state_168__ (k_80_2__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<conid>", "(", ")", "..", "as", "hiding", "qualified"] t__ }
  
  state_168__ k_80_3__ t__ = case t__ of { RightParen v1 -> get >>= k_80_3__ v1; _ -> frown [")"] t__ }
  
  state_171__ k_82_3__ t__ = case t__ of { RightParen v1 -> get >>= k_82_3__ v1; _ -> frown [")"] t__ }
  
  state_173__ k_83_1__ k_85_1__ t__
      = case t__ of {
            RightParen v1 -> k_83_1__ t__;
            Comma v1 -> get >>= state_175__ (k_85_1__ v1);
            _ -> frown [")", ","] t__ }
  
  state_175__ k_85_2__ t__
      = let {
              goto_cname'1__ v1 = k_85_2__ v1;
              goto_var'1__ v1 = reduce_86__ goto_cname'1__ v1;
              goto_con'1__ v1 = reduce_87__ goto_cname'1__ v1;
              goto_varid'2__ v1 v2 = reduce_247__ goto_var'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_251__ goto_con'1__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               LeftParen v1 -> get >>= state_479__ (reduce_248__ goto_var'1__ v1) (reduce_252__ goto_con'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<conid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_179__ k_88_1__ k_89_1__ k_90_1__ t__
      = case t__ of { SemiColon v1 -> get >>= state_180__ (k_89_1__ v1) (k_90_1__ v1); _ -> k_88_1__ t__ }
  
  state_180__ k_89_2__ k_90_2__ t__
      = let {
              goto_fixdecl'1__ v1 = reduce_108__ goto_decl'1__ v1;
              goto_infix'2__ v1 v2 = state_233__ (reduce_110__ goto_fixdecl'1__ v1 v2);
              goto_realtopdecl'1__ v1 = reduce_92__ goto_topdecl'1__ v1;
              goto_topdecl'1__ v1 = k_89_2__ v1;
              goto_decl'1__ v1 = reduce_93__ goto_topdecl'1__ v1;
              goto_signdecl'1__ v1 = reduce_107__ goto_decl'1__ v1;
              goto_vars'1__ v1 = state_250__ (reduce_121__ goto_vars2'1__ v1) (reduce_122__ goto_vars'1__ v1);
              goto_vars2'1__ v1 = state_244__ (reduce_119__ goto_signdecl'1__ v1);
              goto_valdef'1__ v1 = reduce_109__ goto_decl'1__ v1;
              goto_exp0b'1__ v1
                  = state_247__ (reduce_120__ goto_signdecl'1__ v1) (reduce_195__ goto_valdef'1__ v1)
                            (reduce_196__ goto_valdef'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_257__ (reduce_123__ goto_vars'1__ v1) (reduce_197__ goto_valdef'1__ v1)
                                    (reduce_198__ goto_valdef'1__ v1)
                                (reduce_390__ goto_aexp'1__ v1)
                            (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Class v1 -> get >>= state_199__ (reduce_97__ goto_realtopdecl'1__ v1);
               KW_Data v1 -> get >>= state_189__ (reduce_95__ goto_realtopdecl'1__ v1);
               KW_Default v1 -> get >>= state_205__ (reduce_99__ goto_realtopdecl'1__ v1);
               KW_Dependency v1
                   -> get >>= state_219__ (reduce_102__ goto_realtopdecl'1__ v1) (reduce_103__ goto_realtopdecl'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Infix v1 -> get >>= reduce_116__ goto_infix'2__ v1;
               KW_InfixL v1 -> get >>= reduce_117__ goto_infix'2__ v1;
               KW_InfixR v1 -> get >>= reduce_118__ goto_infix'2__ v1;
               KW_Instance v1 -> get >>= state_202__ (reduce_98__ goto_realtopdecl'1__ v1);
               KW_NewType v1 -> get >>= state_194__ (reduce_96__ goto_realtopdecl'1__ v1);
               KW_Type v1
                   -> get
                          >>=
                          state_185__ (reduce_94__ goto_realtopdecl'1__ v1) (reduce_100__ goto_realtopdecl'1__ v1)
                              (reduce_101__ goto_realtopdecl'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> k_90_2__ t__ }
  
  state_185__ k_94_1__ k_100_1__ k_101_1__ t__
      = let {
              goto_simpletype'1__ v1 = state_186__ (k_94_1__ v1);
              goto_conid'2__ v1 v2 = reduce_295__ goto_tycon'1__ v1 v2;
              goto_tycon'1__ v1 = state_209__ (k_100_1__ v1) (k_101_1__ v1) (reduce_152__ goto_simpletype'1__ v1)
            }
        in case t__ of { ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2; _ -> frown ["<conid>"] t__ }
  
  state_186__ k_94_2__ t__ = case t__ of { Equals v1 -> get >>= state_187__ (k_94_2__ v1); _ -> frown ["="] t__ }
  
  state_187__ k_94_3__ t__
      = let {
              goto_xType'1__ v1 = k_94_3__ v1;
              goto_btype'1__ v1
                  = state_258__ (reduce_124__ goto_xType'1__ v1) (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "[", "as", "hiding", "qualified"] t__ }
  
  state_189__ k_95_1__ t__
      = let {
              goto_xQualType'1__ v1 = state_190__ (k_95_1__ v1);
              goto_xType'1__ v1 = reduce_19__ goto_xQualType'1__ v1;
              goto_btype'1__ v1
                  = state_51__ (reduce_17__ goto_xQualType'1__ v1) (reduce_124__ goto_xType'1__ v1)
                                (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1;
              goto_foralls'2__ v1 v2 = state_54__ (reduce_18__ goto_xQualType'1__ v1 v2)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Forall v1 -> get >>= state_617__ (reduce_333__ goto_foralls'2__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "(",
                               "[",
                               "as",
                               "forall",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_190__ k_95_2__ t__ = case t__ of { Equals v1 -> get >>= state_191__ (k_95_2__ v1); _ -> frown ["="] t__ }
  
  state_191__ k_95_3__ t__
      = let {
              goto_mSepBy1'1_constr'1_33__ v1 = state_192__ (k_95_3__ v1);
              goto_mSepBy1''1_constr'1_33__ v1
                  = state_226__ (reduce_104__ goto_mSepBy1'1_constr'1_33__ v1)
                        (reduce_106__ goto_mSepBy1''1_constr'1_33__ v1);
              goto_btype'1__ v1
                  = state_264__ (reduce_127__ goto_btype'1__ v1) (reduce_159__ goto_scontype'1__ v1)
                            (reduce_161__ goto_scontype1'1__ v1)
                        (reduce_165__ goto_sbtype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_constr'1__ v1 = reduce_105__ goto_mSepBy1''1_constr'1_33__ v1;
              goto_scontype'1__ v1 = reduce_153__ goto_constr'1__ v1;
              goto_scontype1'1__ v1
                  = state_322__ (reduce_160__ goto_scontype'1__ v1) (reduce_162__ goto_scontype1'1__ v1);
              goto_sbtype'1__ v1 = state_311__ (reduce_154__ goto_constr'1__ v1);
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_con'1__ v1 = state_314__ (reduce_155__ goto_constr'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = state_491__ (reduce_251__ goto_con'1__ v1 v2) (reduce_274__ goto_qconid2'2__ v1 v2);
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_274__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                      (reduce_134__ goto_atype'1__ v1)
                                                  (reduce_136__ goto_gtycon'1__ v1)
                                              (reduce_137__ goto_gtycon'1__ v1)
                                          (reduce_139__ goto_gtycon'1__ v1)
                                      (reduce_143__ goto_gtycon'1__ v1)
                                  (reduce_250__ goto_qvar'1__ v1)
                              (reduce_252__ goto_con'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               Exclamation v1 -> get >>= state_329__ (reduce_166__ goto_sbtype'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "[", "!", "as", "hiding", "qualified"] t__ }
  
  state_192__ k_95_4__ t__
      = let {
              goto_deriving'1__ v1 = k_95_4__ v1
            }
        in case t__ of {
               KW_Deriving v1
                   -> get
                          >>=
                          state_336__ (reduce_171__ goto_deriving'1__ v1) (reduce_172__ goto_deriving'1__ v1)
                              (reduce_173__ goto_deriving'1__ v1);
               _ -> reduce_170__ goto_deriving'1__ t__ }
  
  state_194__ k_96_1__ t__
      = let {
              goto_xQualType'1__ v1 = state_195__ (k_96_1__ v1);
              goto_xType'1__ v1 = reduce_19__ goto_xQualType'1__ v1;
              goto_btype'1__ v1
                  = state_51__ (reduce_17__ goto_xQualType'1__ v1) (reduce_124__ goto_xType'1__ v1)
                                (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1;
              goto_foralls'2__ v1 v2 = state_54__ (reduce_18__ goto_xQualType'1__ v1 v2)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Forall v1 -> get >>= state_617__ (reduce_333__ goto_foralls'2__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "(",
                               "[",
                               "as",
                               "forall",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_195__ k_96_2__ t__ = case t__ of { Equals v1 -> get >>= state_196__ (k_96_2__ v1); _ -> frown ["="] t__ }
  
  state_196__ k_96_3__ t__
      = let {
              goto_btype'1__ v1
                  = state_264__ (reduce_127__ goto_btype'1__ v1) (reduce_159__ goto_scontype'1__ v1)
                            (reduce_161__ goto_scontype1'1__ v1)
                        (reduce_165__ goto_sbtype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_constr'1__ v1 = state_197__ (k_96_3__ v1);
              goto_scontype'1__ v1 = reduce_153__ goto_constr'1__ v1;
              goto_scontype1'1__ v1
                  = state_322__ (reduce_160__ goto_scontype'1__ v1) (reduce_162__ goto_scontype1'1__ v1);
              goto_sbtype'1__ v1 = state_311__ (reduce_154__ goto_constr'1__ v1);
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_con'1__ v1 = state_314__ (reduce_155__ goto_constr'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = state_491__ (reduce_251__ goto_con'1__ v1 v2) (reduce_274__ goto_qconid2'2__ v1 v2);
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_274__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                      (reduce_134__ goto_atype'1__ v1)
                                                  (reduce_136__ goto_gtycon'1__ v1)
                                              (reduce_137__ goto_gtycon'1__ v1)
                                          (reduce_139__ goto_gtycon'1__ v1)
                                      (reduce_143__ goto_gtycon'1__ v1)
                                  (reduce_250__ goto_qvar'1__ v1)
                              (reduce_252__ goto_con'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               Exclamation v1 -> get >>= state_329__ (reduce_166__ goto_sbtype'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "[", "!", "as", "hiding", "qualified"] t__ }
  
  state_197__ k_96_4__ t__
      = let {
              goto_deriving'1__ v1 = k_96_4__ v1
            }
        in case t__ of {
               KW_Deriving v1
                   -> get
                          >>=
                          state_336__ (reduce_171__ goto_deriving'1__ v1) (reduce_172__ goto_deriving'1__ v1)
                              (reduce_173__ goto_deriving'1__ v1);
               _ -> reduce_170__ goto_deriving'1__ t__ }
  
  state_199__ k_97_1__ t__
      = let {
              goto_xQualType'1__ v1 = state_200__ (k_97_1__ v1);
              goto_xType'1__ v1 = reduce_19__ goto_xQualType'1__ v1;
              goto_btype'1__ v1
                  = state_51__ (reduce_17__ goto_xQualType'1__ v1) (reduce_124__ goto_xType'1__ v1)
                                (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1;
              goto_foralls'2__ v1 v2 = state_54__ (reduce_18__ goto_xQualType'1__ v1 v2)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Forall v1 -> get >>= state_617__ (reduce_333__ goto_foralls'2__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "(",
                               "[",
                               "as",
                               "forall",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_200__ k_97_2__ t__
      = let {
              goto_optcbody'1__ v1 = k_97_2__ v1
            }
        in case t__ of {
               KW_Where v1 -> get >>= state_346__ (reduce_177__ goto_optcbody'1__ v1);
               _ -> reduce_178__ goto_optcbody'1__ t__ }
  
  state_202__ k_98_1__ t__
      = let {
              goto_xQualType'1__ v1 = state_203__ (k_98_1__ v1);
              goto_xType'1__ v1 = reduce_19__ goto_xQualType'1__ v1;
              goto_btype'1__ v1
                  = state_51__ (reduce_17__ goto_xQualType'1__ v1) (reduce_124__ goto_xType'1__ v1)
                                (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1;
              goto_foralls'2__ v1 v2 = state_54__ (reduce_18__ goto_xQualType'1__ v1 v2)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Forall v1 -> get >>= state_617__ (reduce_333__ goto_foralls'2__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "(",
                               "[",
                               "as",
                               "forall",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_203__ k_98_2__ t__
      = let {
              goto_optvaldefs'1__ v1 = k_98_2__ v1
            }
        in case t__ of {
               KW_Where v1
                   -> get >>= state_361__ (reduce_188__ goto_optvaldefs'1__ v1) (reduce_189__ goto_optvaldefs'1__ v1);
               _ -> reduce_190__ goto_optvaldefs'1__ t__ }
  
  state_205__ k_99_1__ t__ = case t__ of { LeftParen v1 -> get >>= state_206__ (k_99_1__ v1); _ -> frown ["("] t__ }
  
  state_206__ k_99_2__ t__
      = let {
              goto_xType'1__ v1 = state_306__ (reduce_148__ goto_types'1__ v1) (reduce_150__ goto_types0'1__ v1);
              goto_btype'1__ v1
                  = state_258__ (reduce_124__ goto_xType'1__ v1) (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_types'1__ v1 = state_303__ (reduce_147__ goto_types'1__ v1) (reduce_149__ goto_types0'1__ v1);
              goto_types0'1__ v1 = state_207__ (k_99_2__ v1);
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               RightParen v1 -> reduce_151__ goto_types0'1__ t__;
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", ")", "[", "as", "hiding", "qualified"] t__ }
  
  state_207__ k_99_3__ t__ = case t__ of { RightParen v1 -> get >>= k_99_3__ v1; _ -> frown [")"] t__ }
  
  state_209__ k_100_2__ k_101_2__ k_152_1__ t__
      = let {
              goto_parenskind'1__ v1 = state_210__ (k_100_2__ v1);
              goto_parenstype'1__ v1 = state_214__ (k_101_2__ v1);
              goto_many'1_kindannottyvar'1__ v1 = k_152_1__ v1;
              goto_many''1_kindannottyvar'1__ v1
                  = state_655__ (reduce_362__ goto_many'1_kindannottyvar'1__ v1)
                        (reduce_379__ goto_many''1_kindannottyvar'1__ v1)
            }
        in case t__ of {
               LeftTyParen v1 -> get >>= state_557__ (reduce_299__ goto_parenstype'1__ v1);
               LeftKiParen v1 -> get >>= state_554__ (reduce_298__ goto_parenskind'1__ v1);
               _ -> reduce_378__ goto_many''1_kindannottyvar'1__ t__ }
  
  state_210__ k_100_3__ t__
      = let {
              goto_many'1_tyvar'1__ v1 = state_211__ (k_100_3__ v1);
              goto_many''1_tyvar'1__ v1
                  = state_656__ (reduce_363__ goto_many'1_tyvar'1__ v1) (reduce_383__ goto_many''1_tyvar'1__ v1)
            }
        in reduce_382__ goto_many''1_tyvar'1__ t__
  
  state_211__ k_100_4__ t__ = case t__ of { Equals v1 -> get >>= state_212__ (k_100_4__ v1); _ -> frown ["="] t__ }
  
  state_212__ k_100_5__ t__
      = let {
              goto_xQualType'1__ v1 = k_100_5__ v1;
              goto_xType'1__ v1 = reduce_19__ goto_xQualType'1__ v1;
              goto_btype'1__ v1
                  = state_51__ (reduce_17__ goto_xQualType'1__ v1) (reduce_124__ goto_xType'1__ v1)
                                (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1;
              goto_foralls'2__ v1 v2 = state_54__ (reduce_18__ goto_xQualType'1__ v1 v2)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Forall v1 -> get >>= state_617__ (reduce_333__ goto_foralls'2__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "(",
                               "[",
                               "as",
                               "forall",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_214__ k_101_3__ t__
      = let {
              goto_many'1_tyvar'1__ v1 = state_215__ (k_101_3__ v1);
              goto_many''1_tyvar'1__ v1
                  = state_656__ (reduce_363__ goto_many'1_tyvar'1__ v1) (reduce_383__ goto_many''1_tyvar'1__ v1)
            }
        in reduce_382__ goto_many''1_tyvar'1__ t__
  
  state_215__ k_101_4__ t__ = case t__ of { Equals v1 -> get >>= state_216__ (k_101_4__ v1); _ -> frown ["="] t__ }
  
  state_216__ k_101_5__ t__
      = let {
              goto_btype'1__ v1
                  = state_264__ (reduce_127__ goto_btype'1__ v1) (reduce_159__ goto_scontype'1__ v1)
                            (reduce_161__ goto_scontype1'1__ v1)
                        (reduce_165__ goto_sbtype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_constr'1__ v1 = state_217__ (k_101_5__ v1);
              goto_scontype'1__ v1 = reduce_153__ goto_constr'1__ v1;
              goto_scontype1'1__ v1
                  = state_322__ (reduce_160__ goto_scontype'1__ v1) (reduce_162__ goto_scontype1'1__ v1);
              goto_sbtype'1__ v1 = state_311__ (reduce_154__ goto_constr'1__ v1);
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_con'1__ v1 = state_314__ (reduce_155__ goto_constr'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = state_491__ (reduce_251__ goto_con'1__ v1 v2) (reduce_274__ goto_qconid2'2__ v1 v2);
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_274__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                      (reduce_134__ goto_atype'1__ v1)
                                                  (reduce_136__ goto_gtycon'1__ v1)
                                              (reduce_137__ goto_gtycon'1__ v1)
                                          (reduce_139__ goto_gtycon'1__ v1)
                                      (reduce_143__ goto_gtycon'1__ v1)
                                  (reduce_250__ goto_qvar'1__ v1)
                              (reduce_252__ goto_con'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               Exclamation v1 -> get >>= state_329__ (reduce_166__ goto_sbtype'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "[", "!", "as", "hiding", "qualified"] t__ }
  
  state_217__ k_101_6__ t__
      = let {
              goto_deriving'1__ v1 = k_101_6__ v1
            }
        in case t__ of {
               KW_Deriving v1
                   -> get
                          >>=
                          state_336__ (reduce_171__ goto_deriving'1__ v1) (reduce_172__ goto_deriving'1__ v1)
                              (reduce_173__ goto_deriving'1__ v1);
               _ -> reduce_170__ goto_deriving'1__ t__ }
  
  state_219__ k_102_1__ k_103_1__ t__
      = let {
              goto_qvar'1__ v1 = state_223__ (k_103_1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_295__ goto_tycon'1__ v1 v2;
              goto_tycon'1__ v1 = state_220__ (k_102_1__ v1)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               LeftParen v1 -> get >>= state_483__ (reduce_250__ goto_qvar'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_220__ k_102_2__ t__ = case t__ of { LeftArrow v1 -> get >>= state_221__ (k_102_2__ v1); _ -> frown ["<-"] t__ }
  
  state_221__ k_102_3__ t__
      = let {
              goto_many'1_tdepbinding'1__ v1 = k_102_3__ v1;
              goto_many''1_tdepbinding'1__ v1
                  = state_652__ (reduce_359__ goto_many'1_tdepbinding'1__ v1)
                        (reduce_367__ goto_many''1_tdepbinding'1__ v1)
            }
        in reduce_366__ goto_many''1_tdepbinding'1__ t__
  
  state_223__ k_103_2__ t__ = case t__ of { LeftArrow v1 -> get >>= state_224__ (k_103_2__ v1); _ -> frown ["<-"] t__ }
  
  state_224__ k_103_3__ t__
      = let {
              goto_many'1_depbinding'1__ v1 = k_103_3__ v1;
              goto_many''1_depbinding'1__ v1
                  = state_654__ (reduce_361__ goto_many'1_depbinding'1__ v1) (reduce_375__ goto_many''1_depbinding'1__ v1)
            }
        in reduce_374__ goto_many''1_depbinding'1__ t__
  
  state_226__ k_104_1__ k_106_1__ t__ = case t__ of { Bar v1 -> get >>= state_228__ (k_106_1__ v1); _ -> k_104_1__ t__ }
  
  state_228__ k_106_2__ t__
      = let {
              goto_btype'1__ v1
                  = state_264__ (reduce_127__ goto_btype'1__ v1) (reduce_159__ goto_scontype'1__ v1)
                            (reduce_161__ goto_scontype1'1__ v1)
                        (reduce_165__ goto_sbtype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_constr'1__ v1 = k_106_2__ v1;
              goto_scontype'1__ v1 = reduce_153__ goto_constr'1__ v1;
              goto_scontype1'1__ v1
                  = state_322__ (reduce_160__ goto_scontype'1__ v1) (reduce_162__ goto_scontype1'1__ v1);
              goto_sbtype'1__ v1 = state_311__ (reduce_154__ goto_constr'1__ v1);
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_con'1__ v1 = state_314__ (reduce_155__ goto_constr'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = state_491__ (reduce_251__ goto_con'1__ v1 v2) (reduce_274__ goto_qconid2'2__ v1 v2);
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_274__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                      (reduce_134__ goto_atype'1__ v1)
                                                  (reduce_136__ goto_gtycon'1__ v1)
                                              (reduce_137__ goto_gtycon'1__ v1)
                                          (reduce_139__ goto_gtycon'1__ v1)
                                      (reduce_143__ goto_gtycon'1__ v1)
                                  (reduce_250__ goto_qvar'1__ v1)
                              (reduce_252__ goto_con'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               Exclamation v1 -> get >>= state_329__ (reduce_166__ goto_sbtype'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "[", "!", "as", "hiding", "qualified"] t__ }
  
  state_233__ k_110_1__ t__
      = let {
              goto_priority'1__ v1 = state_234__ (k_110_1__ v1)
            }
        in case t__ of {
               IntTok v1 v2 -> get >>= reduce_115__ goto_priority'1__ v1 v2;
               _ -> reduce_114__ goto_priority'1__ t__ }
  
  state_234__ k_110_2__ t__
      = let {
              goto_mSepBy1'1_op'1_21__ v1 = k_110_2__ v1;
              goto_mSepBy1''1_op'1_21__ v1
                  = state_236__ (reduce_111__ goto_mSepBy1'1_op'1_21__ v1) (reduce_113__ goto_mSepBy1''1_op'1_21__ v1);
              goto_varop'1__ v1 = reduce_257__ goto_op'1__ v1;
              goto_conop'1__ v1 = reduce_258__ goto_op'1__ v1;
              goto_op'1__ v1 = reduce_112__ goto_mSepBy1''1_op'1_21__ v1;
              goto_consym'2__ v1 v2 = reduce_255__ goto_conop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_259__ goto_varop'1__ v1 v2
            }
        in case t__ of {
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               BackQuote v1 -> get >>= state_503__ (reduce_256__ goto_conop'1__ v1) (reduce_260__ goto_varop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               _ -> frown ["<varsym>", "<consym>", "`", ".", "-", "!"] t__ }
  
  state_236__ k_111_1__ k_113_1__ t__ = case t__ of { Comma v1 -> get >>= state_238__ (k_113_1__ v1); _ -> k_111_1__ t__ }
  
  state_238__ k_113_2__ t__
      = let {
              goto_varop'1__ v1 = reduce_257__ goto_op'1__ v1;
              goto_conop'1__ v1 = reduce_258__ goto_op'1__ v1;
              goto_op'1__ v1 = k_113_2__ v1;
              goto_consym'2__ v1 v2 = reduce_255__ goto_conop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_259__ goto_varop'1__ v1 v2
            }
        in case t__ of {
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               BackQuote v1 -> get >>= state_503__ (reduce_256__ goto_conop'1__ v1) (reduce_260__ goto_varop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               _ -> frown ["<varsym>", "<consym>", "`", ".", "-", "!"] t__ }
  
  state_244__ k_119_1__ t__ = case t__ of { DoubleColon v1 -> get >>= state_245__ (k_119_1__ v1); _ -> frown ["::"] t__ }
  
  state_245__ k_119_2__ t__
      = let {
              goto_xQualType'1__ v1 = k_119_2__ v1;
              goto_xType'1__ v1 = reduce_19__ goto_xQualType'1__ v1;
              goto_btype'1__ v1
                  = state_51__ (reduce_17__ goto_xQualType'1__ v1) (reduce_124__ goto_xType'1__ v1)
                                (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1;
              goto_foralls'2__ v1 v2 = state_54__ (reduce_18__ goto_xQualType'1__ v1 v2)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Forall v1 -> get >>= state_617__ (reduce_333__ goto_foralls'2__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "(",
                               "[",
                               "as",
                               "forall",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_247__ k_120_1__ k_195_1__ k_196_1__ k_218_1__ t__
      = let {
              goto_rhs'2__ v1 v2 = state_373__ (k_195_1__ v1 v2) (k_196_1__ v1 v2);
              goto_gdrh'1__ v1 = state_685__ (reduce_384__ goto_many1'1_gdrh'1__ v1);
              goto_qvarop'1__ v1 = reduce_261__ goto_qop'1__ v1;
              goto_qconop'1__ v1 = reduce_262__ goto_qop'1__ v1;
              goto_qop'1__ v1 = state_410__ (k_218_1__ v1);
              goto_qconsym2'2__ v1 v2 = reduce_265__ goto_qconop'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_278__ goto_qconsym2'2__ v1 v2;
              goto_qvarsym'2__ v1 v2 = reduce_263__ goto_qvarop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_281__ goto_qvarsym'2__ v1 v2;
              goto_qvarsym1'2__ v1 v2 = reduce_282__ goto_qvarsym'2__ v1 v2;
              goto_many1'1_gdrh'1__ v1 = reduce_212__ goto_rhs'2__ v1
            }
        in case t__ of {
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               QVarSym v1 v2 -> get >>= reduce_287__ goto_qvarsym1'2__ v1 v2;
               QConSym v1 v2 -> get >>= reduce_279__ goto_qconsym2'2__ v1 v2;
               BackQuote v1 -> get >>= state_514__ (reduce_264__ goto_qvarop'1__ v1) (reduce_266__ goto_qconop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               DoubleColon v1 -> get >>= state_248__ (k_120_1__ v1);
               Equals v1 -> get >>= state_399__ (reduce_211__ goto_rhs'2__ v1);
               Bar v1 -> get >>= state_687__ (reduce_385__ goto_gdrh'1__ v1);
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               _ -> frown ["<varsym>", "<consym>", "<qvarsym>", "<qconsym>", "`", ".", "::", "=", "|", "-", "!"] t__ }
  
  state_248__ k_120_2__ t__
      = let {
              goto_xType'1__ v1 = reduce_146__ goto_yQualType'1__ v1;
              goto_btype'1__ v1
                  = state_259__ (reduce_124__ goto_xType'1__ v1) (reduce_125__ goto_xType'1__ v1)
                                (reduce_126__ goto_xType'1__ v1)
                            (reduce_127__ goto_btype'1__ v1)
                        (reduce_144__ goto_yQualType'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_yQualType'1__ v1 = k_120_2__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1;
              goto_foralls'2__ v1 v2 = state_300__ (reduce_145__ goto_yQualType'1__ v1 v2)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Forall v1 -> get >>= state_617__ (reduce_333__ goto_foralls'2__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "(",
                               "[",
                               "as",
                               "forall",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_250__ k_121_1__ k_122_1__ t__
      = case t__ of { Comma v1 -> get >>= state_251__ (k_121_1__ v1) (k_122_1__ v1); _ -> frown [","] t__ }
  
  state_251__ k_121_2__ k_122_2__ t__
      = let {
              goto_var'1__ v1 = state_252__ (k_121_2__ v1) (k_122_2__ v1);
              goto_varid'2__ v1 v2 = reduce_247__ goto_var'1__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               LeftParen v1 -> get >>= state_478__ (reduce_248__ goto_var'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_252__ k_121_3__ k_122_3__ t__
      = case t__ of { Comma v1 -> k_122_3__ t__; DoubleColon v1 -> k_121_3__ t__; _ -> frown [",", "::"] t__ }
  
  state_253__ k_122_1__ k_167_1__ t__
      = case t__ of {
            Comma v1 -> get >>= state_254__ (k_122_1__ v1);
            DoubleColon v1 -> get >>= state_331__ (k_167_1__ v1);
            _ -> frown [",", "::"] t__ }
  
  state_254__ k_122_2__ t__
      = let {
              goto_var'1__ v1 = k_122_2__ v1;
              goto_varid'2__ v1 v2 = reduce_247__ goto_var'1__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               LeftParen v1 -> get >>= state_478__ (reduce_248__ goto_var'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_257__ k_123_1__ k_197_1__ k_198_1__ k_390_1__ k_399_1__ k_400_1__ t__
      = let {
              goto_parentype'2__ v1 v2 = k_400_1__ v1 v2
            }
        in case t__ of {
               Comma v1 -> k_123_1__ t__;
               LeftTyParen v1
                   -> get
                          >>=
                          state_563__ (reduce_301__ goto_parentype'2__ v1) (reduce_302__ goto_parentype'2__ v1)
                                      (reduce_303__ goto_parentype'2__ v1)
                                  (reduce_304__ goto_parentype'2__ v1)
                              (reduce_305__ goto_parentype'2__ v1);
               At v1 -> get >>= state_696__ (k_390_1__ v1);
               KW_Extends v1 -> get >>= state_377__ (k_197_1__ v1) (k_198_1__ v1);
               _ -> k_399_1__ t__ }
  
  state_258__ k_124_1__ k_125_1__ k_126_1__ k_127_1__ t__
      = let {
              goto_atype'1__ v1 = k_127_1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_qconsym'1__ v1 = state_262__ (k_125_1__ v1);
              goto_qconsym2'2__ v1 v2 = reduce_277__ goto_qconsym'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_278__ goto_qconsym2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               QConSym v1 v2 -> get >>= reduce_279__ goto_qconsym2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               RightArrow v1 -> get >>= state_260__ (k_124_1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> k_126_1__ t__ }
  
  state_259__ k_124_1__ k_125_1__ k_126_1__ k_127_1__ k_144_1__ t__
      = let {
              goto_atype'1__ v1 = k_127_1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_qconsym'1__ v1 = state_262__ (k_125_1__ v1);
              goto_qconsym2'2__ v1 v2 = reduce_277__ goto_qconsym'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_278__ goto_qconsym2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               QConSym v1 v2 -> get >>= reduce_279__ goto_qconsym2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               RightArrow v1 -> get >>= state_260__ (k_124_1__ v1);
               DoubleArrow v1 -> get >>= state_298__ (k_144_1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> k_126_1__ t__ }
  
  state_260__ k_124_2__ t__
      = let {
              goto_xType'1__ v1 = k_124_2__ v1;
              goto_btype'1__ v1
                  = state_258__ (reduce_124__ goto_xType'1__ v1) (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "[", "as", "hiding", "qualified"] t__ }
  
  state_262__ k_125_2__ t__
      = let {
              goto_xType'1__ v1 = k_125_2__ v1;
              goto_btype'1__ v1
                  = state_258__ (reduce_124__ goto_xType'1__ v1) (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "[", "as", "hiding", "qualified"] t__ }
  
  state_264__ k_127_1__ k_159_1__ k_161_1__ k_165_1__ t__
      = let {
              goto_atype'1__ v1 = k_127_1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               ConSym v1 v2 -> k_165_1__ t__;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               BackQuote v1 -> k_165_1__ t__;
               Exclamation v1 -> get >>= state_323__ (k_161_1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> k_159_1__ t__ }
  
  state_265__ k_127_1__ k_165_1__ t__
      = let {
              goto_atype'1__ v1 = k_127_1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> k_165_1__ t__ }
  
  state_270__ k_130_1__ k_131_2__ t__
      = case t__ of { DoubleColon v1 -> get >>= state_275__ (k_131_2__ v1); _ -> k_130_1__ t__ }
  
  state_271__ k_130_1__ k_314_1__ k_315_1__ k_316_1__ t__
      = case t__ of {
            Comma v1 -> k_314_1__ t__;
            DoubleColon v1 -> get >>= state_589__ (k_315_1__ v1) (k_316_1__ v1);
            Bar v1 -> k_314_1__ t__;
            _ -> k_130_1__ t__ }
  
  state_272__ k_130_1__ k_316_1__ t__
      = case t__ of { DoubleColon v1 -> get >>= state_592__ (k_316_1__ v1); _ -> k_130_1__ t__ }
  
  state_273__ k_131_1__ k_132_1__ k_134_1__ k_136_1__ k_137_1__ k_139_1__ k_143_1__ k_250_1__ t__
      = let {
              goto_xQualType'1__ v1 = state_283__ (k_134_1__ v1);
              goto_xType'1__ v1 = state_57__ (reduce_19__ goto_xQualType'1__ v1) (reduce_148__ goto_types'1__ v1);
              goto_btype'1__ v1
                  = state_51__ (reduce_17__ goto_xQualType'1__ v1) (reduce_124__ goto_xType'1__ v1)
                                (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_types'1__ v1 = state_278__ (k_132_1__ v1) (reduce_147__ goto_types'1__ v1);
              goto_commas'1__ v1 = state_290__ (k_139_1__ v1) (reduce_227__ goto_commas'1__ v1);
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_qconsym'1__ v1 = state_296__ (k_143_1__ v1);
              goto_qconsym2'2__ v1 v2 = reduce_277__ goto_qconsym'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_278__ goto_qconsym2'2__ v1 v2;
              goto_qvarsym'2__ v1 v2 = state_488__ (k_250_1__ v1 v2);
              goto_varsym'2__ v1 v2 = reduce_281__ goto_qvarsym'2__ v1 v2;
              goto_qvarsym1'2__ v1 v2 = reduce_282__ goto_qvarsym'2__ v1 v2;
              goto_tyvar'1__ v1 = state_270__ (reduce_130__ goto_atype'1__ v1) (k_131_1__ v1);
              goto_foralls'2__ v1 v2 = state_54__ (reduce_18__ goto_xQualType'1__ v1 v2)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               QVarSym v1 v2 -> get >>= reduce_287__ goto_qvarsym1'2__ v1 v2;
               QConSym v1 v2 -> get >>= reduce_279__ goto_qconsym2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               RightParen v1 -> get >>= k_136_1__ v1;
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               Comma v1 -> get >>= reduce_228__ goto_commas'1__ v1;
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               RightArrow v1 -> get >>= state_287__ (k_137_1__ v1);
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Forall v1 -> get >>= state_617__ (reduce_333__ goto_foralls'2__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<varsym>",
                               "<consym>",
                               "<qvarsym>",
                               "<qconsym>",
                               "(",
                               ")",
                               "[",
                               ",",
                               ".",
                               "->",
                               "-",
                               "!",
                               "as",
                               "forall",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_274__ k_131_1__ k_132_1__ k_134_1__ k_136_1__ k_137_1__ k_139_1__ k_143_1__ k_250_1__ k_252_1__ t__
      = let {
              goto_xQualType'1__ v1 = state_283__ (k_134_1__ v1);
              goto_xType'1__ v1 = state_57__ (reduce_19__ goto_xQualType'1__ v1) (reduce_148__ goto_types'1__ v1);
              goto_btype'1__ v1
                  = state_51__ (reduce_17__ goto_xQualType'1__ v1) (reduce_124__ goto_xType'1__ v1)
                                (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_types'1__ v1 = state_278__ (k_132_1__ v1) (reduce_147__ goto_types'1__ v1);
              goto_commas'1__ v1 = state_290__ (k_139_1__ v1) (reduce_227__ goto_commas'1__ v1);
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_qconsym'1__ v1 = state_296__ (k_143_1__ v1);
              goto_qconsym2'2__ v1 v2 = reduce_277__ goto_qconsym'1__ v1 v2;
              goto_consym'2__ v1 v2 = state_494__ (k_252_1__ v1 v2) (reduce_278__ goto_qconsym2'2__ v1 v2);
              goto_qvarsym'2__ v1 v2 = state_488__ (k_250_1__ v1 v2);
              goto_varsym'2__ v1 v2 = reduce_281__ goto_qvarsym'2__ v1 v2;
              goto_qvarsym1'2__ v1 v2 = reduce_282__ goto_qvarsym'2__ v1 v2;
              goto_tyvar'1__ v1 = state_270__ (reduce_130__ goto_atype'1__ v1) (k_131_1__ v1);
              goto_foralls'2__ v1 v2 = state_54__ (reduce_18__ goto_xQualType'1__ v1 v2)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               QVarSym v1 v2 -> get >>= reduce_287__ goto_qvarsym1'2__ v1 v2;
               QConSym v1 v2 -> get >>= reduce_279__ goto_qconsym2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               RightParen v1 -> get >>= k_136_1__ v1;
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               Comma v1 -> get >>= reduce_228__ goto_commas'1__ v1;
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               RightArrow v1 -> get >>= state_287__ (k_137_1__ v1);
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Forall v1 -> get >>= state_617__ (reduce_333__ goto_foralls'2__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<varsym>",
                               "<consym>",
                               "<qvarsym>",
                               "<qconsym>",
                               "(",
                               ")",
                               "[",
                               ",",
                               ".",
                               "->",
                               "-",
                               "!",
                               "as",
                               "forall",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_275__ k_131_3__ t__
      = let {
              goto_varop'1__ v1 = reduce_257__ goto_op'1__ v1;
              goto_conop'1__ v1 = reduce_258__ goto_op'1__ v1;
              goto_op'1__ v1 = reduce_323__ goto_kindterm'1__ v1;
              goto_varid'2__ v1 v2 = reduce_326__ goto_kivar'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_255__ goto_conop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_259__ goto_varop'1__ v1 v2;
              goto_kind'1__ v1 = state_276__ (k_131_3__ v1);
              goto_kindterm'1__ v1 = state_600__ (reduce_321__ goto_kind'1__ v1) (reduce_322__ goto_kind'1__ v1);
              goto_kivar'1__ v1 = reduce_324__ goto_kindterm'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               LeftParen v1 -> get >>= state_605__ (reduce_325__ goto_kindterm'1__ v1);
               BackQuote v1 -> get >>= state_503__ (reduce_256__ goto_conop'1__ v1) (reduce_260__ goto_varop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<varsym>", "<consym>", "(", "`", ".", "-", "!", "as", "hiding", "qualified"] t__ }
  
  state_276__ k_131_4__ t__ = case t__ of { RightParen v1 -> get >>= k_131_4__ v1; _ -> frown [")"] t__ }
  
  state_278__ k_132_2__ k_147_1__ t__
      = case t__ of {
            RightParen v1 -> get >>= k_132_2__ v1;
            Comma v1 -> get >>= state_304__ (k_147_1__ v1);
            _ -> frown [")", ","] t__ }
  
  state_280__ k_133_1__ k_138_1__ t__
      = let {
              goto_xType'1__ v1 = state_281__ (k_133_1__ v1);
              goto_btype'1__ v1
                  = state_258__ (reduce_124__ goto_xType'1__ v1) (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               RightSquare v1 -> get >>= k_138_1__ v1;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "[", "]", "as", "hiding", "qualified"] t__ }
  
  state_281__ k_133_2__ t__ = case t__ of { RightSquare v1 -> get >>= k_133_2__ v1; _ -> frown ["]"] t__ }
  
  state_283__ k_134_2__ t__ = case t__ of { RightParen v1 -> get >>= k_134_2__ v1; _ -> frown [")"] t__ }
  
  state_285__ k_135_1__ k_140_1__ k_141_1__ t__
      = let {
              goto_parenkind'2__ v1 v2 = k_140_1__ v1 v2;
              goto_parentype'2__ v1 v2 = k_141_1__ v1 v2
            }
        in case t__ of {
               LeftTyParen v1
                   -> get
                          >>=
                          state_563__ (reduce_301__ goto_parentype'2__ v1) (reduce_302__ goto_parentype'2__ v1)
                                      (reduce_303__ goto_parentype'2__ v1)
                                  (reduce_304__ goto_parentype'2__ v1)
                              (reduce_305__ goto_parentype'2__ v1);
               LeftKiParen v1 -> get >>= state_560__ (reduce_300__ goto_parenkind'2__ v1);
               _ -> k_135_1__ t__ }
  
  state_287__ k_137_2__ t__ = case t__ of { RightParen v1 -> get >>= k_137_2__ v1; _ -> frown [")"] t__ }
  
  state_290__ k_139_2__ k_227_1__ t__
      = case t__ of { RightParen v1 -> get >>= k_139_2__ v1; Comma v1 -> get >>= k_227_1__ v1; _ -> frown [")", ","] t__ }
  
  state_294__ k_142_1__ t__
      = let {
              goto_parentype'2__ v1 v2 = k_142_1__ v1 v2
            }
        in case t__ of {
               LeftTyParen v1
                   -> get
                          >>=
                          state_563__ (reduce_301__ goto_parentype'2__ v1) (reduce_302__ goto_parentype'2__ v1)
                                      (reduce_303__ goto_parentype'2__ v1)
                                  (reduce_304__ goto_parentype'2__ v1)
                              (reduce_305__ goto_parentype'2__ v1);
               _ -> frown ["{|"] t__ }
  
  state_296__ k_143_2__ t__ = case t__ of { RightParen v1 -> get >>= k_143_2__ v1; _ -> frown [")"] t__ }
  
  state_298__ k_144_2__ t__
      = let {
              goto_xQualType'1__ v1 = k_144_2__ v1;
              goto_xType'1__ v1 = reduce_19__ goto_xQualType'1__ v1;
              goto_btype'1__ v1
                  = state_51__ (reduce_17__ goto_xQualType'1__ v1) (reduce_124__ goto_xType'1__ v1)
                                (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1;
              goto_foralls'2__ v1 v2 = state_54__ (reduce_18__ goto_xQualType'1__ v1 v2)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Forall v1 -> get >>= state_617__ (reduce_333__ goto_foralls'2__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "(",
                               "[",
                               "as",
                               "forall",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_300__ k_145_1__ t__
      = let {
              goto_xQualType'1__ v1 = k_145_1__ v1;
              goto_xType'1__ v1 = reduce_19__ goto_xQualType'1__ v1;
              goto_btype'1__ v1
                  = state_51__ (reduce_17__ goto_xQualType'1__ v1) (reduce_124__ goto_xType'1__ v1)
                                (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1;
              goto_foralls'2__ v1 v2 = state_54__ (reduce_18__ goto_xQualType'1__ v1 v2)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Forall v1 -> get >>= state_617__ (reduce_333__ goto_foralls'2__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "(",
                               "[",
                               "as",
                               "forall",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_303__ k_147_1__ k_149_1__ t__
      = case t__ of {
            RightParen v1 -> k_149_1__ t__;
            Comma v1 -> get >>= state_304__ (k_147_1__ v1);
            _ -> frown [")", ","] t__ }
  
  state_304__ k_147_2__ t__
      = let {
              goto_xType'1__ v1 = k_147_2__ v1;
              goto_btype'1__ v1
                  = state_258__ (reduce_124__ goto_xType'1__ v1) (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "[", "as", "hiding", "qualified"] t__ }
  
  state_306__ k_148_1__ k_150_1__ t__
      = case t__ of {
            RightParen v1 -> k_150_1__ t__;
            Comma v1 -> get >>= state_307__ (k_148_1__ v1);
            _ -> frown [")", ","] t__ }
  
  state_307__ k_148_2__ t__
      = let {
              goto_xType'1__ v1 = k_148_2__ v1;
              goto_btype'1__ v1
                  = state_258__ (reduce_124__ goto_xType'1__ v1) (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "[", "as", "hiding", "qualified"] t__ }
  
  state_311__ k_154_1__ t__
      = let {
              goto_conop'1__ v1 = state_312__ (k_154_1__ v1);
              goto_consym'2__ v1 v2 = reduce_255__ goto_conop'1__ v1 v2
            }
        in case t__ of {
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               BackQuote v1 -> get >>= state_502__ (reduce_256__ goto_conop'1__ v1);
               _ -> frown ["<consym>", "`"] t__ }
  
  state_312__ k_154_2__ t__
      = let {
              goto_btype'1__ v1 = state_265__ (reduce_127__ goto_btype'1__ v1) (reduce_165__ goto_sbtype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_sbtype'1__ v1 = k_154_2__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               Exclamation v1 -> get >>= state_329__ (reduce_166__ goto_sbtype'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "[", "!", "as", "hiding", "qualified"] t__ }
  
  state_314__ k_155_1__ t__ = case t__ of { LeftCurly v1 -> get >>= state_315__ (k_155_1__ v1); _ -> frown ["{"] t__ }
  
  state_315__ k_155_2__ t__
      = let {
              goto_mSepBy1'1_fielddecl'1_21__ v1 = state_316__ (k_155_2__ v1);
              goto_mSepBy1''1_fielddecl'1_21__ v1
                  = state_318__ (reduce_156__ goto_mSepBy1'1_fielddecl'1_21__ v1)
                        (reduce_158__ goto_mSepBy1''1_fielddecl'1_21__ v1);
              goto_vars'1__ v1 = state_253__ (reduce_122__ goto_vars'1__ v1) (reduce_167__ goto_fielddecl'1__ v1);
              goto_fielddecl'1__ v1 = reduce_157__ goto_mSepBy1''1_fielddecl'1_21__ v1;
              goto_qvar'1__ v1 = reduce_123__ goto_vars'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               LeftParen v1 -> get >>= state_483__ (reduce_250__ goto_qvar'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_316__ k_155_3__ t__ = case t__ of { RightCurly v1 -> get >>= k_155_3__ v1; _ -> frown ["}"] t__ }
  
  state_318__ k_156_1__ k_158_1__ t__
      = case t__ of {
            RightCurly v1 -> k_156_1__ t__;
            Comma v1 -> get >>= state_320__ (k_158_1__ v1);
            _ -> frown ["}", ","] t__ }
  
  state_320__ k_158_2__ t__
      = let {
              goto_vars'1__ v1 = state_253__ (reduce_122__ goto_vars'1__ v1) (reduce_167__ goto_fielddecl'1__ v1);
              goto_fielddecl'1__ v1 = k_158_2__ v1;
              goto_qvar'1__ v1 = reduce_123__ goto_vars'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               LeftParen v1 -> get >>= state_483__ (reduce_250__ goto_qvar'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_322__ k_160_1__ k_162_1__ t__
      = let {
              goto_atype'1__ v1 = reduce_163__ goto_satype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_satype'1__ v1 = k_162_1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               Exclamation v1 -> get >>= state_327__ (reduce_164__ goto_satype'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> k_160_1__ t__ }
  
  state_323__ k_161_2__ t__
      = let {
              goto_atype'1__ v1 = k_161_2__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "[", "as", "hiding", "qualified"] t__ }
  
  state_327__ k_164_1__ t__
      = let {
              goto_atype'1__ v1 = k_164_1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "[", "as", "hiding", "qualified"] t__ }
  
  state_329__ k_166_1__ t__
      = let {
              goto_atype'1__ v1 = k_166_1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "[", "as", "hiding", "qualified"] t__ }
  
  state_331__ k_167_2__ t__
      = let {
              goto_xType'1__ v1 = reduce_168__ goto_stype'1__ v1;
              goto_btype'1__ v1
                  = state_258__ (reduce_124__ goto_xType'1__ v1) (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_stype'1__ v1 = k_167_2__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               Exclamation v1 -> get >>= state_334__ (reduce_169__ goto_stype'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "[", "!", "as", "hiding", "qualified"] t__ }
  
  state_334__ k_169_1__ t__
      = let {
              goto_atype'1__ v1 = k_169_1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "[", "as", "hiding", "qualified"] t__ }
  
  state_336__ k_171_1__ k_172_1__ k_173_1__ t__
      = let {
              goto_qconid'1__ v1 = reduce_296__ goto_qtycls'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_qtycls'1__ v1 = k_171_1__ v1
            }
        in case t__ of {
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1 -> get >>= state_338__ (k_172_1__ v1) (k_173_1__ v1);
               _ -> frown ["<conid>", "<qconid>", "("] t__ }
  
  state_338__ k_172_2__ k_173_2__ t__
      = let {
              goto_mSepBy1'1_qtycls'1_21__ v1 = state_340__ (k_173_2__ v1);
              goto_mSepBy1''1_qtycls'1_21__ v1
                  = state_342__ (reduce_174__ goto_mSepBy1'1_qtycls'1_21__ v1)
                        (reduce_176__ goto_mSepBy1''1_qtycls'1_21__ v1);
              goto_qconid'1__ v1 = reduce_296__ goto_qtycls'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_qtycls'1__ v1 = reduce_175__ goto_mSepBy1''1_qtycls'1_21__ v1
            }
        in case t__ of {
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               RightParen v1 -> get >>= k_172_2__ v1;
               _ -> frown ["<conid>", "<qconid>", ")"] t__ }
  
  state_340__ k_173_3__ t__ = case t__ of { RightParen v1 -> get >>= k_173_3__ v1; _ -> frown [")"] t__ }
  
  state_342__ k_174_1__ k_176_1__ t__
      = case t__ of {
            RightParen v1 -> k_174_1__ t__;
            Comma v1 -> get >>= state_344__ (k_176_1__ v1);
            _ -> frown [")", ","] t__ }
  
  state_344__ k_176_2__ t__
      = let {
              goto_qconid'1__ v1 = reduce_296__ goto_qtycls'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_qtycls'1__ v1 = k_176_2__ v1
            }
        in case t__ of {
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               _ -> frown ["<conid>", "<qconid>"] t__ }
  
  state_346__ k_177_1__ t__
      = let {
              goto_cdecllist'1__ v1 = k_177_1__ v1;
              goto_layout_on'1__ v1 = state_351__ (reduce_180__ goto_cdecllist'1__ v1)
            }
        in case t__ of {
               LeftCurly v1 -> get >>= state_348__ (reduce_179__ goto_cdecllist'1__ v1);
               _ -> reduce_290__ goto_layout_on'1__ t__ }
  
  state_348__ k_179_1__ t__
      = let {
              goto_semis'1__ v1 = reduce_337__ goto_opt'1_semis'1__ v1;
              goto_cdecls'1__ v1 = state_349__ (k_179_1__ v1);
              goto_opt'1_semis'1__ v1
                  = state_134__ (reduce_58__ goto_semis'1__ v1) (reduce_181__ goto_cdecls'1__ v1)
                        (reduce_182__ goto_cdecls'1__ v1)
            }
        in reduce_336__ goto_opt'1_semis'1__ t__
  
  state_349__ k_179_2__ t__ = case t__ of { RightCurly v1 -> get >>= k_179_2__ v1; _ -> frown ["}"] t__ }
  
  state_351__ k_180_1__ t__
      = let {
              goto_semis'1__ v1 = reduce_337__ goto_opt'1_semis'1__ v1;
              goto_cdecls'1__ v1 = state_352__ (k_180_1__ v1);
              goto_opt'1_semis'1__ v1
                  = state_134__ (reduce_58__ goto_semis'1__ v1) (reduce_181__ goto_cdecls'1__ v1)
                        (reduce_182__ goto_cdecls'1__ v1)
            }
        in reduce_336__ goto_opt'1_semis'1__ t__
  
  state_352__ k_180_2__ t__
      = let {
              goto_close'1__ v1 = k_180_2__ v1
            }
        in case t__ of { VRightCurly -> get >>= reduce_288__ goto_close'1__; _ -> reduce_289__ goto_close'1__ t__ }
  
  state_354__ k_181_2__ k_184_1__ t__
      = let {
              goto_semis'1__ v1 = state_356__ (k_184_1__ v1) (reduce_337__ goto_opt'1_semis'1__ v1);
              goto_opt'1_semis'1__ v1 = state_135__ (reduce_58__ goto_semis'1__ v1) (k_181_2__ v1)
            }
        in reduce_336__ goto_opt'1_semis'1__ t__
  
  state_356__ k_184_2__ k_337_1__ t__
      = let {
              goto_fixdecl'1__ v1 = reduce_186__ goto_cdecl'1__ v1;
              goto_infix'2__ v1 v2 = state_233__ (reduce_110__ goto_fixdecl'1__ v1 v2);
              goto_signdecl'1__ v1 = reduce_185__ goto_cdecl'1__ v1;
              goto_vars'1__ v1 = state_250__ (reduce_121__ goto_vars2'1__ v1) (reduce_122__ goto_vars'1__ v1);
              goto_vars2'1__ v1 = state_244__ (reduce_119__ goto_signdecl'1__ v1);
              goto_cdecl'1__ v1 = k_184_2__ v1;
              goto_valdef'1__ v1 = reduce_187__ goto_cdecl'1__ v1;
              goto_exp0b'1__ v1
                  = state_247__ (reduce_120__ goto_signdecl'1__ v1) (reduce_195__ goto_valdef'1__ v1)
                            (reduce_196__ goto_valdef'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_257__ (reduce_123__ goto_vars'1__ v1) (reduce_197__ goto_valdef'1__ v1)
                                    (reduce_198__ goto_valdef'1__ v1)
                                (reduce_390__ goto_aexp'1__ v1)
                            (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Infix v1 -> get >>= reduce_116__ goto_infix'2__ v1;
               KW_InfixL v1 -> get >>= reduce_117__ goto_infix'2__ v1;
               KW_InfixR v1 -> get >>= reduce_118__ goto_infix'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> k_337_1__ t__ }
  
  state_361__ k_188_1__ k_189_1__ t__
      = let {
              goto_layout_on'1__ v1 = state_365__ (k_189_1__ v1)
            }
        in case t__ of { LeftCurly v1 -> get >>= state_362__ (k_188_1__ v1); _ -> reduce_290__ goto_layout_on'1__ t__ }
  
  state_362__ k_188_2__ t__
      = let {
              goto_semis'1__ v1 = reduce_337__ goto_opt'1_semis'1__ v1;
              goto_valdefs'1__ v1 = state_363__ (k_188_2__ v1);
              goto_opt'1_semis'1__ v1
                  = state_136__ (reduce_58__ goto_semis'1__ v1) (reduce_191__ goto_valdefs'1__ v1)
                        (reduce_192__ goto_valdefs'1__ v1)
            }
        in reduce_336__ goto_opt'1_semis'1__ t__
  
  state_363__ k_188_3__ t__ = case t__ of { RightCurly v1 -> get >>= k_188_3__ v1; _ -> frown ["}"] t__ }
  
  state_365__ k_189_2__ t__
      = let {
              goto_semis'1__ v1 = reduce_337__ goto_opt'1_semis'1__ v1;
              goto_valdefs'1__ v1 = state_366__ (k_189_2__ v1);
              goto_opt'1_semis'1__ v1
                  = state_136__ (reduce_58__ goto_semis'1__ v1) (reduce_191__ goto_valdefs'1__ v1)
                        (reduce_192__ goto_valdefs'1__ v1)
            }
        in reduce_336__ goto_opt'1_semis'1__ t__
  
  state_366__ k_189_3__ t__
      = let {
              goto_close'1__ v1 = k_189_3__ v1
            }
        in case t__ of { VRightCurly -> get >>= reduce_288__ goto_close'1__; _ -> reduce_289__ goto_close'1__ t__ }
  
  state_368__ k_191_2__ k_193_1__ t__
      = let {
              goto_semis'1__ v1 = state_369__ (k_193_1__ v1) (reduce_337__ goto_opt'1_semis'1__ v1);
              goto_opt'1_semis'1__ v1 = state_137__ (reduce_58__ goto_semis'1__ v1) (k_191_2__ v1)
            }
        in reduce_336__ goto_opt'1_semis'1__ t__
  
  state_369__ k_193_2__ k_337_1__ t__
      = let {
              goto_valdef'1__ v1 = k_193_2__ v1;
              goto_exp0b'1__ v1
                  = state_372__ (reduce_195__ goto_valdef'1__ v1) (reduce_196__ goto_valdef'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_376__ (reduce_197__ goto_valdef'1__ v1) (reduce_198__ goto_valdef'1__ v1)
                                (reduce_390__ goto_aexp'1__ v1)
                            (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> k_337_1__ t__ }
  
  state_372__ k_195_1__ k_196_1__ k_218_1__ t__
      = let {
              goto_rhs'2__ v1 v2 = state_373__ (k_195_1__ v1 v2) (k_196_1__ v1 v2);
              goto_gdrh'1__ v1 = state_685__ (reduce_384__ goto_many1'1_gdrh'1__ v1);
              goto_qvarop'1__ v1 = reduce_261__ goto_qop'1__ v1;
              goto_qconop'1__ v1 = reduce_262__ goto_qop'1__ v1;
              goto_qop'1__ v1 = state_410__ (k_218_1__ v1);
              goto_qconsym2'2__ v1 v2 = reduce_265__ goto_qconop'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_278__ goto_qconsym2'2__ v1 v2;
              goto_qvarsym'2__ v1 v2 = reduce_263__ goto_qvarop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_281__ goto_qvarsym'2__ v1 v2;
              goto_qvarsym1'2__ v1 v2 = reduce_282__ goto_qvarsym'2__ v1 v2;
              goto_many1'1_gdrh'1__ v1 = reduce_212__ goto_rhs'2__ v1
            }
        in case t__ of {
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               QVarSym v1 v2 -> get >>= reduce_287__ goto_qvarsym1'2__ v1 v2;
               QConSym v1 v2 -> get >>= reduce_279__ goto_qconsym2'2__ v1 v2;
               BackQuote v1 -> get >>= state_514__ (reduce_264__ goto_qvarop'1__ v1) (reduce_266__ goto_qconop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Equals v1 -> get >>= state_399__ (reduce_211__ goto_rhs'2__ v1);
               Bar v1 -> get >>= state_687__ (reduce_385__ goto_gdrh'1__ v1);
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               _ -> frown ["<varsym>", "<consym>", "<qvarsym>", "<qconsym>", "`", ".", "=", "|", "-", "!"] t__ }
  
  state_373__ k_195_2__ k_196_2__ t__
      = case t__ of { KW_Where v1 -> get >>= state_374__ (k_196_2__ v1); _ -> k_195_2__ t__ }
  
  state_374__ k_196_3__ t__
      = let {
              goto_decllist'2__ v1 v2 = k_196_3__ v1 v2;
              goto_layout_on'1__ v1 = state_389__ (reduce_205__ goto_decllist'2__ v1)
            }
        in case t__ of {
               LeftCurly v1 -> get >>= state_386__ (reduce_204__ goto_decllist'2__ v1);
               _ -> reduce_290__ goto_layout_on'1__ t__ }
  
  state_376__ k_197_1__ k_198_1__ k_390_1__ k_399_1__ k_400_1__ t__
      = let {
              goto_parentype'2__ v1 v2 = k_400_1__ v1 v2
            }
        in case t__ of {
               LeftTyParen v1
                   -> get
                          >>=
                          state_563__ (reduce_301__ goto_parentype'2__ v1) (reduce_302__ goto_parentype'2__ v1)
                                      (reduce_303__ goto_parentype'2__ v1)
                                  (reduce_304__ goto_parentype'2__ v1)
                              (reduce_305__ goto_parentype'2__ v1);
               At v1 -> get >>= state_696__ (k_390_1__ v1);
               KW_Extends v1 -> get >>= state_377__ (k_197_1__ v1) (k_198_1__ v1);
               _ -> k_399_1__ t__ }
  
  state_377__ k_197_2__ k_198_2__ t__
      = let {
              goto_qvar'1__ v1 = state_378__ (k_197_2__ v1) (k_198_2__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               LeftParen v1 -> get >>= state_483__ (reduce_250__ goto_qvar'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_378__ k_197_3__ k_198_3__ t__
      = case t__ of { KW_Where v1 -> get >>= state_379__ (k_198_3__ v1); _ -> k_197_3__ t__ }
  
  state_379__ k_198_4__ t__
      = let {
              goto_mSepBy'1_reassignment'1_15__ v1 = k_198_4__ v1;
              goto_mSepBy1'1_reassignment'1_15__ v1 = reduce_200__ goto_mSepBy'1_reassignment'1_15__ v1;
              goto_mSepBy1''1_reassignment'1_15__ v1
                  = state_382__ (reduce_201__ goto_mSepBy1'1_reassignment'1_15__ v1)
                        (reduce_203__ goto_mSepBy1''1_reassignment'1_15__ v1);
              goto_reassignment'1__ v1 = reduce_202__ goto_mSepBy1''1_reassignment'1_15__ v1;
              goto_qvar'1__ v1 = state_396__ (reduce_210__ goto_reassignment'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               LeftParen v1 -> get >>= state_483__ (reduce_250__ goto_qvar'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> reduce_199__ goto_mSepBy'1_reassignment'1_15__ t__ }
  
  state_382__ k_201_1__ k_203_1__ t__
      = case t__ of { SemiColon v1 -> get >>= state_384__ (k_203_1__ v1); _ -> k_201_1__ t__ }
  
  state_384__ k_203_2__ t__
      = let {
              goto_reassignment'1__ v1 = k_203_2__ v1;
              goto_qvar'1__ v1 = state_396__ (reduce_210__ goto_reassignment'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               LeftParen v1 -> get >>= state_483__ (reduce_250__ goto_qvar'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_386__ k_204_1__ t__
      = let {
              goto_semis'1__ v1 = reduce_337__ goto_opt'1_semis'1__ v1;
              goto_decls'2__ v1 v2 = state_387__ (k_204_1__ v1 v2);
              goto_opt'1_semis'1__ v1
                  = state_138__ (reduce_58__ goto_semis'1__ v1) (reduce_206__ goto_decls'2__ v1)
                        (reduce_207__ goto_decls'2__ v1)
            }
        in reduce_336__ goto_opt'1_semis'1__ t__
  
  state_387__ k_204_2__ t__ = case t__ of { RightCurly v1 -> get >>= k_204_2__ v1; _ -> frown ["}"] t__ }
  
  state_389__ k_205_1__ t__
      = let {
              goto_semis'1__ v1 = reduce_337__ goto_opt'1_semis'1__ v1;
              goto_decls'2__ v1 v2 = state_390__ (k_205_1__ v1 v2);
              goto_opt'1_semis'1__ v1
                  = state_138__ (reduce_58__ goto_semis'1__ v1) (reduce_206__ goto_decls'2__ v1)
                        (reduce_207__ goto_decls'2__ v1)
            }
        in reduce_336__ goto_opt'1_semis'1__ t__
  
  state_390__ k_205_2__ t__
      = let {
              goto_close'1__ v1 = k_205_2__ v1
            }
        in case t__ of { VRightCurly -> get >>= reduce_288__ goto_close'1__; _ -> reduce_289__ goto_close'1__ t__ }
  
  state_392__ k_206_2__ k_209_1__ t__
      = let {
              goto_semis'1__ v1 = state_394__ (k_209_1__ v1) (reduce_337__ goto_opt'1_semis'1__ v1);
              goto_opt'1_semis'1__ v1 = state_139__ (reduce_58__ goto_semis'1__ v1) (k_206_2__ v1)
            }
        in reduce_336__ goto_opt'1_semis'1__ t__
  
  state_394__ k_209_2__ k_337_1__ t__
      = let {
              goto_fixdecl'1__ v1 = reduce_108__ goto_decl'1__ v1;
              goto_infix'2__ v1 v2 = state_233__ (reduce_110__ goto_fixdecl'1__ v1 v2);
              goto_decl'1__ v1 = k_209_2__ v1;
              goto_signdecl'1__ v1 = reduce_107__ goto_decl'1__ v1;
              goto_vars'1__ v1 = state_250__ (reduce_121__ goto_vars2'1__ v1) (reduce_122__ goto_vars'1__ v1);
              goto_vars2'1__ v1 = state_244__ (reduce_119__ goto_signdecl'1__ v1);
              goto_valdef'1__ v1 = reduce_109__ goto_decl'1__ v1;
              goto_exp0b'1__ v1
                  = state_247__ (reduce_120__ goto_signdecl'1__ v1) (reduce_195__ goto_valdef'1__ v1)
                            (reduce_196__ goto_valdef'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_257__ (reduce_123__ goto_vars'1__ v1) (reduce_197__ goto_valdef'1__ v1)
                                    (reduce_198__ goto_valdef'1__ v1)
                                (reduce_390__ goto_aexp'1__ v1)
                            (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Infix v1 -> get >>= reduce_116__ goto_infix'2__ v1;
               KW_InfixL v1 -> get >>= reduce_117__ goto_infix'2__ v1;
               KW_InfixR v1 -> get >>= reduce_118__ goto_infix'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> k_337_1__ t__ }
  
  state_396__ k_210_1__ t__ = case t__ of { KW_As v1 -> get >>= state_397__ (k_210_1__ v1); _ -> frown ["as"] t__ }
  
  state_397__ k_210_2__ t__
      = let {
              goto_qvar'1__ v1 = k_210_2__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               LeftParen v1 -> get >>= state_483__ (reduce_250__ goto_qvar'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_399__ k_211_1__ t__
      = let {
              goto_xExp'1__ v1 = k_211_1__ v1;
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_58__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                            (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_403__ k_214_1__ k_216_1__ k_218_1__ t__
      = let {
              goto_qvarop'1__ v1 = reduce_261__ goto_qop'1__ v1;
              goto_qconop'1__ v1 = reduce_262__ goto_qop'1__ v1;
              goto_qop'1__ v1 = state_405__ (k_216_1__ v1) (k_218_1__ v1);
              goto_qconsym2'2__ v1 v2 = reduce_265__ goto_qconop'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_278__ goto_qconsym2'2__ v1 v2;
              goto_qvarsym'2__ v1 v2 = reduce_263__ goto_qvarop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_281__ goto_qvarsym'2__ v1 v2;
              goto_qvarsym1'2__ v1 v2 = reduce_282__ goto_qvarsym'2__ v1 v2
            }
        in case t__ of {
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               QVarSym v1 v2 -> get >>= reduce_287__ goto_qvarsym1'2__ v1 v2;
               QConSym v1 v2 -> get >>= reduce_279__ goto_qconsym2'2__ v1 v2;
               BackQuote v1 -> get >>= state_514__ (reduce_264__ goto_qvarop'1__ v1) (reduce_266__ goto_qconop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               _ -> k_214_1__ t__ }
  
  state_405__ k_216_2__ k_218_2__ t__
      = let {
              goto_exp10a'1__ v1 = k_216_2__ v1;
              goto_exp10b'1__ v1 = k_218_2__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_406__ k_216_2__ k_218_2__ k_405_3__ t__
      = let {
              goto_exp10a'1__ v1 = k_216_2__ v1;
              goto_exp10b'1__ v1 = k_218_2__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               RightParen v1 -> get >>= k_405_3__ v1;
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               ")",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_409__ k_218_1__ k_235_1__ k_236_1__ t__
      = let {
              goto_ralt'2__ v1 v2 = state_454__ (k_235_1__ v1 v2) (k_236_1__ v1 v2);
              goto_gdpat'1__ v1 = state_765__ (reduce_446__ goto_many1'1_gdpat'1__ v1);
              goto_qvarop'1__ v1 = reduce_261__ goto_qop'1__ v1;
              goto_qconop'1__ v1 = reduce_262__ goto_qop'1__ v1;
              goto_qop'1__ v1 = state_410__ (k_218_1__ v1);
              goto_qconsym2'2__ v1 v2 = reduce_265__ goto_qconop'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_278__ goto_qconsym2'2__ v1 v2;
              goto_qvarsym'2__ v1 v2 = reduce_263__ goto_qvarop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_281__ goto_qvarsym'2__ v1 v2;
              goto_qvarsym1'2__ v1 v2 = reduce_282__ goto_qvarsym'2__ v1 v2;
              goto_many1'1_gdpat'1__ v1 = reduce_238__ goto_ralt'2__ v1
            }
        in case t__ of {
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               QVarSym v1 v2 -> get >>= reduce_287__ goto_qvarsym1'2__ v1 v2;
               QConSym v1 v2 -> get >>= reduce_279__ goto_qconsym2'2__ v1 v2;
               BackQuote v1 -> get >>= state_514__ (reduce_264__ goto_qvarop'1__ v1) (reduce_266__ goto_qconop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Bar v1 -> get >>= state_767__ (reduce_447__ goto_gdpat'1__ v1);
               RightArrow v1 -> get >>= state_457__ (reduce_237__ goto_ralt'2__ v1);
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               _ -> frown ["<varsym>", "<consym>", "<qvarsym>", "<qconsym>", "`", ".", "|", "->", "-", "!"] t__ }
  
  state_410__ k_218_2__ t__
      = let {
              goto_exp10b'1__ v1 = k_218_2__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_412__ k_219_1__ t__
      = let {
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = state_413__ (k_219_1__ v1)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "~",
                               "as",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_413__ k_219_2__ t__ = case t__ of { RightArrow v1 -> get >>= state_414__ (k_219_2__ v1); _ -> frown ["->"] t__ }
  
  state_414__ k_219_3__ t__
      = let {
              goto_xExp'1__ v1 = k_219_3__ v1;
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_58__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                            (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_416__ k_220_1__ t__
      = let {
              goto_decllist'2__ v1 v2 = state_419__ (k_220_1__ v1 v2);
              goto_layout_on'1__ v1 = state_389__ (reduce_205__ goto_decllist'2__ v1)
            }
        in case t__ of {
               LeftCurly v1 -> get >>= state_386__ (reduce_204__ goto_decllist'2__ v1);
               _ -> reduce_290__ goto_layout_on'1__ t__ }
  
  state_417__ k_220_1__ k_241_1__ t__
      = let {
              goto_decllist'2__ v1 v2 = state_420__ (k_220_1__ v1 v2) (k_241_1__ v1 v2);
              goto_layout_on'1__ v1 = state_389__ (reduce_205__ goto_decllist'2__ v1)
            }
        in case t__ of {
               LeftCurly v1 -> get >>= state_386__ (reduce_204__ goto_decllist'2__ v1);
               _ -> reduce_290__ goto_layout_on'1__ t__ }
  
  state_418__ k_220_1__ k_425_1__ t__
      = let {
              goto_decllist'2__ v1 v2 = state_421__ (k_220_1__ v1 v2) (k_425_1__ v1 v2);
              goto_layout_on'1__ v1 = state_389__ (reduce_205__ goto_decllist'2__ v1)
            }
        in case t__ of {
               LeftCurly v1 -> get >>= state_386__ (reduce_204__ goto_decllist'2__ v1);
               _ -> reduce_290__ goto_layout_on'1__ t__ }
  
  state_419__ k_220_2__ t__ = case t__ of { KW_In v1 -> get >>= state_422__ (k_220_2__ v1); _ -> frown ["in"] t__ }
  
  state_420__ k_220_2__ k_241_2__ t__
      = case t__ of {
            SemiColon v1 -> get >>= state_466__ (k_241_2__ v1);
            KW_In v1 -> get >>= state_422__ (k_220_2__ v1);
            _ -> frown [";", "in"] t__ }
  
  state_421__ k_220_2__ k_425_2__ t__ = case t__ of { KW_In v1 -> get >>= state_422__ (k_220_2__ v1); _ -> k_425_2__ t__ }
  
  state_422__ k_220_3__ t__
      = let {
              goto_xExp'1__ v1 = k_220_3__ v1;
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_58__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                            (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_424__ k_221_1__ t__
      = let {
              goto_xExp'1__ v1 = state_425__ (k_221_1__ v1);
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_58__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                            (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_425__ k_221_2__ t__ = case t__ of { KW_Then v1 -> get >>= state_426__ (k_221_2__ v1); _ -> frown ["then"] t__ }
  
  state_426__ k_221_3__ t__
      = let {
              goto_xExp'1__ v1 = state_427__ (k_221_3__ v1);
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_58__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                            (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_427__ k_221_4__ t__ = case t__ of { KW_Else v1 -> get >>= state_428__ (k_221_4__ v1); _ -> frown ["else"] t__ }
  
  state_428__ k_221_5__ t__
      = let {
              goto_xExp'1__ v1 = k_221_5__ v1;
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_58__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                            (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_430__ k_222_1__ t__
      = let {
              goto_xExp'1__ v1 = state_431__ (k_222_1__ v1);
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_58__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                            (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_431__ k_222_2__ t__ = case t__ of { KW_Of v1 -> get >>= state_432__ (k_222_2__ v1); _ -> frown ["of"] t__ }
  
  state_432__ k_222_3__ t__
      = let {
              goto_altslist'2__ v1 v2 = k_222_3__ v1 v2;
              goto_layout_on'1__ v1 = state_447__ (reduce_230__ goto_altslist'2__ v1)
            }
        in case t__ of {
               LeftCurly v1 -> get >>= state_444__ (reduce_229__ goto_altslist'2__ v1);
               _ -> reduce_290__ goto_layout_on'1__ t__ }
  
  state_434__ k_223_1__ t__
      = let {
              goto_fexp'1__ v1 = k_223_1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "~",
                               "as",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_435__ k_223_1__ k_284_1__ t__
      = let {
              goto_fexp'1__ v1 = k_223_1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               RightParen v1 -> k_284_1__ t__;
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               ")",
                               "[",
                               "_",
                               "~",
                               "as",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_437__ k_224_1__ t__
      = let {
              goto_stmtlist'1__ v1 = k_224_1__ v1;
              goto_layout_on_for_do'1__ v1 = state_463__ (reduce_240__ goto_stmtlist'1__ v1)
            }
        in case t__ of {
               LeftCurly v1 -> get >>= state_460__ (reduce_239__ goto_stmtlist'1__ v1);
               _ -> reduce_291__ goto_layout_on_for_do'1__ t__ }
  
  state_441__ k_227_1__ k_429_2__ t__
      = case t__ of { RightParen v1 -> get >>= k_429_2__ v1; Comma v1 -> get >>= k_227_1__ v1; _ -> frown [")", ","] t__ }
  
  state_444__ k_229_1__ t__
      = let {
              goto_semis'1__ v1 = reduce_337__ goto_opt'1_semis'1__ v1;
              goto_alts'1__ v1 = state_445__ (k_229_1__ v1);
              goto_opt'1_semis'1__ v1
                  = state_140__ (reduce_58__ goto_semis'1__ v1) (reduce_231__ goto_alts'1__ v1)
                        (reduce_232__ goto_alts'1__ v1)
            }
        in reduce_336__ goto_opt'1_semis'1__ t__
  
  state_445__ k_229_2__ t__ = case t__ of { RightCurly v1 -> get >>= k_229_2__ v1; _ -> frown ["}"] t__ }
  
  state_447__ k_230_1__ t__
      = let {
              goto_semis'1__ v1 = reduce_337__ goto_opt'1_semis'1__ v1;
              goto_alts'1__ v1 = state_448__ (k_230_1__ v1);
              goto_opt'1_semis'1__ v1
                  = state_140__ (reduce_58__ goto_semis'1__ v1) (reduce_231__ goto_alts'1__ v1)
                        (reduce_232__ goto_alts'1__ v1)
            }
        in reduce_336__ goto_opt'1_semis'1__ t__
  
  state_448__ k_230_2__ t__
      = let {
              goto_close'1__ v1 = k_230_2__ v1
            }
        in case t__ of { VRightCurly -> get >>= reduce_288__ goto_close'1__; _ -> reduce_289__ goto_close'1__ t__ }
  
  state_450__ k_231_2__ k_234_1__ t__
      = let {
              goto_semis'1__ v1 = state_452__ (k_234_1__ v1) (reduce_337__ goto_opt'1_semis'1__ v1);
              goto_opt'1_semis'1__ v1 = state_141__ (reduce_58__ goto_semis'1__ v1) (k_231_2__ v1)
            }
        in reduce_336__ goto_opt'1_semis'1__ t__
  
  state_452__ k_234_2__ k_337_1__ t__
      = let {
              goto_exp0b'1__ v1
                  = state_409__ (reduce_218__ goto_exp0b'1__ v1) (reduce_235__ goto_alt'1__ v1)
                        (reduce_236__ goto_alt'1__ v1);
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_alt'1__ v1 = k_234_2__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> k_337_1__ t__ }
  
  state_454__ k_235_2__ k_236_2__ t__
      = case t__ of { KW_Where v1 -> get >>= state_455__ (k_236_2__ v1); _ -> k_235_2__ t__ }
  
  state_455__ k_236_3__ t__
      = let {
              goto_decllist'2__ v1 v2 = k_236_3__ v1 v2;
              goto_layout_on'1__ v1 = state_389__ (reduce_205__ goto_decllist'2__ v1)
            }
        in case t__ of {
               LeftCurly v1 -> get >>= state_386__ (reduce_204__ goto_decllist'2__ v1);
               _ -> reduce_290__ goto_layout_on'1__ t__ }
  
  state_457__ k_237_1__ t__
      = let {
              goto_xExp'1__ v1 = k_237_1__ v1;
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_58__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                            (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_460__ k_239_1__ t__
      = let {
              goto_xExp'1__ v1
                  = state_472__ (reduce_243__ goto_stmts'1__ v1) (reduce_245__ goto_stmts'1__ v1)
                        (reduce_246__ goto_stmts'1__ v1);
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_59__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                                (reduce_216__ goto_exp0a'1__ v1)
                            (reduce_218__ goto_exp0b'1__ v1)
                        (reduce_242__ goto_stmts'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_stmts'1__ v1 = state_461__ (k_239_1__ v1);
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               SemiColon v1 -> get >>= state_475__ (reduce_244__ goto_stmts'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_417__ (reduce_220__ goto_exp10a'1__ v1) (reduce_241__ goto_stmts'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               ";",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_461__ k_239_2__ t__ = case t__ of { RightCurly v1 -> get >>= k_239_2__ v1; _ -> frown ["}"] t__ }
  
  state_463__ k_240_1__ t__
      = let {
              goto_xExp'1__ v1
                  = state_472__ (reduce_243__ goto_stmts'1__ v1) (reduce_245__ goto_stmts'1__ v1)
                        (reduce_246__ goto_stmts'1__ v1);
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_59__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                                (reduce_216__ goto_exp0a'1__ v1)
                            (reduce_218__ goto_exp0b'1__ v1)
                        (reduce_242__ goto_stmts'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_stmts'1__ v1 = state_464__ (k_240_1__ v1);
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               SemiColon v1 -> get >>= state_475__ (reduce_244__ goto_stmts'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_417__ (reduce_220__ goto_exp10a'1__ v1) (reduce_241__ goto_stmts'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               ";",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_464__ k_240_2__ t__
      = let {
              goto_close'1__ v1 = k_240_2__ v1
            }
        in case t__ of { VRightCurly -> get >>= reduce_288__ goto_close'1__; _ -> reduce_289__ goto_close'1__ t__ }
  
  state_466__ k_241_3__ t__
      = let {
              goto_xExp'1__ v1
                  = state_472__ (reduce_243__ goto_stmts'1__ v1) (reduce_245__ goto_stmts'1__ v1)
                        (reduce_246__ goto_stmts'1__ v1);
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_59__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                                (reduce_216__ goto_exp0a'1__ v1)
                            (reduce_218__ goto_exp0b'1__ v1)
                        (reduce_242__ goto_stmts'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_stmts'1__ v1 = k_241_3__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               SemiColon v1 -> get >>= state_475__ (reduce_244__ goto_stmts'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_417__ (reduce_220__ goto_exp10a'1__ v1) (reduce_241__ goto_stmts'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               ";",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_468__ k_242_2__ t__
      = let {
              goto_xExp'1__ v1 = state_469__ (k_242_2__ v1);
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_58__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                            (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_469__ k_242_3__ t__ = case t__ of { SemiColon v1 -> get >>= state_470__ (k_242_3__ v1); _ -> frown [";"] t__ }
  
  state_470__ k_242_4__ t__
      = let {
              goto_xExp'1__ v1
                  = state_472__ (reduce_243__ goto_stmts'1__ v1) (reduce_245__ goto_stmts'1__ v1)
                        (reduce_246__ goto_stmts'1__ v1);
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_59__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                                (reduce_216__ goto_exp0a'1__ v1)
                            (reduce_218__ goto_exp0b'1__ v1)
                        (reduce_242__ goto_stmts'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_stmts'1__ v1 = k_242_4__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               SemiColon v1 -> get >>= state_475__ (reduce_244__ goto_stmts'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_417__ (reduce_220__ goto_exp10a'1__ v1) (reduce_241__ goto_stmts'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               ";",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_472__ k_243_1__ k_245_1__ k_246_1__ t__
      = case t__ of { SemiColon v1 -> get >>= state_473__ (k_243_1__ v1) (k_245_1__ v1); _ -> k_246_1__ t__ }
  
  state_473__ k_243_2__ k_245_2__ t__
      = let {
              goto_xExp'1__ v1
                  = state_472__ (reduce_243__ goto_stmts'1__ v1) (reduce_245__ goto_stmts'1__ v1)
                        (reduce_246__ goto_stmts'1__ v1);
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_59__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                                (reduce_216__ goto_exp0a'1__ v1)
                            (reduce_218__ goto_exp0b'1__ v1)
                        (reduce_242__ goto_stmts'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_stmts'1__ v1 = k_243_2__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               SemiColon v1 -> get >>= state_475__ (reduce_244__ goto_stmts'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_417__ (reduce_220__ goto_exp10a'1__ v1) (reduce_241__ goto_stmts'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> k_245_2__ t__ }
  
  state_475__ k_244_1__ t__
      = let {
              goto_xExp'1__ v1
                  = state_472__ (reduce_243__ goto_stmts'1__ v1) (reduce_245__ goto_stmts'1__ v1)
                        (reduce_246__ goto_stmts'1__ v1);
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_59__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                                (reduce_216__ goto_exp0a'1__ v1)
                            (reduce_218__ goto_exp0b'1__ v1)
                        (reduce_242__ goto_stmts'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_stmts'1__ v1 = k_244_1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               SemiColon v1 -> get >>= state_475__ (reduce_244__ goto_stmts'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_417__ (reduce_220__ goto_exp10a'1__ v1) (reduce_241__ goto_stmts'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               ";",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_478__ k_248_1__ t__
      = let {
              goto_varsym'2__ v1 v2 = state_480__ (k_248_1__ v1 v2)
            }
        in case t__ of {
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               _ -> frown ["<varsym>", ".", "-", "!"] t__ }
  
  state_479__ k_248_1__ k_252_1__ t__
      = let {
              goto_consym'2__ v1 v2 = state_493__ (k_252_1__ v1 v2);
              goto_varsym'2__ v1 v2 = state_480__ (k_248_1__ v1 v2)
            }
        in case t__ of {
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               _ -> frown ["<varsym>", "<consym>", ".", "-", "!"] t__ }
  
  state_480__ k_248_2__ t__ = case t__ of { RightParen v1 -> get >>= k_248_2__ v1; _ -> frown [")"] t__ }
  
  state_483__ k_250_1__ t__
      = let {
              goto_qvarsym'2__ v1 v2 = state_488__ (k_250_1__ v1 v2);
              goto_varsym'2__ v1 v2 = reduce_281__ goto_qvarsym'2__ v1 v2;
              goto_qvarsym1'2__ v1 v2 = reduce_282__ goto_qvarsym'2__ v1 v2
            }
        in case t__ of {
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               QVarSym v1 v2 -> get >>= reduce_287__ goto_qvarsym1'2__ v1 v2;
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               _ -> frown ["<varsym>", "<qvarsym>", ".", "-", "!"] t__ }
  
  state_484__ k_250_1__ k_254_1__ t__
      = let {
              goto_qconsym2'2__ v1 v2 = state_498__ (k_254_1__ v1 v2);
              goto_consym'2__ v1 v2 = reduce_278__ goto_qconsym2'2__ v1 v2;
              goto_qvarsym'2__ v1 v2 = state_488__ (k_250_1__ v1 v2);
              goto_varsym'2__ v1 v2 = reduce_281__ goto_qvarsym'2__ v1 v2;
              goto_qvarsym1'2__ v1 v2 = reduce_282__ goto_qvarsym'2__ v1 v2
            }
        in case t__ of {
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               QVarSym v1 v2 -> get >>= reduce_287__ goto_qvarsym1'2__ v1 v2;
               QConSym v1 v2 -> get >>= reduce_279__ goto_qconsym2'2__ v1 v2;
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               _ -> frown ["<varsym>", "<consym>", "<qvarsym>", "<qconsym>", ".", "-", "!"] t__ }
  
  state_485__ k_250_1__ k_254_1__ k_373_1__ t__
      = let {
              goto_qcname'1__ v1 = state_667__ (k_373_1__ v1);
              goto_qvar'1__ v1 = reduce_59__ goto_qcname'1__ v1;
              goto_qcon'1__ v1 = reduce_60__ goto_qcname'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_qconsym2'2__ v1 v2 = state_498__ (k_254_1__ v1 v2);
              goto_consym'2__ v1 v2 = reduce_278__ goto_qconsym2'2__ v1 v2;
              goto_qvarsym'2__ v1 v2 = state_488__ (k_250_1__ v1 v2);
              goto_varsym'2__ v1 v2 = reduce_281__ goto_qvarsym'2__ v1 v2;
              goto_qvarsym1'2__ v1 v2 = reduce_282__ goto_qvarsym'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               QVarSym v1 v2 -> get >>= reduce_287__ goto_qvarsym1'2__ v1 v2;
               QConSym v1 v2 -> get >>= reduce_279__ goto_qconsym2'2__ v1 v2;
               LeftParen v1 -> get >>= state_484__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<varsym>",
                               "<consym>",
                               "<qvarsym>",
                               "<qconsym>",
                               "(",
                               ".",
                               "-",
                               "!",
                               "as",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_486__ k_250_1__ k_254_1__ k_403_1__ k_405_1__ k_406_1__ k_427_1__ k_429_1__ t__
      = let {
              goto_xExp'1__ v1 = reduce_409__ goto_mSepBy1''1_xExp'1_21__ v1;
              goto_mSepBy1'1_xExp'1_21__ v1 = state_713__ (k_403_1__ v1);
              goto_mSepBy1''1_xExp'1_21__ v1
                  = state_723__ (reduce_408__ goto_mSepBy1'1_xExp'1_21__ v1)
                        (reduce_410__ goto_mSepBy1''1_xExp'1_21__ v1);
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_60__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                                (reduce_216__ goto_exp0a'1__ v1)
                            (reduce_218__ goto_exp0b'1__ v1)
                        (k_405_1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_commas'1__ v1 = state_441__ (reduce_227__ goto_commas'1__ v1) (k_429_1__ v1);
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvaropm'1__ v1 = reduce_431__ goto_qopm'1__ v1;
              goto_qconop'1__ v1 = reduce_432__ goto_qopm'1__ v1;
              goto_qopm'1__ v1 = state_719__ (k_406_1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_qconsym2'2__ v1 v2 = state_499__ (k_254_1__ v1 v2) (reduce_265__ goto_qconop'1__ v1 v2);
              goto_consym'2__ v1 v2 = reduce_278__ goto_qconsym2'2__ v1 v2;
              goto_qvarsym'2__ v1 v2 = state_488__ (k_250_1__ v1 v2);
              goto_qvarsymm'1__ v1 = reduce_433__ goto_qvaropm'1__ v1;
              goto_varsym'2__ v1 v2 = reduce_281__ goto_qvarsym'2__ v1 v2;
              goto_varsymm'1__ v1 = reduce_435__ goto_qvarsymm'1__ v1;
              goto_qvarsym1'2__ v1 v2
                  = state_538__ (reduce_282__ goto_qvarsym'2__ v1 v2) (reduce_436__ goto_qvarsymm'1__ v1 v2);
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               VarSym v1 v2
                   -> get >>= state_540__ (reduce_283__ goto_varsym'2__ v1 v2) (reduce_437__ goto_varsymm'1__ v1 v2);
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               QVarSym v1 v2 -> get >>= reduce_287__ goto_qvarsym1'2__ v1 v2;
               QConSym v1 v2 -> get >>= reduce_279__ goto_qconsym2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               RightParen v1 -> get >>= k_427_1__ v1;
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Comma v1 -> get >>= reduce_228__ goto_commas'1__ v1;
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               BackQuote v1 -> get >>= state_518__ (reduce_266__ goto_qconop'1__ v1) (reduce_434__ goto_qvaropm'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_435__ (reduce_223__ goto_exp10b'1__ v1) (reduce_284__ goto_varsym'2__ v1);
               Exclamation v1 -> get >>= state_543__ (reduce_285__ goto_varsym'2__ v1) (reduce_438__ goto_varsymm'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<varsym>",
                               "<consym>",
                               "<qvarsym>",
                               "<qconsym>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               ")",
                               "[",
                               ",",
                               "_",
                               "`",
                               ".",
                               "\\",
                               "~",
                               "-",
                               "!",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_487__ k_250_1__ k_377_1__ t__
      = let {
              goto_qvar'1__ v1 = state_673__ (k_377_1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qvarsym'2__ v1 v2 = state_488__ (k_250_1__ v1 v2);
              goto_varsym'2__ v1 v2 = reduce_281__ goto_qvarsym'2__ v1 v2;
              goto_qvarsym1'2__ v1 v2 = reduce_282__ goto_qvarsym'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               QVarSym v1 v2 -> get >>= reduce_287__ goto_qvarsym1'2__ v1 v2;
               LeftParen v1 -> get >>= state_483__ (reduce_250__ goto_qvar'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<varsym>",
                               "<qvarsym>",
                               "(",
                               ".",
                               "-",
                               "!",
                               "as",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_488__ k_250_2__ t__ = case t__ of { RightParen v1 -> get >>= k_250_2__ v1; _ -> frown [")"] t__ }
  
  state_491__ k_251_1__ k_274_1__ t__ = case t__ of { LeftCurly v1 -> k_251_1__ t__; _ -> k_274_1__ t__ }
  
  state_492__ k_252_1__ t__
      = let {
              goto_consym'2__ v1 v2 = state_493__ (k_252_1__ v1 v2)
            }
        in case t__ of { ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2; _ -> frown ["<consym>"] t__ }
  
  state_493__ k_252_2__ t__ = case t__ of { RightParen v1 -> get >>= k_252_2__ v1; _ -> frown [")"] t__ }
  
  state_494__ k_252_2__ k_278_1__ t__ = case t__ of { RightParen v1 -> get >>= k_252_2__ v1; _ -> frown [")"] t__ }
  
  state_497__ k_254_1__ t__
      = let {
              goto_qconsym2'2__ v1 v2 = state_498__ (k_254_1__ v1 v2);
              goto_consym'2__ v1 v2 = reduce_278__ goto_qconsym2'2__ v1 v2
            }
        in case t__ of {
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               QConSym v1 v2 -> get >>= reduce_279__ goto_qconsym2'2__ v1 v2;
               _ -> frown ["<consym>", "<qconsym>"] t__ }
  
  state_498__ k_254_2__ t__ = case t__ of { RightParen v1 -> get >>= k_254_2__ v1; _ -> frown [")"] t__ }
  
  state_499__ k_254_2__ k_265_1__ t__ = case t__ of { RightParen v1 -> get >>= k_254_2__ v1; _ -> k_265_1__ t__ }
  
  state_502__ k_256_1__ t__
      = let {
              goto_conid'2__ v1 v2 = state_504__ (k_256_1__ v1 v2)
            }
        in case t__ of { ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2; _ -> frown ["<conid>"] t__ }
  
  state_503__ k_256_1__ k_260_1__ t__
      = let {
              goto_varid'2__ v1 v2 = state_509__ (k_260_1__ v1 v2);
              goto_conid'2__ v1 v2 = state_504__ (k_256_1__ v1 v2)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<conid>", "as", "hiding", "qualified"] t__ }
  
  state_504__ k_256_2__ t__ = case t__ of { BackQuote v1 -> get >>= k_256_2__ v1; _ -> frown ["`"] t__ }
  
  state_509__ k_260_2__ t__ = case t__ of { BackQuote v1 -> get >>= k_260_2__ v1; _ -> frown ["`"] t__ }
  
  state_514__ k_264_1__ k_266_1__ t__
      = let {
              goto_qvarid'2__ v1 v2 = state_515__ (k_264_1__ v1 v2);
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid2'2__ v1 v2 = state_519__ (k_266_1__ v1 v2);
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "as", "hiding", "qualified"] t__ }
  
  state_515__ k_264_2__ t__ = case t__ of { BackQuote v1 -> get >>= k_264_2__ v1; _ -> frown ["`"] t__ }
  
  state_518__ k_266_1__ k_434_1__ t__
      = let {
              goto_qvarid'2__ v1 v2 = state_756__ (k_434_1__ v1 v2);
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid2'2__ v1 v2 = state_519__ (k_266_1__ v1 v2);
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "as", "hiding", "qualified"] t__ }
  
  state_519__ k_266_2__ t__ = case t__ of { BackQuote v1 -> get >>= k_266_2__ v1; _ -> frown ["`"] t__ }
  
  state_522__ k_267_1__ k_297_1__ t__ = case t__ of { LeftTyParen v1 -> k_267_1__ t__; _ -> k_297_1__ t__ }
  
  state_538__ k_282_1__ k_436_1__ t__ = case t__ of { RightParen v1 -> k_282_1__ t__; _ -> k_436_1__ t__ }
  
  state_540__ k_283_1__ k_437_1__ t__ = case t__ of { RightParen v1 -> k_283_1__ t__; _ -> k_437_1__ t__ }
  
  state_543__ k_285_1__ k_438_1__ t__ = case t__ of { RightParen v1 -> k_285_1__ t__; _ -> k_438_1__ t__ }
  
  state_554__ k_298_1__ t__
      = let {
              goto_varop'1__ v1 = reduce_257__ goto_op'1__ v1;
              goto_conop'1__ v1 = reduce_258__ goto_op'1__ v1;
              goto_op'1__ v1 = reduce_319__ goto_kindarg'1__ v1;
              goto_varid'2__ v1 v2 = reduce_326__ goto_kivar'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_255__ goto_conop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_259__ goto_varop'1__ v1 v2;
              goto_kindarg'1__ v1 = state_555__ (k_298_1__ v1);
              goto_kivar'1__ v1 = state_597__ (reduce_320__ goto_kindarg'1__ v1)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               BackQuote v1 -> get >>= state_503__ (reduce_256__ goto_conop'1__ v1) (reduce_260__ goto_varop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<varsym>", "<consym>", "`", ".", "-", "!", "as", "hiding", "qualified"] t__ }
  
  state_555__ k_298_2__ t__ = case t__ of { RightKiParen v1 -> get >>= k_298_2__ v1; _ -> frown ["]}"] t__ }
  
  state_557__ k_299_1__ t__
      = let {
              goto_xType'1__ v1 = reduce_327__ goto_typeorconsym'1__ v1;
              goto_btype'1__ v1
                  = state_258__ (reduce_124__ goto_xType'1__ v1) (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_qconsym'1__ v1 = reduce_328__ goto_typeorconsym'1__ v1;
              goto_qconsym2'2__ v1 v2 = reduce_277__ goto_qconsym'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_278__ goto_qconsym2'2__ v1 v2;
              goto_tyvar'1__ v1 = reduce_130__ goto_atype'1__ v1;
              goto_typeorconsym'1__ v1 = reduce_331__ goto_typearg'1__ v1;
              goto_typearg'1__ v1 = state_558__ (k_299_1__ v1)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               QConSym v1 v2 -> get >>= reduce_279__ goto_qconsym2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               DoubleColon v1 -> get >>= state_613__ (reduce_330__ goto_typeorconsym'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_611__ (reduce_329__ goto_typeorconsym'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<consym>",
                               "<qconsym>",
                               "(",
                               "[",
                               "::",
                               "as",
                               "case",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_558__ k_299_2__ t__ = case t__ of { RightTyParen v1 -> get >>= k_299_2__ v1; _ -> frown ["|}"] t__ }
  
  state_560__ k_300_1__ t__
      = let {
              goto_varop'1__ v1 = reduce_257__ goto_op'1__ v1;
              goto_conop'1__ v1 = reduce_258__ goto_op'1__ v1;
              goto_op'1__ v1 = reduce_323__ goto_kindterm'1__ v1;
              goto_varid'2__ v1 v2 = reduce_326__ goto_kivar'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_255__ goto_conop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_259__ goto_varop'1__ v1 v2;
              goto_kind'1__ v1 = state_561__ (k_300_1__ v1);
              goto_kindterm'1__ v1 = state_600__ (reduce_321__ goto_kind'1__ v1) (reduce_322__ goto_kind'1__ v1);
              goto_kivar'1__ v1 = reduce_324__ goto_kindterm'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               LeftParen v1 -> get >>= state_605__ (reduce_325__ goto_kindterm'1__ v1);
               BackQuote v1 -> get >>= state_503__ (reduce_256__ goto_conop'1__ v1) (reduce_260__ goto_varop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<varsym>", "<consym>", "(", "`", ".", "-", "!", "as", "hiding", "qualified"] t__ }
  
  state_561__ k_300_2__ t__ = case t__ of { RightKiParen v1 -> get >>= k_300_2__ v1; _ -> frown ["]}"] t__ }
  
  state_563__ k_301_1__ k_302_1__ k_303_1__ k_304_1__ k_305_1__ t__
      = let {
              goto_xType'1__ v1 = reduce_327__ goto_typeorconsym'1__ v1;
              goto_btype'1__ v1
                  = state_258__ (reduce_124__ goto_xType'1__ v1) (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_varkindannottyvar'1__ v1 = state_566__ (k_302_1__ v1) (k_303_1__ v1) (k_305_1__ v1);
              goto_nooptkindannottyvar'1__ v1 = state_573__ (k_304_1__ v1);
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_qconsym'1__ v1 = reduce_328__ goto_typeorconsym'1__ v1;
              goto_qconsym2'2__ v1 v2 = reduce_277__ goto_qconsym'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_278__ goto_qconsym2'2__ v1 v2;
              goto_tyvar'1__ v1
                  = state_271__ (reduce_130__ goto_atype'1__ v1) (reduce_314__ goto_varkindannottyvar'1__ v1)
                            (reduce_315__ goto_varkindannottyvar'1__ v1)
                        (reduce_316__ goto_nooptkindannottyvar'1__ v1);
              goto_typeorconsym'1__ v1 = state_564__ (k_301_1__ v1)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               QConSym v1 v2 -> get >>= reduce_279__ goto_qconsym2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               DoubleColon v1 -> get >>= state_613__ (reduce_330__ goto_typeorconsym'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_611__ (reduce_329__ goto_typeorconsym'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<consym>",
                               "<qconsym>",
                               "(",
                               "[",
                               "::",
                               "as",
                               "case",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_564__ k_301_2__ t__ = case t__ of { RightTyParen v1 -> get >>= k_301_2__ v1; _ -> frown ["|}"] t__ }
  
  state_566__ k_302_2__ k_303_2__ k_305_2__ t__
      = case t__ of {
            Comma v1 -> get >>= state_567__ (k_302_2__ v1) (k_303_2__ v1);
            Bar v1 -> get >>= state_575__ (k_305_2__ v1);
            _ -> frown [",", "|"] t__ }
  
  state_567__ k_302_3__ k_303_3__ t__
      = let {
              goto_mSepBy1'1_varkindannottyvar'1_21__ v1 = state_568__ (k_302_3__ v1) (k_303_3__ v1);
              goto_mSepBy1''1_varkindannottyvar'1_21__ v1
                  = state_579__ (reduce_308__ goto_mSepBy1'1_varkindannottyvar'1_21__ v1)
                        (reduce_311__ goto_mSepBy1''1_varkindannottyvar'1_21__ v1);
              goto_varkindannottyvar'1__ v1 = reduce_310__ goto_mSepBy1''1_varkindannottyvar'1_21__ v1;
              goto_varid'2__ v1 v2 = reduce_297__ goto_tyvar'1__ v1 v2;
              goto_tyvar'1__ v1
                  = state_587__ (reduce_314__ goto_varkindannottyvar'1__ v1) (reduce_315__ goto_varkindannottyvar'1__ v1)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "as", "hiding", "qualified"] t__ }
  
  state_568__ k_302_4__ k_303_4__ t__
      = case t__ of {
            RightTyParen v1 -> get >>= k_302_4__ v1;
            Bar v1 -> get >>= state_570__ (k_303_4__ v1);
            _ -> frown ["|}", "|"] t__ }
  
  state_570__ k_303_5__ t__
      = let {
              goto_mSepBy'1_zType'1_21__ v1 = state_571__ (k_303_5__ v1);
              goto_mSepBy1'1_zType'1_21__ v1 = reduce_307__ goto_mSepBy'1_zType'1_21__ v1;
              goto_mSepBy1''1_zType'1_21__ v1
                  = state_580__ (reduce_309__ goto_mSepBy1'1_zType'1_21__ v1)
                        (reduce_313__ goto_mSepBy1''1_zType'1_21__ v1);
              goto_xType'1__ v1 = reduce_317__ goto_zType'1__ v1;
              goto_btype'1__ v1
                  = state_258__ (reduce_124__ goto_xType'1__ v1) (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_nooptkindannottyvar'1__ v1 = reduce_318__ goto_zType'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1
                  = state_272__ (reduce_130__ goto_atype'1__ v1) (reduce_316__ goto_nooptkindannottyvar'1__ v1);
              goto_zType'1__ v1 = reduce_312__ goto_mSepBy1''1_zType'1_21__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               RightTyParen v1 -> reduce_306__ goto_mSepBy'1_zType'1_21__ t__;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "[", "|}", "as", "hiding", "qualified"]
                          t__ }
  
  state_571__ k_303_6__ t__ = case t__ of { RightTyParen v1 -> get >>= k_303_6__ v1; _ -> frown ["|}"] t__ }
  
  state_573__ k_304_2__ t__ = case t__ of { RightTyParen v1 -> get >>= k_304_2__ v1; _ -> frown ["|}"] t__ }
  
  state_575__ k_305_3__ t__
      = let {
              goto_mSepBy'1_zType'1_21__ v1 = state_576__ (k_305_3__ v1);
              goto_mSepBy1'1_zType'1_21__ v1 = reduce_307__ goto_mSepBy'1_zType'1_21__ v1;
              goto_mSepBy1''1_zType'1_21__ v1
                  = state_580__ (reduce_309__ goto_mSepBy1'1_zType'1_21__ v1)
                        (reduce_313__ goto_mSepBy1''1_zType'1_21__ v1);
              goto_xType'1__ v1 = reduce_317__ goto_zType'1__ v1;
              goto_btype'1__ v1
                  = state_258__ (reduce_124__ goto_xType'1__ v1) (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_nooptkindannottyvar'1__ v1 = reduce_318__ goto_zType'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1
                  = state_272__ (reduce_130__ goto_atype'1__ v1) (reduce_316__ goto_nooptkindannottyvar'1__ v1);
              goto_zType'1__ v1 = reduce_312__ goto_mSepBy1''1_zType'1_21__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               RightTyParen v1 -> reduce_306__ goto_mSepBy'1_zType'1_21__ t__;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "[", "|}", "as", "hiding", "qualified"]
                          t__ }
  
  state_576__ k_305_4__ t__ = case t__ of { RightTyParen v1 -> get >>= k_305_4__ v1; _ -> frown ["|}"] t__ }
  
  state_579__ k_308_1__ k_311_1__ t__ = case t__ of { Comma v1 -> get >>= state_582__ (k_311_1__ v1); _ -> k_308_1__ t__ }
  
  state_580__ k_309_1__ k_313_1__ t__
      = case t__ of {
            Comma v1 -> get >>= state_585__ (k_313_1__ v1);
            RightTyParen v1 -> k_309_1__ t__;
            _ -> frown [",", "|}"] t__ }
  
  state_582__ k_311_2__ t__
      = let {
              goto_varkindannottyvar'1__ v1 = k_311_2__ v1;
              goto_varid'2__ v1 v2 = reduce_297__ goto_tyvar'1__ v1 v2;
              goto_tyvar'1__ v1
                  = state_587__ (reduce_314__ goto_varkindannottyvar'1__ v1) (reduce_315__ goto_varkindannottyvar'1__ v1)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "as", "hiding", "qualified"] t__ }
  
  state_585__ k_313_2__ t__
      = let {
              goto_xType'1__ v1 = reduce_317__ goto_zType'1__ v1;
              goto_btype'1__ v1
                  = state_258__ (reduce_124__ goto_xType'1__ v1) (reduce_125__ goto_xType'1__ v1)
                            (reduce_126__ goto_xType'1__ v1)
                        (reduce_127__ goto_btype'1__ v1);
              goto_atype'1__ v1 = reduce_128__ goto_btype'1__ v1;
              goto_gtycon'1__ v1 = reduce_129__ goto_atype'1__ v1;
              goto_nooptkindannottyvar'1__ v1 = reduce_318__ goto_zType'1__ v1;
              goto_qvar'1__ v1 = state_294__ (reduce_142__ goto_gtycon'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = state_522__ (reduce_267__ goto_qvarid'2__ v1 v2) (reduce_297__ goto_tyvar'1__ v1 v2);
              goto_qconid'1__ v1
                  = state_285__ (reduce_135__ goto_gtycon'1__ v1) (reduce_140__ goto_gtycon'1__ v1)
                        (reduce_141__ goto_gtycon'1__ v1);
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_tyvar'1__ v1
                  = state_272__ (reduce_130__ goto_atype'1__ v1) (reduce_316__ goto_nooptkindannottyvar'1__ v1);
              goto_zType'1__ v1 = k_313_2__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_273__ (reduce_131__ goto_atype'1__ v1) (reduce_132__ goto_atype'1__ v1)
                                                  (reduce_134__ goto_atype'1__ v1)
                                              (reduce_136__ goto_gtycon'1__ v1)
                                          (reduce_137__ goto_gtycon'1__ v1)
                                      (reduce_139__ goto_gtycon'1__ v1)
                                  (reduce_143__ goto_gtycon'1__ v1)
                              (reduce_250__ goto_qvar'1__ v1);
               LeftSquare v1 -> get >>= state_280__ (reduce_133__ goto_atype'1__ v1) (reduce_138__ goto_gtycon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "[", "as", "hiding", "qualified"] t__ }
  
  state_587__ k_314_1__ k_315_1__ t__
      = case t__ of { DoubleColon v1 -> get >>= state_588__ (k_315_1__ v1); _ -> k_314_1__ t__ }
  
  state_588__ k_315_2__ t__
      = let {
              goto_varop'1__ v1 = reduce_257__ goto_op'1__ v1;
              goto_conop'1__ v1 = reduce_258__ goto_op'1__ v1;
              goto_op'1__ v1 = reduce_323__ goto_kindterm'1__ v1;
              goto_varid'2__ v1 v2 = reduce_326__ goto_kivar'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_255__ goto_conop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_259__ goto_varop'1__ v1 v2;
              goto_kind'1__ v1 = k_315_2__ v1;
              goto_kindterm'1__ v1 = state_600__ (reduce_321__ goto_kind'1__ v1) (reduce_322__ goto_kind'1__ v1);
              goto_kivar'1__ v1 = reduce_324__ goto_kindterm'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               LeftParen v1 -> get >>= state_605__ (reduce_325__ goto_kindterm'1__ v1);
               BackQuote v1 -> get >>= state_503__ (reduce_256__ goto_conop'1__ v1) (reduce_260__ goto_varop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<varsym>", "<consym>", "(", "`", ".", "-", "!", "as", "hiding", "qualified"] t__ }
  
  state_589__ k_315_2__ k_316_2__ t__
      = let {
              goto_varop'1__ v1 = reduce_257__ goto_op'1__ v1;
              goto_conop'1__ v1 = reduce_258__ goto_op'1__ v1;
              goto_op'1__ v1 = reduce_323__ goto_kindterm'1__ v1;
              goto_varid'2__ v1 v2 = reduce_326__ goto_kivar'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_255__ goto_conop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_259__ goto_varop'1__ v1 v2;
              goto_kind'1__ v1 = state_591__ (k_315_2__ v1) (k_316_2__ v1);
              goto_kindterm'1__ v1 = state_600__ (reduce_321__ goto_kind'1__ v1) (reduce_322__ goto_kind'1__ v1);
              goto_kivar'1__ v1 = reduce_324__ goto_kindterm'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               LeftParen v1 -> get >>= state_605__ (reduce_325__ goto_kindterm'1__ v1);
               BackQuote v1 -> get >>= state_503__ (reduce_256__ goto_conop'1__ v1) (reduce_260__ goto_varop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<varsym>", "<consym>", "(", "`", ".", "-", "!", "as", "hiding", "qualified"] t__ }
  
  state_591__ k_315_3__ k_316_3__ t__ = case t__ of { RightTyParen v1 -> k_316_3__ t__; _ -> k_315_3__ t__ }
  
  state_592__ k_316_2__ t__
      = let {
              goto_varop'1__ v1 = reduce_257__ goto_op'1__ v1;
              goto_conop'1__ v1 = reduce_258__ goto_op'1__ v1;
              goto_op'1__ v1 = reduce_323__ goto_kindterm'1__ v1;
              goto_varid'2__ v1 v2 = reduce_326__ goto_kivar'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_255__ goto_conop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_259__ goto_varop'1__ v1 v2;
              goto_kind'1__ v1 = k_316_2__ v1;
              goto_kindterm'1__ v1 = state_600__ (reduce_321__ goto_kind'1__ v1) (reduce_322__ goto_kind'1__ v1);
              goto_kivar'1__ v1 = reduce_324__ goto_kindterm'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               LeftParen v1 -> get >>= state_605__ (reduce_325__ goto_kindterm'1__ v1);
               BackQuote v1 -> get >>= state_503__ (reduce_256__ goto_conop'1__ v1) (reduce_260__ goto_varop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<varsym>", "<consym>", "(", "`", ".", "-", "!", "as", "hiding", "qualified"] t__ }
  
  state_597__ k_320_1__ t__ = case t__ of { RightArrow v1 -> get >>= state_598__ (k_320_1__ v1); _ -> frown ["->"] t__ }
  
  state_598__ k_320_2__ t__
      = let {
              goto_varid'2__ v1 v2 = reduce_326__ goto_kivar'1__ v1 v2;
              goto_kivar'1__ v1 = k_320_2__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "as", "hiding", "qualified"] t__ }
  
  state_600__ k_321_1__ k_322_1__ t__
      = case t__ of { RightArrow v1 -> get >>= state_601__ (k_321_1__ v1); _ -> k_322_1__ t__ }
  
  state_601__ k_321_2__ t__
      = let {
              goto_varop'1__ v1 = reduce_257__ goto_op'1__ v1;
              goto_conop'1__ v1 = reduce_258__ goto_op'1__ v1;
              goto_op'1__ v1 = reduce_323__ goto_kindterm'1__ v1;
              goto_varid'2__ v1 v2 = reduce_326__ goto_kivar'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_255__ goto_conop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_259__ goto_varop'1__ v1 v2;
              goto_kind'1__ v1 = k_321_2__ v1;
              goto_kindterm'1__ v1 = state_600__ (reduce_321__ goto_kind'1__ v1) (reduce_322__ goto_kind'1__ v1);
              goto_kivar'1__ v1 = reduce_324__ goto_kindterm'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               LeftParen v1 -> get >>= state_605__ (reduce_325__ goto_kindterm'1__ v1);
               BackQuote v1 -> get >>= state_503__ (reduce_256__ goto_conop'1__ v1) (reduce_260__ goto_varop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<varsym>", "<consym>", "(", "`", ".", "-", "!", "as", "hiding", "qualified"] t__ }
  
  state_605__ k_325_1__ t__
      = let {
              goto_varop'1__ v1 = reduce_257__ goto_op'1__ v1;
              goto_conop'1__ v1 = reduce_258__ goto_op'1__ v1;
              goto_op'1__ v1 = reduce_323__ goto_kindterm'1__ v1;
              goto_varid'2__ v1 v2 = reduce_326__ goto_kivar'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_255__ goto_conop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_259__ goto_varop'1__ v1 v2;
              goto_kind'1__ v1 = state_606__ (k_325_1__ v1);
              goto_kindterm'1__ v1 = state_600__ (reduce_321__ goto_kind'1__ v1) (reduce_322__ goto_kind'1__ v1);
              goto_kivar'1__ v1 = reduce_324__ goto_kindterm'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               LeftParen v1 -> get >>= state_605__ (reduce_325__ goto_kindterm'1__ v1);
               BackQuote v1 -> get >>= state_503__ (reduce_256__ goto_conop'1__ v1) (reduce_260__ goto_varop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<varsym>", "<consym>", "(", "`", ".", "-", "!", "as", "hiding", "qualified"] t__ }
  
  state_606__ k_325_2__ t__ = case t__ of { RightParen v1 -> get >>= k_325_2__ v1; _ -> frown [")"] t__ }
  
  state_611__ k_329_1__ t__
      = let {
              goto_qcon'1__ v1 = k_329_1__ v1;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2
            }
        in case t__ of {
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1 -> get >>= state_497__ (reduce_254__ goto_qcon'1__ v1);
               _ -> frown ["<conid>", "<qconid>", "("] t__ }
  
  state_613__ k_330_1__ t__
      = let {
              goto_varop'1__ v1 = reduce_257__ goto_op'1__ v1;
              goto_conop'1__ v1 = reduce_258__ goto_op'1__ v1;
              goto_op'1__ v1 = k_330_1__ v1;
              goto_consym'2__ v1 v2 = reduce_255__ goto_conop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_259__ goto_varop'1__ v1 v2
            }
        in case t__ of {
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               BackQuote v1 -> get >>= state_503__ (reduce_256__ goto_conop'1__ v1) (reduce_260__ goto_varop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               _ -> frown ["<varsym>", "<consym>", "`", ".", "-", "!"] t__ }
  
  state_617__ k_333_1__ t__
      = let {
              goto_var'1__ v1 = state_773__ (reduce_451__ goto_many1'1_var'1__ v1);
              goto_varid'2__ v1 v2 = reduce_247__ goto_var'1__ v1 v2;
              goto_many1'1_var'1__ v1 = state_618__ (k_333_1__ v1)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               LeftParen v1 -> get >>= state_478__ (reduce_248__ goto_var'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_618__ k_333_2__ t__ = case t__ of { Dot v1 -> get >>= k_333_2__ v1; _ -> frown ["."] t__ }
  
  state_623__ k_340_1__ k_341_1__ t__
      = let {
              goto_mSepBy1'1_export'1_21__ v1 = state_624__ (k_340_1__ v1);
              goto_mSepBy1''1_export'1_21__ v1
                  = state_629__ (reduce_342__ goto_mSepBy1'1_export'1_21__ v1)
                        (reduce_344__ goto_mSepBy1''1_export'1_21__ v1);
              goto_export'1__ v1 = reduce_343__ goto_mSepBy1''1_export'1_21__ v1;
              goto_qvar'1__ v1 = reduce_345__ goto_export'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_354__ goto_qtyconorcls'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_qtyconorcls'1__ v1
                  = state_634__ (reduce_346__ goto_export'1__ v1) (reduce_347__ goto_export'1__ v1)
                            (reduce_348__ goto_export'1__ v1)
                        (reduce_349__ goto_export'1__ v1);
              goto_opt'1_21__ v1 = state_627__ (k_341_1__ v1)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1 -> get >>= state_483__ (reduce_250__ goto_qvar'1__ v1);
               RightParen v1 -> reduce_334__ goto_opt'1_21__ t__;
               Comma v1 -> get >>= reduce_335__ goto_opt'1_21__ v1;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Module v1 -> get >>= state_641__ (reduce_350__ goto_export'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "(",
                               ")",
                               ",",
                               "as",
                               "hiding",
                               "module",
                               "qualified"]
                          t__ }
  
  state_624__ k_340_2__ t__
      = let {
              goto_opt'1_21__ v1 = state_625__ (k_340_2__ v1)
            }
        in case t__ of {
               RightParen v1 -> reduce_334__ goto_opt'1_21__ t__;
               Comma v1 -> get >>= reduce_335__ goto_opt'1_21__ v1;
               _ -> frown [")", ","] t__ }
  
  state_625__ k_340_3__ t__ = case t__ of { RightParen v1 -> get >>= k_340_3__ v1; _ -> frown [")"] t__ }
  
  state_627__ k_341_2__ t__ = case t__ of { RightParen v1 -> get >>= k_341_2__ v1; _ -> frown [")"] t__ }
  
  state_629__ k_342_1__ k_344_1__ t__
      = case t__ of {
            RightParen v1 -> k_342_1__ t__;
            Comma v1 -> get >>= state_631__ (k_344_1__ v1);
            _ -> frown [")", ","] t__ }
  
  state_631__ k_344_2__ t__
      = let {
              goto_export'1__ v1 = k_344_2__ v1;
              goto_qvar'1__ v1 = reduce_345__ goto_export'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_354__ goto_qtyconorcls'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_qtyconorcls'1__ v1
                  = state_634__ (reduce_346__ goto_export'1__ v1) (reduce_347__ goto_export'1__ v1)
                            (reduce_348__ goto_export'1__ v1)
                        (reduce_349__ goto_export'1__ v1)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1 -> get >>= state_483__ (reduce_250__ goto_qvar'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Module v1 -> get >>= state_641__ (reduce_350__ goto_export'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "as", "hiding", "module", "qualified"] t__ }
  
  state_634__ k_346_1__ k_347_1__ k_348_1__ k_349_1__ t__
      = case t__ of {
            LeftParen v1 -> get >>= state_635__ (k_347_1__ v1) (k_348_1__ v1) (k_349_1__ v1);
            _ -> k_346_1__ t__ }
  
  state_635__ k_347_2__ k_348_2__ k_349_2__ t__
      = let {
              goto_mSepBy1'1_qcname'1_21__ v1 = state_639__ (k_349_2__ v1);
              goto_mSepBy1''1_qcname'1_21__ v1
                  = state_643__ (reduce_351__ goto_mSepBy1'1_qcname'1_21__ v1)
                        (reduce_353__ goto_mSepBy1''1_qcname'1_21__ v1);
              goto_qcname'1__ v1 = reduce_352__ goto_mSepBy1''1_qcname'1_21__ v1;
              goto_qvar'1__ v1 = reduce_59__ goto_qcname'1__ v1;
              goto_qcon'1__ v1 = reduce_60__ goto_qcname'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1 -> get >>= state_484__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1);
               RightParen v1 -> get >>= k_348_2__ v1;
               DotDot v1 -> get >>= state_636__ (k_347_2__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", ")", "..", "as", "hiding", "qualified"]
                          t__ }
  
  state_636__ k_347_3__ t__ = case t__ of { RightParen v1 -> get >>= k_347_3__ v1; _ -> frown [")"] t__ }
  
  state_639__ k_349_3__ t__ = case t__ of { RightParen v1 -> get >>= k_349_3__ v1; _ -> frown [")"] t__ }
  
  state_641__ k_350_1__ t__
      = let {
              goto_modid'1__ v1 = k_350_1__ v1
            }
        in case t__ of {
               ConId v1 v2 -> get >>= reduce_292__ goto_modid'1__ v1 v2;
               QConId v1 v2 -> get >>= reduce_293__ goto_modid'1__ v1 v2;
               _ -> frown ["<conid>", "<qconid>"] t__ }
  
  state_643__ k_351_1__ k_353_1__ t__
      = case t__ of {
            RightParen v1 -> k_351_1__ t__;
            Comma v1 -> get >>= state_645__ (k_353_1__ v1);
            _ -> frown [")", ","] t__ }
  
  state_645__ k_353_2__ t__
      = let {
              goto_qcname'1__ v1 = k_353_2__ v1;
              goto_qvar'1__ v1 = reduce_59__ goto_qcname'1__ v1;
              goto_qcon'1__ v1 = reduce_60__ goto_qcname'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1 -> get >>= state_484__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_649__ k_357_1__ t__
      = let {
              goto_modid'1__ v1 = k_357_1__ v1
            }
        in case t__ of {
               ConId v1 v2 -> get >>= reduce_292__ goto_modid'1__ v1 v2;
               QConId v1 v2 -> get >>= reduce_293__ goto_modid'1__ v1 v2;
               _ -> frown ["<conid>", "<qconid>"] t__ }
  
  state_651__ k_358_1__ k_365_1__ t__
      = let {
              goto_qcname'1__ v1 = k_365_1__ v1;
              goto_qvar'1__ v1 = reduce_59__ goto_qcname'1__ v1;
              goto_qcon'1__ v1 = reduce_60__ goto_qcname'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1 -> get >>= state_484__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> k_358_1__ t__ }
  
  state_652__ k_359_1__ k_367_1__ t__
      = let {
              goto_tdepbinding'1__ v1 = k_367_1__ v1;
              goto_qconid'1__ v1 = reduce_354__ goto_qtyconorcls'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_qtyconorcls'1__ v1 = reduce_368__ goto_tdepbinding'1__ v1
            }
        in case t__ of {
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1 -> get >>= state_660__ (reduce_369__ goto_tdepbinding'1__ v1);
               _ -> k_359_1__ t__ }
  
  state_653__ k_360_1__ k_371_1__ t__
      = let {
              goto_qcname'1__ v1 = reduce_372__ goto_cdepbinding'1__ v1;
              goto_cdepbinding'1__ v1 = k_371_1__ v1;
              goto_qvar'1__ v1 = reduce_59__ goto_qcname'1__ v1;
              goto_qcon'1__ v1 = reduce_60__ goto_qcname'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_485__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                              (reduce_373__ goto_cdepbinding'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> k_360_1__ t__ }
  
  state_654__ k_361_1__ k_375_1__ t__
      = let {
              goto_depbinding'1__ v1 = k_375_1__ v1;
              goto_qvar'1__ v1 = reduce_376__ goto_depbinding'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               LeftParen v1 -> get >>= state_487__ (reduce_250__ goto_qvar'1__ v1) (reduce_377__ goto_depbinding'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> k_361_1__ t__ }
  
  state_655__ k_362_1__ k_379_1__ t__
      = let {
              goto_kindannottyvar'1__ v1 = k_379_1__ v1;
              goto_varid'2__ v1 v2 = reduce_297__ goto_tyvar'1__ v1 v2;
              goto_tyvar'1__ v1 = reduce_380__ goto_kindannottyvar'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               LeftParen v1 -> get >>= state_679__ (reduce_381__ goto_kindannottyvar'1__ v1);
               Equals v1 -> k_362_1__ t__;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "(", "=", "as", "hiding", "qualified"] t__ }
  
  state_656__ k_363_1__ k_383_1__ t__
      = let {
              goto_varid'2__ v1 v2 = reduce_297__ goto_tyvar'1__ v1 v2;
              goto_tyvar'1__ v1 = k_383_1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               Equals v1 -> k_363_1__ t__;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "=", "as", "hiding", "qualified"] t__ }
  
  state_660__ k_369_1__ t__
      = let {
              goto_qconid'1__ v1 = reduce_354__ goto_qtyconorcls'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_qtyconorcls'1__ v1 = state_661__ (k_369_1__ v1)
            }
        in case t__ of {
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               _ -> frown ["<conid>", "<qconid>"] t__ }
  
  state_661__ k_369_2__ t__ = case t__ of { KW_As v1 -> get >>= state_662__ (k_369_2__ v1); _ -> frown ["as"] t__ }
  
  state_662__ k_369_3__ t__
      = let {
              goto_qconid'1__ v1 = reduce_354__ goto_qtyconorcls'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_qtyconorcls'1__ v1 = state_663__ (k_369_3__ v1)
            }
        in case t__ of {
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               _ -> frown ["<conid>", "<qconid>"] t__ }
  
  state_663__ k_369_4__ t__ = case t__ of { RightParen v1 -> get >>= k_369_4__ v1; _ -> frown [")"] t__ }
  
  state_667__ k_373_2__ t__ = case t__ of { KW_As v1 -> get >>= state_668__ (k_373_2__ v1); _ -> frown ["as"] t__ }
  
  state_668__ k_373_3__ t__
      = let {
              goto_qcname'1__ v1 = state_669__ (k_373_3__ v1);
              goto_qvar'1__ v1 = reduce_59__ goto_qcname'1__ v1;
              goto_qcon'1__ v1 = reduce_60__ goto_qcname'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               LeftParen v1 -> get >>= state_484__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "<conid>", "<qconid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_669__ k_373_4__ t__ = case t__ of { RightParen v1 -> get >>= k_373_4__ v1; _ -> frown [")"] t__ }
  
  state_673__ k_377_2__ t__ = case t__ of { KW_As v1 -> get >>= state_674__ (k_377_2__ v1); _ -> frown ["as"] t__ }
  
  state_674__ k_377_3__ t__
      = let {
              goto_qvar'1__ v1 = state_675__ (k_377_3__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               LeftParen v1 -> get >>= state_483__ (reduce_250__ goto_qvar'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_675__ k_377_4__ t__ = case t__ of { RightParen v1 -> get >>= k_377_4__ v1; _ -> frown [")"] t__ }
  
  state_679__ k_381_1__ t__
      = let {
              goto_varid'2__ v1 v2 = reduce_297__ goto_tyvar'1__ v1 v2;
              goto_tyvar'1__ v1 = state_680__ (k_381_1__ v1)
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "as", "hiding", "qualified"] t__ }
  
  state_680__ k_381_2__ t__ = case t__ of { DoubleColon v1 -> get >>= state_681__ (k_381_2__ v1); _ -> frown ["::"] t__ }
  
  state_681__ k_381_3__ t__
      = let {
              goto_varop'1__ v1 = reduce_257__ goto_op'1__ v1;
              goto_conop'1__ v1 = reduce_258__ goto_op'1__ v1;
              goto_op'1__ v1 = reduce_323__ goto_kindterm'1__ v1;
              goto_varid'2__ v1 v2 = reduce_326__ goto_kivar'1__ v1 v2;
              goto_consym'2__ v1 v2 = reduce_255__ goto_conop'1__ v1 v2;
              goto_varsym'2__ v1 v2 = reduce_259__ goto_varop'1__ v1 v2;
              goto_kind'1__ v1 = state_682__ (k_381_3__ v1);
              goto_kindterm'1__ v1 = state_600__ (reduce_321__ goto_kind'1__ v1) (reduce_322__ goto_kind'1__ v1);
              goto_kivar'1__ v1 = reduce_324__ goto_kindterm'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               VarSym v1 v2 -> get >>= reduce_283__ goto_varsym'2__ v1 v2;
               ConSym v1 v2 -> get >>= reduce_280__ goto_consym'2__ v1 v2;
               LeftParen v1 -> get >>= state_605__ (reduce_325__ goto_kindterm'1__ v1);
               BackQuote v1 -> get >>= state_503__ (reduce_256__ goto_conop'1__ v1) (reduce_260__ goto_varop'1__ v1);
               Dot v1 -> get >>= reduce_286__ goto_varsym'2__ v1;
               Minus v1 -> get >>= reduce_284__ goto_varsym'2__ v1;
               Exclamation v1 -> get >>= reduce_285__ goto_varsym'2__ v1;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<varsym>", "<consym>", "(", "`", ".", "-", "!", "as", "hiding", "qualified"] t__ }
  
  state_682__ k_381_4__ t__ = case t__ of { RightParen v1 -> get >>= k_381_4__ v1; _ -> frown [")"] t__ }
  
  state_685__ k_384_1__ t__
      = let {
              goto_many'1_gdrh'1__ v1 = k_384_1__ v1;
              goto_many''1_gdrh'1__ v1
                  = state_691__ (reduce_386__ goto_many'1_gdrh'1__ v1) (reduce_388__ goto_many''1_gdrh'1__ v1)
            }
        in reduce_387__ goto_many''1_gdrh'1__ t__
  
  state_687__ k_385_1__ t__
      = let {
              goto_exp0'1__ v1 = state_688__ (k_385_1__ v1);
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_403__ (reduce_214__ goto_exp0'1__ v1) (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_688__ k_385_2__ t__ = case t__ of { Equals v1 -> get >>= state_689__ (k_385_2__ v1); _ -> frown ["="] t__ }
  
  state_689__ k_385_3__ t__
      = let {
              goto_xExp'1__ v1 = k_385_3__ v1;
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_58__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                            (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_691__ k_386_1__ k_388_1__ t__
      = let {
              goto_gdrh'1__ v1 = k_388_1__ v1
            }
        in case t__ of { Bar v1 -> get >>= state_687__ (reduce_385__ goto_gdrh'1__ v1); _ -> k_386_1__ t__ }
  
  state_693__ k_389_1__ t__
      = let {
              goto_many'1_aexp'1__ v1 = k_389_1__ v1;
              goto_many''1_aexp'1__ v1
                  = state_763__ (reduce_443__ goto_many'1_aexp'1__ v1) (reduce_445__ goto_many''1_aexp'1__ v1)
            }
        in reduce_444__ goto_many''1_aexp'1__ t__
  
  state_695__ k_390_1__ k_399_1__ k_400_1__ t__
      = let {
              goto_parentype'2__ v1 v2 = k_400_1__ v1 v2
            }
        in case t__ of {
               LeftTyParen v1
                   -> get
                          >>=
                          state_563__ (reduce_301__ goto_parentype'2__ v1) (reduce_302__ goto_parentype'2__ v1)
                                      (reduce_303__ goto_parentype'2__ v1)
                                  (reduce_304__ goto_parentype'2__ v1)
                              (reduce_305__ goto_parentype'2__ v1);
               At v1 -> get >>= state_696__ (k_390_1__ v1);
               _ -> k_399_1__ t__ }
  
  state_696__ k_390_2__ t__
      = let {
              goto_aexp'1__ v1 = k_390_2__ v1;
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "~",
                               "as",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_698__ k_391_1__ t__
      = let {
              goto_aexp'1__ v1 = k_391_1__ v1;
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "~",
                               "as",
                               "hiding",
                               "qualified"]
                          t__ }
  
  state_700__ k_392_1__ k_393_1__ k_394_1__ t__
      = case t__ of { LeftCurly v1 -> get >>= state_701__ (k_393_1__ v1) (k_394_1__ v1); _ -> k_392_1__ t__ }
  
  state_701__ k_393_2__ k_394_2__ t__
      = let {
              goto_mSepBy1'1_fbind'1_21__ v1 = state_703__ (k_394_2__ v1);
              goto_mSepBy1''1_fbind'1_21__ v1
                  = state_706__ (reduce_396__ goto_mSepBy1'1_fbind'1_21__ v1)
                        (reduce_398__ goto_mSepBy1''1_fbind'1_21__ v1);
              goto_fbind'1__ v1 = reduce_397__ goto_mSepBy1''1_fbind'1_21__ v1;
              goto_qvar'1__ v1 = state_746__ (reduce_426__ goto_fbind'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               LeftParen v1 -> get >>= state_483__ (reduce_250__ goto_qvar'1__ v1);
               RightCurly v1 -> get >>= k_393_2__ v1;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "(", "}", "as", "hiding", "qualified"] t__ }
  
  state_703__ k_394_3__ t__ = case t__ of { RightCurly v1 -> get >>= k_394_3__ v1; _ -> frown ["}"] t__ }
  
  state_706__ k_396_1__ k_398_1__ t__
      = case t__ of {
            RightCurly v1 -> k_396_1__ t__;
            Comma v1 -> get >>= state_708__ (k_398_1__ v1);
            _ -> frown ["}", ","] t__ }
  
  state_708__ k_398_2__ t__
      = let {
              goto_fbind'1__ v1 = k_398_2__ v1;
              goto_qvar'1__ v1 = state_746__ (reduce_426__ goto_fbind'1__ v1);
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               LeftParen v1 -> get >>= state_483__ (reduce_250__ goto_qvar'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "<qvarid>", "(", "as", "hiding", "qualified"] t__ }
  
  state_713__ k_403_2__ t__ = case t__ of { RightParen v1 -> get >>= k_403_2__ v1; _ -> frown [")"] t__ }
  
  state_715__ k_404_1__ k_428_1__ t__
      = let {
              goto_xExp'1__ v1
                  = state_727__ (reduce_411__ goto_list'1__ v1) (reduce_413__ goto_list'1__ v1)
                                        (reduce_414__ goto_list'1__ v1)
                                    (reduce_415__ goto_list'1__ v1)
                                (reduce_416__ goto_list'1__ v1)
                            (reduce_417__ goto_list'1__ v1)
                        (reduce_422__ goto_lexps'1__ v1);
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_58__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                            (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_list'1__ v1 = state_716__ (k_404_1__ v1);
              goto_lexps'1__ v1 = state_728__ (reduce_412__ goto_list'1__ v1) (reduce_421__ goto_lexps'1__ v1);
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               RightSquare v1 -> get >>= k_428_1__ v1;
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "]",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_716__ k_404_2__ t__ = case t__ of { RightSquare v1 -> get >>= k_404_2__ v1; _ -> frown ["]"] t__ }
  
  state_719__ k_406_2__ t__
      = let {
              goto_exp0'1__ v1 = state_720__ (k_406_2__ v1);
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_403__ (reduce_214__ goto_exp0'1__ v1) (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_720__ k_406_3__ t__ = case t__ of { RightParen v1 -> get >>= k_406_3__ v1; _ -> frown [")"] t__ }
  
  state_723__ k_408_1__ k_410_1__ t__
      = case t__ of {
            RightParen v1 -> k_408_1__ t__;
            Comma v1 -> get >>= state_725__ (k_410_1__ v1);
            _ -> frown [")", ","] t__ }
  
  state_725__ k_410_2__ t__
      = let {
              goto_xExp'1__ v1 = k_410_2__ v1;
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_58__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                            (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_727__ k_411_1__ k_413_1__ k_414_1__ k_415_1__ k_416_1__ k_417_1__ k_422_1__ t__
      = case t__ of {
            RightSquare v1 -> k_411_1__ t__;
            Comma v1 -> get >>= state_730__ (k_414_1__ v1) (k_416_1__ v1) (k_422_1__ v1);
            DotDot v1 -> get >>= state_729__ (k_413_1__ v1) (k_415_1__ v1);
            Bar v1 -> get >>= state_735__ (k_417_1__ v1);
            _ -> frown ["]", ",", "..", "|"] t__ }
  
  state_728__ k_412_1__ k_421_1__ t__
      = case t__ of {
            RightSquare v1 -> k_412_1__ t__;
            Comma v1 -> get >>= state_741__ (k_421_1__ v1);
            _ -> frown ["]", ","] t__ }
  
  state_729__ k_413_2__ k_415_2__ t__
      = let {
              goto_xExp'1__ v1 = k_415_2__ v1;
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_58__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                            (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               RightSquare v1 -> k_413_2__ t__;
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "]",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_730__ k_414_2__ k_416_2__ k_422_2__ t__
      = let {
              goto_xExp'1__ v1 = state_731__ (k_414_2__ v1) (k_416_2__ v1) (k_422_2__ v1);
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_58__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                            (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_731__ k_414_3__ k_416_3__ k_422_3__ t__
      = case t__ of { DotDot v1 -> get >>= state_732__ (k_414_3__ v1) (k_416_3__ v1); _ -> k_422_3__ t__ }
  
  state_732__ k_414_4__ k_416_4__ t__
      = let {
              goto_xExp'1__ v1 = k_416_4__ v1;
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_58__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                            (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               RightSquare v1 -> k_414_4__ t__;
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "]",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_735__ k_417_2__ t__
      = let {
              goto_xExp'1__ v1 = reduce_424__ goto_qual'1__ v1;
              goto_mSepBy1'1_qual'1_21__ v1 = k_417_2__ v1;
              goto_mSepBy1''1_qual'1_21__ v1
                  = state_737__ (reduce_418__ goto_mSepBy1'1_qual'1_21__ v1)
                        (reduce_420__ goto_mSepBy1''1_qual'1_21__ v1);
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_61__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                                (reduce_216__ goto_exp0a'1__ v1)
                            (reduce_218__ goto_exp0b'1__ v1)
                        (reduce_423__ goto_qual'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_qual'1__ v1 = reduce_419__ goto_mSepBy1''1_qual'1_21__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_418__ (reduce_220__ goto_exp10a'1__ v1) (reduce_425__ goto_qual'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_737__ k_418_1__ k_420_1__ t__
      = case t__ of {
            RightSquare v1 -> k_418_1__ t__;
            Comma v1 -> get >>= state_739__ (k_420_1__ v1);
            _ -> frown ["]", ","] t__ }
  
  state_739__ k_420_2__ t__
      = let {
              goto_xExp'1__ v1 = reduce_424__ goto_qual'1__ v1;
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_61__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                                (reduce_216__ goto_exp0a'1__ v1)
                            (reduce_218__ goto_exp0b'1__ v1)
                        (reduce_423__ goto_qual'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_qual'1__ v1 = k_420_2__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_418__ (reduce_220__ goto_exp10a'1__ v1) (reduce_425__ goto_qual'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_741__ k_421_2__ t__
      = let {
              goto_xExp'1__ v1 = k_421_2__ v1;
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_58__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                            (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_743__ k_423_2__ t__
      = let {
              goto_xExp'1__ v1 = k_423_2__ v1;
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_58__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                            (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_746__ k_426_1__ t__ = case t__ of { Equals v1 -> get >>= state_747__ (k_426_1__ v1); _ -> frown ["="] t__ }
  
  state_747__ k_426_2__ t__
      = let {
              goto_xExp'1__ v1 = k_426_2__ v1;
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_58__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                            (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_756__ k_434_2__ t__ = case t__ of { BackQuote v1 -> get >>= k_434_2__ v1; _ -> frown ["`"] t__ }
  
  state_763__ k_443_1__ k_445_1__ t__
      = let {
              goto_aexp'1__ v1 = k_445_1__ v1;
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> k_443_1__ t__ }
  
  state_765__ k_446_1__ t__
      = let {
              goto_many'1_gdpat'1__ v1 = k_446_1__ v1;
              goto_many''1_gdpat'1__ v1
                  = state_771__ (reduce_448__ goto_many'1_gdpat'1__ v1) (reduce_450__ goto_many''1_gdpat'1__ v1)
            }
        in reduce_449__ goto_many''1_gdpat'1__ t__
  
  state_767__ k_447_1__ t__
      = let {
              goto_exp0'1__ v1 = state_768__ (k_447_1__ v1);
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_403__ (reduce_214__ goto_exp0'1__ v1) (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_768__ k_447_2__ t__ = case t__ of { RightArrow v1 -> get >>= state_769__ (k_447_2__ v1); _ -> frown ["->"] t__ }
  
  state_769__ k_447_3__ t__
      = let {
              goto_xExp'1__ v1 = k_447_3__ v1;
              goto_exp0'1__ v1 = reduce_21__ goto_xExp'1__ v1;
              goto_exp0a'1__ v1 = reduce_213__ goto_exp0'1__ v1;
              goto_exp0b'1__ v1
                  = state_58__ (reduce_20__ goto_xExp'1__ v1) (reduce_214__ goto_exp0'1__ v1)
                            (reduce_216__ goto_exp0a'1__ v1)
                        (reduce_218__ goto_exp0b'1__ v1);
              goto_exp10a'1__ v1 = reduce_215__ goto_exp0a'1__ v1;
              goto_exp10b'1__ v1 = reduce_217__ goto_exp0b'1__ v1;
              goto_fexp'1__ v1 = reduce_225__ goto_exp10b'1__ v1;
              goto_aexp'1__ v1 = state_693__ (reduce_389__ goto_many1'1_aexp'1__ v1);
              goto_aexp1'1__ v1
                  = state_700__ (reduce_392__ goto_aexp'1__ v1) (reduce_393__ goto_aexp1'1__ v1)
                        (reduce_394__ goto_aexp1'1__ v1);
              goto_aexp2'1__ v1 = reduce_395__ goto_aexp1'1__ v1;
              goto_gcon'1__ v1 = reduce_401__ goto_aexp2'1__ v1;
              goto_qvar'1__ v1
                  = state_695__ (reduce_390__ goto_aexp'1__ v1) (reduce_399__ goto_aexp2'1__ v1)
                        (reduce_400__ goto_aexp2'1__ v1);
              goto_qcon'1__ v1 = reduce_430__ goto_gcon'1__ v1;
              goto_qvarid'2__ v1 v2 = reduce_249__ goto_qvar'1__ v1 v2;
              goto_varid'2__ v1 v2 = reduce_267__ goto_qvarid'2__ v1 v2;
              goto_qconid'1__ v1 = reduce_253__ goto_qcon'1__ v1;
              goto_qconid2'2__ v1 v2 = reduce_273__ goto_qconid'1__ v1 v2;
              goto_conid'2__ v1 v2 = reduce_274__ goto_qconid2'2__ v1 v2;
              goto_literal'1__ v1 = reduce_402__ goto_aexp2'1__ v1;
              goto_many1'1_aexp'1__ v1 = reduce_226__ goto_fexp'1__ v1
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               QVarId v1 v2 -> get >>= reduce_268__ goto_qvarid'2__ v1 v2;
               ConId v1 v2 -> get >>= reduce_276__ goto_conid'2__ v1 v2;
               QConId v1 v2 -> get >>= reduce_275__ goto_qconid2'2__ v1 v2;
               IntTok v1 v2 -> get >>= reduce_439__ goto_literal'1__ v1 v2;
               FloatTok v1 v2 -> get >>= reduce_441__ goto_literal'1__ v1 v2;
               Character v1 v2 -> get >>= reduce_440__ goto_literal'1__ v1 v2;
               StringTok v1 v2 -> get >>= reduce_442__ goto_literal'1__ v1 v2;
               LeftParen v1
                   -> get
                          >>=
                          state_486__ (reduce_250__ goto_qvar'1__ v1) (reduce_254__ goto_qcon'1__ v1)
                                              (reduce_403__ goto_aexp2'1__ v1)
                                          (reduce_405__ goto_aexp2'1__ v1)
                                      (reduce_406__ goto_aexp2'1__ v1)
                                  (reduce_427__ goto_gcon'1__ v1)
                              (reduce_429__ goto_gcon'1__ v1);
               LeftSquare v1 -> get >>= state_715__ (reduce_404__ goto_aexp2'1__ v1) (reduce_428__ goto_gcon'1__ v1);
               Underscore v1 -> get >>= reduce_407__ goto_aexp2'1__ v1;
               Backslash v1 -> get >>= state_412__ (reduce_219__ goto_exp10a'1__ v1);
               Tilde v1 -> get >>= state_698__ (reduce_391__ goto_aexp'1__ v1);
               Minus v1 -> get >>= state_434__ (reduce_223__ goto_exp10b'1__ v1);
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Case v1 -> get >>= state_430__ (reduce_222__ goto_exp10b'1__ v1);
               KW_Do v1 -> get >>= state_437__ (reduce_224__ goto_exp10b'1__ v1);
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_If v1 -> get >>= state_424__ (reduce_221__ goto_exp10a'1__ v1);
               KW_Let v1 -> get >>= state_416__ (reduce_220__ goto_exp10a'1__ v1);
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _
                   -> frown
                              ["<varid>",
                               "<qvarid>",
                               "<conid>",
                               "<qconid>",
                               "<integer literal>",
                               "<float literal>",
                               "<character literal>",
                               "<string literal>",
                               "(",
                               "[",
                               "_",
                               "\\",
                               "~",
                               "-",
                               "as",
                               "case",
                               "do",
                               "hiding",
                               "if",
                               "let",
                               "qualified"]
                          t__ }
  
  state_771__ k_448_1__ k_450_1__ t__
      = let {
              goto_gdpat'1__ v1 = k_450_1__ v1
            }
        in case t__ of { Bar v1 -> get >>= state_767__ (reduce_447__ goto_gdpat'1__ v1); _ -> k_448_1__ t__ }
  
  state_773__ k_451_1__ t__
      = let {
              goto_many'1_var'1__ v1 = k_451_1__ v1;
              goto_many''1_var'1__ v1
                  = state_775__ (reduce_452__ goto_many'1_var'1__ v1) (reduce_454__ goto_many''1_var'1__ v1)
            }
        in reduce_453__ goto_many''1_var'1__ t__
  
  state_775__ k_452_1__ k_454_1__ t__
      = let {
              goto_var'1__ v1 = k_454_1__ v1;
              goto_varid'2__ v1 v2 = reduce_247__ goto_var'1__ v1 v2
            }
        in case t__ of {
               VarId v1 v2 -> get >>= reduce_269__ goto_varid'2__ v1 v2;
               LeftParen v1 -> get >>= state_478__ (reduce_248__ goto_var'1__ v1);
               Dot v1 -> k_452_1__ t__;
               KW_As v1 -> get >>= reduce_270__ goto_varid'2__ v1;
               KW_Hiding v1 -> get >>= reduce_272__ goto_varid'2__ v1;
               KW_Qualified v1 -> get >>= reduce_271__ goto_varid'2__ v1;
               _ -> frown ["<varid>", "(", ".", "as", "hiding", "qualified"] t__ }
  
  reduce_12__ g__ range_begin version _ name _ exportedTypes _ exportedValues range_where range_end ((declarations,deps))
      ts__
      = ( do { checkInterfaceVersion version
              ; let { range = rangeFromTo range_begin range_end
                    }
              ; return (ModuleInterface_ModuleInterface range name exportedTypes exportedValues declarations
                       ,deps
                       ) 
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_13__ g__ range_begin name exports_ _ range_end body ts__
      = g__
                ( let { range   = rangeFromTo range_begin range_end 
              ; exports = maybe MaybeExports_Nothing MaybeExports_Just exports_ }
          in  Module_Module range (MaybeModuleName_Just name) exports body )
            ts__
  
  reduce_14__ g__ range body ts__ = g__ ( Module_Module range MaybeModuleName_Nothing MaybeExports_Nothing body ) ts__
  
  reduce_15__ g__ names ts__ = g__ ( names ) ts__
  
  reduce_16__ g__ declarations ts__ = g__ ( declarations ) ts__
  
  reduce_17__ g__ context_ _ type_ ts__
      = ( do { context <- checkContext context_
              ; let { range = rangeFromTo (getRange context_) (getRange type_) }
              ; return $ Type_Qualified range context type_
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_18__ g__ range_begin typevariables type_ ts__
      = g__ ( let { range = rangeFromTo range_begin (getRange type_) }
          in  Type_Forall range typevariables type_ )
            ts__
  
  reduce_19__ g__ type_ ts__ = g__ ( type_ ) ts__
  
  reduce_20__ g__ expression _ type_ ts__
      = g__
                ( let { range = rangeFromTo (getRange expression) (getRange type_) }
          in  Expression_Typed range expression type_ )
            ts__
  
  reduce_21__ g__ expression ts__ = g__ ( expression ) ts__
  
  reduce_22__ g__ name ts__ = g__ ( name ) ts__
  
  reduce_23__ g__ name ts__ = g__ ( name ) ts__
  
  reduce_24__ g__ name ts__ = g__ ( name ) ts__
  
  reduce_25__ g__ name ts__ = g__ ( name ) ts__
  
  reduce_26__ g__ kindargument ts__ = g__ ( kindargument ) ts__
  
  reduce_27__ g__ kind ts__ = g__ ( kind ) ts__
  
  reduce_28__ g__ range_begin ((declarations,deps)) range_end ts__
      = g__ ( rangeFromTo range_begin range_end ) ( (declarations,deps) ) ts__
  
  reduce_29__ g__ _ ((declarations,deps)) _ ts__ = g__ ( getRange declarations ) ( (declarations,deps) ) ts__
  
  reduce_30__ g__ _ declarations_ deps_ _ ts__
      = g__
                ( let { declarations = declarations_ [] 
              ; deps         = concat (deps_ [])
              }
          in  (groupIDecls declarations,deps) )
            ts__
  
  reduce_31__ g__ _ ts__ = g__ ( ([],[]) ) ts__
  
  reduce_32__ g__ a b ts__ = g__ ( \ as -> a : as ) ( \ bs -> b : bs ) ts__
  
  reduce_33__ g__ sa sb _ a b ts__ = g__ ( \ as -> sa (a : as) ) ( \ bs -> sb (b : bs) ) ts__
  
  reduce_34__ g__ range_begin name _ kind range_equal constructors types ts__
      = g__ ( InterfaceDeclaration_Abstract (rangeFromTo range_begin (getRange types)) name constructors types kind )
                ( [] )
            ts__
  
  reduce_35__ g__ range_begin name _ kind ts__
      = g__ ( InterfaceDeclaration_Abstract (rangeFromTo range_begin (getRange kind)) name [] [] kind ) ( [] ) ts__
  
  reduce_36__ g__ range_begin name _ _ types range_end ts__
      = g__ ( InterfaceDeclaration_Consig (rangeFromTo range_begin range_end) (isCon name) name types ) ( [] ) ts__
  
  reduce_37__ g__ range_begin name ts__
      = g__ ( InterfaceDeclaration_Binding (rangeFromTo range_begin (getRange name)) name ) ( [] ) ts__
  
  reduce_38__ g__ range_begin module_ _ range_end ((declarations,deps)) ts__
      = g__ ( InterfaceDeclaration_SetModule (rangeFromTo range_begin range_end) module_ declarations ) ( deps ) ts__
  
  reduce_39__ g__ range_begin module_ ts__
      = g__ ( InterfaceDeclaration_Import (rangeFromTo range_begin (getRange module_)) module_ ) ( [module_] ) ts__
  
  reduce_40__ g__ range_begin name _ kind _ range_arrow bindings ts__
      = g__
                    ( let { range_end = if null bindings then
                              range_arrow
                            else
                              getRange bindings
              }
          in  InterfaceDeclaration_Dependency 
                (rangeFromTo range_begin range_end)
                name kind bindings
        )
                ( [] )
            ts__
  
  reduce_41__ g__ declaration ts__
      = g__ ( InterfaceDeclaration_Declarations (getRange declaration) [declaration] ) ( [] ) ts__
  
  reduce_42__ g__ ts__ = g__ ( [] ) ts__
  
  reduce_43__ g__ ( as ) ts__ = g__ ( as ) ts__
  
  reduce_44__ g__ ts__ = g__ ( [] ) ( [] ) ts__
  
  reduce_45__ g__ ( as ) ( bs ) ts__ = g__ ( as ) ( bs ) ts__
  
  reduce_46__ g__ s ts__ = g__ ( s [] ) ts__
  
  reduce_47__ g__ s t ts__ = g__ ( s [] ) ( t [] ) ts__
  
  reduce_48__ g__ a ts__ = g__ ( \ as -> a : as ) ts__
  
  reduce_49__ g__ s _ a ts__ = g__ ( \ as -> s (a : as) ) ts__
  
  reduce_50__ g__ a b ts__ = g__ ( \ as -> a : as ) ( \ bs -> b : bs ) ts__
  
  reduce_51__ g__ sa sb _ a b ts__ = g__ ( \ as -> sa (a : as) ) ( \ bs -> sb (b : bs) ) ts__
  
  reduce_52__ g__ name _ type_ ts__ = g__ ( name ) ( type_ ) ts__
  
  reduce_53__ g__ range_begin body range_end ts__ = g__ ( rangeFromTo range_begin range_end ) ( body ) ts__
  
  reduce_54__ g__ _ body _ ts__ = g__ ( getRange body ) ( body ) ts__
  
  reduce_55__ g__ importdeclarations_ range_semi declarations ts__
      = g__
                ( let { range_begin = if null importdeclarations then range_semi else getRange importdeclarations
              ; range       = rangeFromTo range_begin (getRange declarations)
              ; importdeclarations = importdeclarations_ []
              }
          in  Body_Body range importdeclarations declarations )
            ts__
  
  reduce_56__ g__ declarations ts__ = g__ ( Body_Body (getRange declarations) [] declarations ) ts__
  
  reduce_57__ g__ importdeclarations_ ts__
      = g__
                ( let { importdeclarations = importdeclarations_ [] }
          in  Body_Body (getRange importdeclarations) (importdeclarations) [] )
            ts__
  
  reduce_58__ g__ range_optbegin range_end ts__
      = g__ ( let { range_begin = maybe range_end    id range_optbegin }
          in  rangeFromTo range_begin range_end )
            ts__
  
  reduce_59__ g__ name ts__ = g__ ( name ) ts__
  
  reduce_60__ g__ name ts__ = g__ ( name ) ts__
  
  reduce_61__ g__ xs _ x ts__ = g__ ( \ s -> xs (x:s) ) ts__
  
  reduce_62__ g__ xs range ts__ = g__ ( \ s -> xs (ImportDeclaration_Empty range:s) ) ts__
  
  reduce_63__ g__ x ts__ = g__ ( \ s -> x:s ) ts__
  
  reduce_64__ g__ ts__ = g__ ( id ) ts__
  
  reduce_65__ g__ range_begin qualified name asname_ importspecification ts__
      = g__
                ( let { range_end = case importspecification of 
                              { MaybeImportSpecification_Just _ 
                                  -> getRange importspecification
                              ; _ -> maybe (getRange name) getRange asname_
                              }
              ; range  = rangeFromTo range_begin range_end
              ; asname = maybe MaybeModuleName_Nothing MaybeModuleName_Just asname_ }
          in  ImportDeclaration_Import range qualified name asname 
                                       importspecification )
            ts__
  
  reduce_66__ g__ ts__ = g__ ( False ) ts__
  
  reduce_67__ g__ _ ts__ = g__ ( True ) ts__
  
  reduce_68__ g__ importspecification ts__ = g__ ( MaybeImportSpecification_Just importspecification ) ts__
  
  reduce_69__ g__ ts__ = g__ ( MaybeImportSpecification_Nothing ) ts__
  
  reduce_70__ g__ hiding range_begin imports _ range_end ts__
      = g__
                ( let { range = rangeFromTo range_begin range_end }
          in  ImportSpecification_Import range hiding imports )
            ts__
  
  reduce_71__ g__ ts__ = g__ ( [] ) ts__
  
  reduce_72__ g__ ( as ) ts__ = g__ ( as ) ts__
  
  reduce_73__ g__ s ts__ = g__ ( s [] ) ts__
  
  reduce_74__ g__ a ts__ = g__ ( \ as -> a : as ) ts__
  
  reduce_75__ g__ s _ a ts__ = g__ ( \ as -> s (a : as) ) ts__
  
  reduce_76__ g__ ts__ = g__ ( False ) ts__
  
  reduce_77__ g__ _ ts__ = g__ ( True ) ts__
  
  reduce_78__ g__ name ts__ = g__ ( Import_Variable (getRange name) name ) ts__
  
  reduce_79__ g__ range name ts__ = g__ ( Import_TypeOrClass range name MaybeNames_Nothing ) ts__
  
  reduce_80__ g__ range_begin name _ _ range_end ts__
      = g__ ( let { range = rangeFromTo range_begin range_end }
          in  Import_TypeOrClassComplete range name ) ts__
  
  reduce_81__ g__ range_begin name _ range_end ts__
      = g__
                ( let { range = rangeFromTo range_begin range_end }
          in  Import_TypeOrClass range name (MaybeNames_Just []) )
            ts__
  
  reduce_82__ g__ range_begin name _ names range_end ts__
      = g__
                ( let { range = rangeFromTo range_begin range_end }
          in  Import_TypeOrClass range name (MaybeNames_Just names) )
            ts__
  
  reduce_83__ g__ s ts__ = g__ ( s [] ) ts__
  
  reduce_84__ g__ a ts__ = g__ ( \ as -> a : as ) ts__
  
  reduce_85__ g__ s _ a ts__ = g__ ( \ as -> s (a : as) ) ts__
  
  reduce_86__ g__ name ts__ = g__ ( name ) ts__
  
  reduce_87__ g__ name ts__ = g__ ( name ) ts__
  
  reduce_88__ g__ declarations ts__ = g__ ( groupDecls (reverse declarations) ) ts__
  
  reduce_89__ g__ xs _ x ts__ = g__ ( x : xs ) ts__
  
  reduce_90__ g__ xs range ts__ = g__ ( Declaration_Empty range : xs ) ts__
  
  reduce_91__ g__ x ts__ = g__ ( [x] ) ts__
  
  reduce_92__ g__ declaration ts__ = g__ ( declaration ) ts__
  
  reduce_93__ g__ declaration ts__ = g__ ( declaration ) ts__
  
  reduce_94__ g__ range_begin simpletype _ type_ ts__
      = g__
                ( let { range = rangeFromTo range_begin (getRange type_) }
          in Declaration_Type range simpletype type_ )
            ts__
  
  reduce_95__ g__ range_begin type_ _ constructors derivings ts__
      = ( do { (context,simpletype) <- checkSimpletype type_
              ; let { range_end = if null derivings then getRange constructors
                                                    else getRange derivings
                    ; range     = rangeFromTo range_begin range_end
                    }
              ; return $ Declaration_Data range context simpletype 
                                          constructors derivings
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_96__ g__ range_begin type_ _ constructor derivings ts__
      = ( do { (context,simpletype) <- checkSimpletype type_
              ; let { range_end = if null derivings then getRange constructor
                                                    else getRange derivings
                    ; range     = rangeFromTo range_begin range_end
                    }
              ; return $ Declaration_Newtype range context simpletype 
                                             constructor derivings
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_97__ g__ range_begin type_ where_ ts__
      = ( do { (context,simpletype) <- checkSimpletype type_
              ; checkSimplecontext context
              ; let { range_end = case where_ of
                        { MaybeDeclarations_Nothing -> getRange simpletype
                        ; MaybeDeclarations_Just x  -> getRange x
                        }
                    
                    ; range     = rangeFromTo range_begin range_end
                    }
              ; return $ Declaration_Class range context simpletype where_
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_98__ g__ range_begin type_ where_ ts__
      = ( do { (context,name,types) <- checkInstance type_
              ; checkSimplecontext context
              ; let { range_end = case where_ of
                        { MaybeDeclarations_Nothing -> getRange types
                        ; MaybeDeclarations_Just x  -> getRange x
                        }
                    ; range     = rangeFromTo range_begin range_end
                    }
              ; return $ Declaration_Instance range context name types where_
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_99__ g__ range_begin _ types range_end ts__
      = g__ ( let { range = rangeFromTo range_begin range_end }
          in  Declaration_Default range types ) ts__
  
  reduce_100__ g__ range_begin name kind arguments _ type_ ts__
      = g__
                ( let { range = rangeFromTo range_begin (getRange type_) }
          in  Declaration_KindIndexedType range
                [KindIndexedBinding_KindIndexedBinding range name kind arguments type_] )
            ts__
  
  reduce_101__ g__ range_begin name typeargument typevariables _ constructor derivings ts__
      = g__
                ( let { range_end = if null derivings then getRange constructor
                                              else getRange derivings
              ; range     = rangeFromTo range_begin range_end
              }
          in  Declaration_TypeIndexedType range name typeargument typevariables
                                          constructor derivings )
            ts__
  
  reduce_102__ g__ range_begin name range_arrow bindings ts__
      = g__
                ( let { range_end = if null bindings then range_arrow
                                             else getRange bindings
              ; range     = rangeFromTo range_begin range_end
              }
          in  Declaration_Dependency range True {- typelevel -} name bindings
        )
            ts__
  
  reduce_103__ g__ range_begin name range_arrow bindings ts__
      = g__
                ( let { range_end = if null bindings then range_arrow
                                             else getRange bindings
              ; range     = rangeFromTo range_begin range_end
              }
          in  Declaration_Dependency range False {- typelevel -} name bindings
        )
            ts__
  
  reduce_104__ g__ s ts__ = g__ ( s [] ) ts__
  
  reduce_105__ g__ a ts__ = g__ ( \ as -> a : as ) ts__
  
  reduce_106__ g__ s _ a ts__ = g__ ( \ as -> s (a : as) ) ts__
  
  reduce_107__ g__ declaration ts__ = g__ ( declaration ) ts__
  
  reduce_108__ g__ declaration ts__ = g__ ( declaration ) ts__
  
  reduce_109__ g__ declaration ts__ = g__ ( declaration ) ts__
  
  reduce_110__ g__ range_begin fixity priority operators ts__
      = g__
                ( let { range = rangeFromTo range_begin (getRange operators) }
          in  Declaration_Fixity range fixity priority operators 
        )
            ts__
  
  reduce_111__ g__ s ts__ = g__ ( s [] ) ts__
  
  reduce_112__ g__ a ts__ = g__ ( \ as -> a : as ) ts__
  
  reduce_113__ g__ s _ a ts__ = g__ ( \ as -> s (a : as) ) ts__
  
  reduce_114__ g__ ts__ = g__ ( MaybeInt_Nothing ) ts__
  
  reduce_115__ g__ x _ ts__
      = ( do { p <- checkPrec x
              ; return (MaybeInt_Just $ fromInteger $ readInteger p)
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_116__ g__ range ts__ = g__ range (Fixity_Infix range) ts__
  
  reduce_117__ g__ range ts__ = g__ range (Fixity_Infixl range) ts__
  
  reduce_118__ g__ range ts__ = g__ range (Fixity_Infixr range) ts__
  
  reduce_119__ g__ names_rev _ type_ ts__
      = g__
                ( let { names = reverse names_rev
              ; range = rangeFromTo (getRange names) (getRange type_) }
          in  Declaration_TypeSignature range names type_ )
            ts__
  
  reduce_120__ g__ gentysig _ type_ ts__ = ( checkTypeSignature gentysig type_
        ) >>= (\ v1 -> g__ v1 ts__)
  
  reduce_121__ g__ xs _ x ts__ = g__ ( x : xs ) ts__
  
  reduce_122__ g__ xs _ x ts__ = g__ ( x : xs ) ts__
  
  reduce_123__ g__ x ts__ = ( do { checkUnQual x
              ; return [x]
              }
        ) >>= (\ v1 -> g__ v1 ts__)
  
  reduce_124__ g__ argument1 range_fun argument2 ts__
      = g__
                ( let { range = rangeFromTo (getRange argument1) 
                                    (getRange argument2) }
          in  Type_Application range False
                               (Type_Constructor range_fun hsFunName)
                               [argument1,argument2] )
            ts__
  
  reduce_125__ g__ argument1 function argument2 ts__
      = g__
                ( let { range = rangeFromTo (getRange argument1) (getRange argument2) }
          in  Type_Application range False 
                               (Type_Constructor (getRange function) function)
                               [argument1,argument2] )
            ts__
  
  reduce_126__ g__ type_ ts__ = g__ ( type_ ) ts__
  
  reduce_127__ g__ function_ argument ts__
      = g__
                ( case function_ of
          { Type_Application range_begin True function arguments
              -> Type_Application (rangeFromTo range_begin (getRange argument))
                                  True function (arguments ++ [argument])
          ; _ -> Type_Application (rangeFromTo (getRange function_)
                                               (getRange argument))
                                  True function_ [argument] } )
            ts__
  
  reduce_128__ g__ type_ ts__ = g__ ( type_ ) ts__
  
  reduce_129__ g__ type_ ts__ = g__ ( type_ ) ts__
  
  reduce_130__ g__ name ts__ = g__ ( Type_Variable (getRange name) name ) ts__
  
  reduce_131__ g__ range_begin name _ kind range_end ts__
      = g__ ( let { range = rangeFromTo range_begin range_end }
          in  Type_Kinded range name kind ) ts__
  
  reduce_132__ g__ range_begin arguments_rev range_end ts__
      = g__
                ( let { range = rangeFromTo range_begin range_end 
              ; n     = length arguments_rev }
          in  Type_Application range False 
                               (Type_Constructor range (hsTupleName n))
                               (reverse arguments_rev) )
            ts__
  
  reduce_133__ g__ range_begin type_ range_end ts__
      = g__
                ( let { range = rangeFromTo range_begin range_end }
          in  Type_Application range False 
                               (Type_Constructor range hsListName) 
                               [type_] )
            ts__
  
  reduce_134__ g__ range_begin type_ range_end ts__
      = g__ ( let { range = rangeFromTo range_begin range_end }
          in  Type_Parenthesized range type_ ) ts__
  
  reduce_135__ g__ name ts__ = g__ ( Type_Constructor (getRange name) name ) ts__
  
  reduce_136__ g__ range_begin range_end ts__
      = g__ ( let { range = rangeFromTo range_begin range_end }
          in  Type_Constructor range hsUnitName ) ts__
  
  reduce_137__ g__ range_begin _ range_end ts__
      = g__ ( let { range = rangeFromTo range_begin range_end }
          in  Type_Constructor range hsFunName ) ts__
  
  reduce_138__ g__ range_begin range_end ts__
      = g__ ( let { range = rangeFromTo range_begin range_end }
          in  Type_Constructor range hsListName ) ts__
  
  reduce_139__ g__ range_begin n range_end ts__
      = g__ ( let { range = rangeFromTo range_begin range_end }
          in  Type_Constructor range (hsTupleName (n+1)) )
            ts__
  
  reduce_140__ g__ name range_end kind ts__
      = g__ ( let { range = rangeFromTo (getRange name) range_end }
          in  Type_KindApplication range name kind )
            ts__
  
  reduce_141__ g__ name range_end type_ ts__
      = g__ ( let { range = rangeFromTo (getRange name) range_end }
          in  Type_TypeApplication range name type_ )
            ts__
  
  reduce_142__ g__ name range_end type_ ts__
      = g__ ( let { range = rangeFromTo (getRange name) range_end }
          in  Type_TypeApplication range name type_ )
            ts__
  
  reduce_143__ g__ range_begin name range_end ts__
      = g__
                ( let { range = rangeFromTo range_begin range_end }
          in  Type_Parenthesized range
                (Type_Constructor (getRange name) name) )
            ts__
  
  reduce_144__ g__ context_ _ type_ ts__
      = ( do { context <- checkTypeSignatureContext context_
              ; let { range = rangeFromTo (getRange context_) (getRange type_) }
              ; return $ Type_Qualified range context type_
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_145__ g__ range_begin typevariables type_ ts__
      = g__ ( let { range = rangeFromTo range_begin (getRange type_) }
          in  Type_Forall range typevariables type_ )
            ts__
  
  reduce_146__ g__ type_ ts__ = g__ ( type_ ) ts__
  
  reduce_147__ g__ xs _ x ts__ = g__ ( x : xs ) ts__
  
  reduce_148__ g__ x1 _ x2 ts__ = g__ ( [x2,x1] ) ts__
  
  reduce_149__ g__ xs ts__ = g__ ( reverse xs ) ts__
  
  reduce_150__ g__ x ts__ = g__ ( [x] ) ts__
  
  reduce_151__ g__ ts__ = g__ ( [] ) ts__
  
  reduce_152__ g__ name typevariables ts__
      = g__
                ( let { range_end = if null typevariables then getRange name
                                                  else getRange typevariables
              ; range = rangeFromTo (getRange name) range_end
              }
          in SimpleType_SimpleType range name typevariables 
        )
            ts__
  
  reduce_153__ g__ constructor ts__ = g__ ( constructor ) ts__
  
  reduce_154__ g__ leftType constructorOperator rightType ts__
      = g__
                ( let { range = rangeFromTo (getRange leftType) (getRange rightType) }
          in  Constructor_Infix range leftType constructorOperator rightType )
            ts__
  
  reduce_155__ g__ constructor _ fieldDeclarations range_end ts__
      = g__
                ( let { range = rangeFromTo (getRange constructor) range_end }
          in  Constructor_Record range constructor fieldDeclarations )
            ts__
  
  reduce_156__ g__ s ts__ = g__ ( s [] ) ts__
  
  reduce_157__ g__ a ts__ = g__ ( \ as -> a : as ) ts__
  
  reduce_158__ g__ s _ a ts__ = g__ ( \ as -> s (a : as) ) ts__
  
  reduce_159__ g__ type_ ts__
      = ( do { (constructor,types_) <- splitTyConApp type_
              ; let { types = map (\t -> 
                        AnnotatedType_AnnotatedType (getRange t) False t)
                        types_
                    }
              ; return $ 
                    Constructor_Constructor (getRange type_) 
                                            constructor types
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_160__ g__ constructor ts__ = g__ ( constructor ) ts__
  
  reduce_161__ g__ type__ _ type_ ts__
      = ( do { (constructor,types_) <- splitTyConApp type__
              ; let { types = map (\t ->
                        AnnotatedType_AnnotatedType (getRange t) False t)
                        types_ ++
                        [AnnotatedType_AnnotatedType (getRange type_) True type_]
                    
                    ; range = rangeFromTo (getRange type__) (getRange type_)
                    }
              ; return $ Constructor_Constructor range constructor types
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_162__ g__ ((Constructor_Constructor range_begin constructor types_)) type_ ts__
      = g__
                ( let { range = rangeFromTo range_begin (getRange type_) }
          in  Constructor_Constructor range constructor (types_ ++ [type_]) )
            ts__
  
  reduce_163__ g__ type_ ts__ = g__ ( AnnotatedType_AnnotatedType (getRange type_) False type_ ) ts__
  
  reduce_164__ g__ range_begin type_ ts__
      = g__
                ( let { range = rangeFromTo range_begin (getRange type_) }
          in  AnnotatedType_AnnotatedType range True type_ )
            ts__
  
  reduce_165__ g__ type_ ts__ = g__ ( AnnotatedType_AnnotatedType (getRange type_) False type_ ) ts__
  
  reduce_166__ g__ range_begin type_ ts__
      = g__
                ( let { range = rangeFromTo range_begin (getRange type_) }
          in  AnnotatedType_AnnotatedType range True type_ )
            ts__
  
  reduce_167__ g__ names_rev _ type_ ts__
      = g__
                ( let { names = reverse names_rev
              ; range = rangeFromTo (getRange names) (getRange type_) }
          in  FieldDeclaration_FieldDeclaration range names type_ )
            ts__
  
  reduce_168__ g__ type_ ts__ = g__ ( AnnotatedType_AnnotatedType (getRange type_) False type_ ) ts__
  
  reduce_169__ g__ range_begin type_ ts__
      = g__
                ( let { range = rangeFromTo range_begin (getRange type_) }
          in  AnnotatedType_AnnotatedType range True type_ )
            ts__
  
  reduce_170__ g__ ts__ = g__ ( [] ) ts__
  
  reduce_171__ g__ _ name ts__ = g__ ( [name] ) ts__
  
  reduce_172__ g__ _ _ _ ts__ = g__ ( [] ) ts__
  
  reduce_173__ g__ _ _ names _ ts__ = g__ ( names ) ts__
  
  reduce_174__ g__ s ts__ = g__ ( s [] ) ts__
  
  reduce_175__ g__ a ts__ = g__ ( \ as -> a : as ) ts__
  
  reduce_176__ g__ s _ a ts__ = g__ ( \ as -> s (a : as) ) ts__
  
  reduce_177__ g__ _ declarations ts__ = g__ ( MaybeDeclarations_Just declarations ) ts__
  
  reduce_178__ g__ ts__ = g__ ( MaybeDeclarations_Nothing ) ts__
  
  reduce_179__ g__ _ declarations _ ts__ = g__ ( declarations ) ts__
  
  reduce_180__ g__ _ declarations _ ts__ = g__ ( declarations ) ts__
  
  reduce_181__ g__ _ declarations_ _ ts__ = g__ ( groupDecls (declarations_ []) ) ts__
  
  reduce_182__ g__ _ ts__ = g__ ( [] ) ts__
  
  reduce_183__ g__ a ts__ = g__ ( \ as -> a : as ) ts__
  
  reduce_184__ g__ s _ a ts__ = g__ ( \ as -> s (a : as) ) ts__
  
  reduce_185__ g__ declaration ts__ = g__ ( declaration ) ts__
  
  reduce_186__ g__ declaration ts__ = g__ ( declaration ) ts__
  
  reduce_187__ g__ declaration ts__ = ( checkSingle declaration ) >>= (\ v1 -> g__ v1 ts__)
  
  reduce_188__ g__ _ _ declarations _ ts__ = g__ ( MaybeDeclarations_Just declarations ) ts__
  
  reduce_189__ g__ _ _ declarations _ ts__ = g__ ( MaybeDeclarations_Just declarations ) ts__
  
  reduce_190__ g__ ts__ = g__ ( MaybeDeclarations_Nothing ) ts__
  
  reduce_191__ g__ _ declarations _ ts__ = g__ ( reverse declarations ) ts__
  
  reduce_192__ g__ _ ts__ = g__ ( [] ) ts__
  
  reduce_193__ g__ declarations_ _ declaration_ ts__
      = ( do { declaration <- checkSingle declaration_
              ; return (declaration : declarations_)
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_194__ g__ declaration_ ts__
      = ( do { declaration <- checkSingle declaration_
              ; return [declaration]
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_195__ g__ lefthandside range_expression righthandside_ ts__
      = ( do { let { range = rangeFromTo (getRange lefthandside) (range_rhs)
                    ; range_rhs = range_expression
                    ; righthandside = righthandside_ range_rhs MaybeDeclarations_Nothing
                    }
              ; checkValDef (range,lefthandside,righthandside) 
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_196__ g__ lefthandside range_expression righthandside_ range_where _ where_ ts__
      = ( do { let { range_end = if null where_ then range_where else (getRange where_)
                    ; range = rangeFromTo (getRange lefthandside) range_end
                    ; range_rhs = rangeFromTo range_expression range_end
                    ; righthandside = righthandside_ range_rhs (MaybeDeclarations_Just where_)
                    }
              ; checkValDef (range,lefthandside,righthandside) 
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_197__ g__ name _ extended ts__
      = ( do { let { range = rangeFromTo (getRange name) (getRange extended)
                    }
              ; checkUnQual name
              ; return $ Declaration_TypeIndexedValue range
                           [TypeIndexedBinding_DefaultCase range name extended
                             MaybeReassignments_Nothing]
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_198__ g__ name _ extended range_where reassignments ts__
      = ( do { let { range_end = if null reassignments then range_where else (getRange reassignments)
                    ; range = rangeFromTo (getRange name) range_end
                    }
              ; checkUnQual name
              ; return $ Declaration_TypeIndexedValue range
                           [TypeIndexedBinding_DefaultCase range name extended
                             (MaybeReassignments_Just reassignments)]
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_199__ g__ ts__ = g__ ( [] ) ts__
  
  reduce_200__ g__ ( as ) ts__ = g__ ( as ) ts__
  
  reduce_201__ g__ s ts__ = g__ ( s [] ) ts__
  
  reduce_202__ g__ a ts__ = g__ ( \ as -> a : as ) ts__
  
  reduce_203__ g__ s _ a ts__ = g__ ( \ as -> s (a : as) ) ts__
  
  reduce_204__ g__ range_begin _ declarations range_end ts__
      = g__ ( rangeFromTo range_begin range_end ) ( groupDecls declarations ) ts__
  
  reduce_205__ g__ _ _ declarations _ ts__ = g__ ( getRange declarations ) ( groupDecls declarations ) ts__
  
  reduce_206__ g__ range_optbegin declarations_ range_optend ts__
      = g__
                    ( let { range_r     = getRange (declarations_ [])
              ; range_begin = maybe range_r      id range_optbegin
              ; range_end   = maybe range_r      id range_optend
              ; range       = rangeFromTo range_begin range_end
              }
          in  range )
                ( declarations_ [] )
            ts__
  
  reduce_207__ g__ range_opt ts__ = g__ ( maybe unknownRange id range_opt ) ( [] ) ts__
  
  reduce_208__ g__ a ts__ = g__ ( \ as -> a : as ) ts__
  
  reduce_209__ g__ s _ a ts__ = g__ ( \ as -> s (a : as) ) ts__
  
  reduce_210__ g__ name _ as ts__
      = g__
                ( let { range = rangeFromTo (getRange name) (getRange as) }
          in  Reassignment_Reassignment range name as
        )
            ts__
  
  reduce_211__ g__ _ expression_ ts__
      = ( do { expression <- checkExpr expression_
              ; return (\ range ds -> RightHandSide_Expression range expression ds)
              }
        )
            >>=
            (\ v2 -> g__ ( getRange expression_ ) v2 ts__)
  
  reduce_212__ g__ guardedexpressions ts__
      = g__ ( getRange guardedexpressions ) (\ range ds -> RightHandSide_Guarded range guardedexpressions ds ) ts__
  
  reduce_213__ g__ expression ts__ = g__ ( expression ) ts__
  
  reduce_214__ g__ expression ts__ = g__ ( expression ) ts__
  
  reduce_215__ g__ expression ts__ = g__ ( expression ) ts__
  
  reduce_216__ g__ leftExpression_ operator rightExpression_ ts__
      = g__
                ( let { range = rangeFromTo (getRange leftExpression) 
                                    (getRange rightExpression)
              ; leftExpression = MaybeExpression_Just leftExpression_
              ; rightExpression = MaybeExpression_Just rightExpression_ }
          in  Expression_InfixApplication range 
                                          leftExpression
                                          operator
                                          rightExpression )
            ts__
  
  reduce_217__ g__ expression ts__ = g__ ( expression ) ts__
  
  reduce_218__ g__ leftExpression_ operator rightExpression_ ts__
      = g__
                ( let { range = rangeFromTo (getRange leftExpression) 
                                    (getRange rightExpression)
              ; leftExpression = MaybeExpression_Just leftExpression_
              ; rightExpression = MaybeExpression_Just rightExpression_ }
          in  Expression_InfixApplication range 
                                          leftExpression
                                          operator
                                          rightExpression )
            ts__
  
  reduce_219__ g__ range_begin patterns_ _ expression ts__
      = ( do { patterns <- checkPatterns patterns_
              ; let { range = rangeFromTo range_begin (getRange expression) }
              ; return $ Expression_Lambda range patterns expression
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_220__ g__ range_begin _ declarations _ expression ts__
      = g__
                ( let { range = rangeFromTo range_begin (getRange expression) }
          in  Expression_Let range declarations expression )
            ts__
  
  reduce_221__ g__ range_begin guardExpression _ thenExpression _ elseExpression ts__
      = g__
                ( let { range = rangeFromTo range_begin (getRange elseExpression) }
          in  Expression_If range guardExpression 
                            thenExpression elseExpression )
            ts__
  
  reduce_222__ g__ range_begin expression _ range_end alternatives ts__
      = g__
                ( let { range = rangeFromTo range_begin range_end }
          in  Expression_Case range expression alternatives )
            ts__
  
  reduce_223__ g__ range_begin expression ts__
      = g__
                ( let { range = rangeFromTo range_begin (getRange expression) }
          in  Expression_Negate range expression 
        )
            ts__
  
  reduce_224__ g__ range_begin statements ts__
      = g__
                ( let { range_end = if null statements then range_begin
                                               else getRange statements
              ; range = rangeFromTo range_begin range_end 
              }
          in  Expression_Do range statements
        )
            ts__
  
  reduce_225__ g__ expression ts__ = g__ ( expression ) ts__
  
  reduce_226__ g__ expressions ts__
      = g__
                ( case expressions of
             { [expression]         -> expression
             ; (function:arguments) ->
                 let { range = rangeFromTo (getRange function)
                                           (getRange arguments) }
                 in  Expression_NormalApplication range function arguments } )
            ts__
  
  reduce_227__ g__ n _ ts__ = g__ ( n + 1 ) ts__
  
  reduce_228__ g__ _ ts__ = g__ ( 1 ) ts__
  
  reduce_229__ g__ range_begin alternatives range_end ts__
      = g__ ( rangeFromTo range_begin range_end ) ( alternatives ) ts__
  
  reduce_230__ g__ _ alternatives _ ts__ = g__ ( getRange alternatives ) ( alternatives ) ts__
  
  reduce_231__ g__ _ alternatives_ _ ts__ = g__ ( alternatives_ [] ) ts__
  
  reduce_232__ g__ _ ts__ = g__ ( [] ) ts__
  
  reduce_233__ g__ a ts__ = g__ ( \ as -> a : as ) ts__
  
  reduce_234__ g__ s _ a ts__ = g__ ( \ as -> s (a : as) ) ts__
  
  reduce_235__ g__ pattern_ range_rhs righthandside_ ts__
      = ( do { pattern <- checkPattern pattern_
              ; let { range = rangeFromTo (getRange pattern) 
                                          (getRange righthandside)
                    ; righthandside = righthandside_ range_rhs
                                                     MaybeDeclarations_Nothing
                    }
              ; return (Alternative_Alternative range pattern righthandside)
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_236__ g__ pattern_ range_rhs_begin righthandside_ _ range_end where_ ts__
      = ( do { pattern <- checkPattern pattern_
              ; let { range_rhs = rangeFromTo range_rhs_begin (range_end)
                    ; range = rangeFromTo (getRange pattern) (range_end)
                    ; righthandside = righthandside_ range_rhs
                                                     (MaybeDeclarations_Just where_)
                    }
              ; return (Alternative_Alternative range pattern righthandside)
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_237__ g__ range_begin expression ts__
      = g__ ( rangeFromTo range_begin (getRange expression) )
                ( \ range where_ -> RightHandSide_Expression range expression where_ )
            ts__
  
  reduce_238__ g__ guardexpressions ts__
      = g__ ( getRange guardexpressions ) ( \ range where_ -> RightHandSide_Guarded range guardexpressions where_ ) ts__
  
  reduce_239__ g__ _ statements _ ts__ = g__ ( statements ) ts__
  
  reduce_240__ g__ _ statements _ ts__ = g__ ( statements ) ts__
  
  reduce_241__ g__ range_begin range_end declarations _ statements ts__
      = g__
                ( let { range = rangeFromTo range_begin range_end }
          in  (Statement_Let range declarations : statements)
        )
            ts__
  
  reduce_242__ g__ pattern_ _ expression _ statements ts__
      = ( do { pattern <- checkPattern pattern_
              ; let { range = rangeFromTo (getRange pattern)
                                          (getRange expression)
                    }
              ; return ( Statement_Generator range pattern expression 
                       : statements
                       )
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_243__ g__ expression _ statements ts__
      = g__ ( ( Statement_Expression (getRange expression) expression
          : statements
          )
        ) ts__
  
  reduce_244__ g__ _ statements ts__ = g__ ( statements ) ts__
  
  reduce_245__ g__ expression _ ts__ = g__ ( [Statement_Expression (getRange expression) expression] ) ts__
  
  reduce_246__ g__ expression ts__ = g__ ( [Statement_Expression (getRange expression) expression] ) ts__
  
  reduce_247__ g__ range name_ ts__ = g__ ( name_ range [] ) ts__
  
  reduce_248__ g__ range_begin _ name_ range_end ts__
      = g__ ( let { range = rangeFromTo range_begin range_end }
          in  name_ range []
        ) ts__
  
  reduce_249__ g__ range name_ ts__ = g__ ( name_ range ) ts__
  
  reduce_250__ g__ range_begin _ name_ range_end ts__
      = g__ ( let { range = rangeFromTo range_begin range_end }
          in  name_ range
        ) ts__
  
  reduce_251__ g__ range name_ ts__ = g__ ( name_ range [] ) ts__
  
  reduce_252__ g__ range_begin _ name_ range_end ts__
      = g__ ( let { range = rangeFromTo range_begin range_end }
          in  name_ range []
        ) ts__
  
  reduce_253__ g__ name ts__ = g__ ( name ) ts__
  
  reduce_254__ g__ range_begin _ name_ range_end ts__
      = g__ ( let { range = rangeFromTo range_begin range_end }
          in  name_ range
        ) ts__
  
  reduce_255__ g__ range name_ ts__ = g__ ( name_ range [] ) ts__
  
  reduce_256__ g__ range_begin _ name_ range_end ts__
      = g__ ( let { range = rangeFromTo range_begin range_end }
          in  name_ range []
        ) ts__
  
  reduce_257__ g__ name ts__ = g__ ( name ) ts__
  
  reduce_258__ g__ name ts__ = g__ ( name ) ts__
  
  reduce_259__ g__ range name_ ts__ = g__ ( name_ range [] ) ts__
  
  reduce_260__ g__ range_begin _ name_ range_end ts__
      = g__ ( let { range = rangeFromTo range_begin range_end }
          in  name_ range []
        ) ts__
  
  reduce_261__ g__ name ts__ = g__ ( Expression_Variable (getRange name) name ) ts__
  
  reduce_262__ g__ name ts__ = g__ ( Expression_Constructor (getRange name) name ) ts__
  
  reduce_263__ g__ range name_ ts__ = g__ ( name_ range ) ts__
  
  reduce_264__ g__ range_begin _ name_ range_end ts__
      = g__ ( let { range = rangeFromTo range_begin range_end }
          in name_ range
        ) ts__
  
  reduce_265__ g__ range name_ ts__ = g__ ( name_ range ) ts__
  
  reduce_266__ g__ range_begin _ name_ range_end ts__
      = g__ ( let { range = rangeFromTo range_begin range_end }
          in  name_ range
        ) ts__
  
  reduce_267__ g__ range name_ ts__ = g__ ( range ) ( \ range -> name_ range [] ) ts__
  
  reduce_268__ g__ ((md,name)) range ts__ = g__ ( range ) ( \ range -> unmangle name range md ) ts__
  
  reduce_269__ g__ x range ts__ = g__ range (unmangle x) ts__
  
  reduce_270__ g__ range ts__ = g__ range hsAsName ts__
  
  reduce_271__ g__ range ts__ = g__ range hsQualifiedName ts__
  
  reduce_272__ g__ range ts__ = g__ range hsHidingName ts__
  
  reduce_273__ g__ range name_ ts__ = g__ ( name_ range ) ts__
  
  reduce_274__ g__ range name_ ts__ = g__ ( range ) ( \ range -> name_ range [] ) ts__
  
  reduce_275__ g__ ((md,name)) range ts__ = g__ ( range ) ( \ range -> unmangle name range md ) ts__
  
  reduce_276__ g__ x range ts__ = g__ range (unmangle x) ts__
  
  reduce_277__ g__ range name_ ts__ = g__ ( name_ range ) ts__
  
  reduce_278__ g__ range name_ ts__ = g__ ( range ) ( \ range -> name_ range [] ) ts__
  
  reduce_279__ g__ ((md,name)) range ts__ = g__ ( range ) ( \ range -> Name_Operator range md name ) ts__
  
  reduce_280__ g__ name range ts__ = g__ ( range ) ( \ range module_ -> Name_Operator range module_ name ) ts__
  
  reduce_281__ g__ range name_ ts__ = g__ ( range ) ( \ range -> name_ range [] ) ts__
  
  reduce_282__ g__ range name_ ts__ = g__ ( range ) ( name_ ) ts__
  
  reduce_283__ g__ name range ts__ = g__ ( range ) ( \ range module_ -> Name_Operator range module_ name ) ts__
  
  reduce_284__ g__ range ts__ = g__ ( range ) ( hsMinusName ) ts__
  
  reduce_285__ g__ range ts__ = g__ ( range ) ( hsPlingName ) ts__
  
  reduce_286__ g__ range ts__ = g__ ( range ) ( hsDotName ) ts__
  
  reduce_287__ g__ ((md,name)) range ts__ = g__ ( range ) ( \ range -> Name_Operator range md name ) ts__
  
  reduce_288__ g__ ts__ = ( return () ) >>= (\ v1 -> g__ v1 ts__)
  
  reduce_289__ g__ ts__ = ( popContext ) >>= (\ v1 -> g__ v1 ts__)
  
  reduce_290__ g__ ts__ = ( layoutOn True {- strict -} ) >>= (\ v1 -> g__ v1 ts__)
  
  reduce_291__ g__ ts__ = ( layoutOn False ) >>= (\ v1 -> g__ v1 ts__)
  
  reduce_292__ g__ md range ts__ = g__ ( ModuleName_ModuleName range [md] ) ts__
  
  reduce_293__ g__ ((md,md0)) range ts__ = g__ ( ModuleName_ModuleName range (md ++ [md0]) ) ts__
  
  reduce_294__ g__ range name_ ts__ = g__ ( range ) ( name_ range [] ) ts__
  
  reduce_295__ g__ range name_ ts__ = g__ ( name_ range [] ) ts__
  
  reduce_296__ g__ name ts__ = g__ ( name ) ts__
  
  reduce_297__ g__ range name_ ts__ = g__ ( name_ range [] ) ts__
  
  reduce_298__ g__ _ kindargument _ ts__ = g__ ( kindargument ) ts__
  
  reduce_299__ g__ _ typeargument _ ts__ = g__ ( typeargument ) ts__
  
  reduce_300__ g__ range_begin kind range_end ts__ = g__ ( rangeFromTo range_begin range_end ) ( kind ) ts__
  
  reduce_301__ g__ range_begin type_ range_end ts__ = g__ ( rangeFromTo range_begin range_end ) ( type_ ) ts__
  
  reduce_302__ g__ range_begin first _ generics range_end ts__
      = g__ ( rangeFromTo range_begin range_end )
                ( let { range_begin = getRange first
              ; range_end   = getRange generics
              ; range = rangeFromTo range_begin range_end
              }
          in  Type_TypeSignature range (first : generics) []
        )
            ts__
  
  reduce_303__ g__ range_begin first _ generics _ parametrics range_end ts__
      = g__ ( rangeFromTo range_begin range_end )
                ( let { range_begin = getRange first
              ; range_end   = getRange parametrics
              ; range       = rangeFromTo range_begin range_end
              }
          in  Type_TypeSignature range (first : generics) parametrics
        )
            ts__
  
  reduce_304__ g__ range_begin generic range_end ts__
      = g__ ( rangeFromTo range_begin range_end ) ( Type_TypeSignature (getRange generic) [generic] [] ) ts__
  
  reduce_305__ g__ range_begin generic _ parametrics range_end ts__
      = g__ ( rangeFromTo range_begin range_end )
                ( let { range_begin = getRange generic
              ; range_end   = getRange parametrics
              ; range       = rangeFromTo range_begin range_end
              }
          in  Type_TypeSignature range [generic] parametrics )
            ts__
  
  reduce_306__ g__ ts__ = g__ ( [] ) ts__
  
  reduce_307__ g__ ( as ) ts__ = g__ ( as ) ts__
  
  reduce_308__ g__ s ts__ = g__ ( s [] ) ts__
  
  reduce_309__ g__ s ts__ = g__ ( s [] ) ts__
  
  reduce_310__ g__ a ts__ = g__ ( \ as -> a : as ) ts__
  
  reduce_311__ g__ s _ a ts__ = g__ ( \ as -> s (a : as) ) ts__
  
  reduce_312__ g__ a ts__ = g__ ( \ as -> a : as ) ts__
  
  reduce_313__ g__ s _ a ts__ = g__ ( \ as -> s (a : as) ) ts__
  
  reduce_314__ g__ name ts__
      = g__
                ( let { range = getRange name }
          in KindAnnotatedName_KindAnnotatedName range name MaybeKind_Nothing
        )
            ts__
  
  reduce_315__ g__ name _ kind ts__
      = g__
                ( let { range_begin  =  getRange name
              ; range_end    =  getRange kind
              ; range        =  rangeFromTo range_begin range_end }
          in KindAnnotatedName_KindAnnotatedName range name (MaybeKind_Just kind)
        )
            ts__
  
  reduce_316__ g__ name _ kind ts__
      = g__
                ( let { range_begin  =  getRange name
              ; range_end    =  getRange kind
              ; range        =  rangeFromTo range_begin range_end }
          in KindAnnotatedName_KindAnnotatedName range name (MaybeKind_Just kind)
        )
            ts__
  
  reduce_317__ g__ tp ts__ = g__ ( tp ) ts__
  
  reduce_318__ g__ tp ts__
      = g__
                ( case tp of
            { KindAnnotatedName_KindAnnotatedName r n (MaybeKind_Just k) ->
                Type_Kinded r n k } )
            ts__
  
  reduce_319__ g__ star ts__
      = ( do { checkStar star
              ; return (KindArgument_Star (getRange star))
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_320__ g__ leftkindvariable _ rightkindvariable ts__
      = g__
                ( let { range = rangeFromTo (getRange leftkindvariable )
                                    (getRange rightkindvariable)
              }
          in  KindArgument_Arrow range leftkindvariable rightkindvariable
        )
            ts__
  
  reduce_321__ g__ leftKind _ rightKind ts__
      = g__
                ( let { range = rangeFromTo (getRange leftKind)
                                    (getRange rightKind)
              }
          in  Kind_Arrow range leftKind rightKind
        )
            ts__
  
  reduce_322__ g__ kind ts__ = g__ ( kind ) ts__
  
  reduce_323__ g__ star ts__
      = ( do { checkStar star
              ; return $ Kind_Star (getRange star)
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_324__ g__ name ts__ = g__ ( Kind_Variable (getRange name) name ) ts__
  
  reduce_325__ g__ range_begin kind range_end ts__
      = g__ ( let { range = rangeFromTo range_begin range_end }
          in  Kind_Parenthesized range kind
        ) ts__
  
  reduce_326__ g__ range name_ ts__ = g__ ( name_ range [] ) ts__
  
  reduce_327__ g__ type_ ts__ = g__ ( type_ ) ts__
  
  reduce_328__ g__ name ts__ = g__ ( Type_Constructor (getRange name) name ) ts__
  
  reduce_329__ g__ range_begin name ts__
      = g__ ( let { range = rangeFromTo range_begin (getRange name) }
          in  Type_ConstructorCase range name 
        )
            ts__
  
  reduce_330__ g__ range_begin star ts__
      = ( do { checkStar star
              ; return $ Type_KindStarCase 
                           (rangeFromTo range_begin (getRange star))
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_331__ g__ type_ ts__ = ( checkTyArg True type_ ) >>= (\ v1 -> g__ v1 ts__)
  
  reduce_332__ g__ type_ ts__ = ( checkTyArg False type_ ) >>= (\ v1 -> g__ v1 ts__)
  
  reduce_333__ g__ range_begin names range_end ts__ = g__ ( rangeFromTo range_begin range_end ) ( names ) ts__
  
  reduce_334__ g__ ts__ = g__ Nothing ts__
  
  reduce_335__ g__ a ts__ = g__ (Just a) ts__
  
  reduce_336__ g__ ts__ = g__ Nothing ts__
  
  reduce_337__ g__ a ts__ = g__ (Just a) ts__
  
  reduce_338__ g__ ts__ = g__ Nothing ts__
  
  reduce_339__ g__ a ts__ = g__ (Just a) ts__
  
  reduce_340__ g__ _ exports _ _ ts__ = g__ ( exports ) ts__
  
  reduce_341__ g__ _ _ _ ts__ = g__ ( [] ) ts__
  
  reduce_342__ g__ s ts__ = g__ ( s [] ) ts__
  
  reduce_343__ g__ a ts__ = g__ ( \ as -> a : as ) ts__
  
  reduce_344__ g__ s _ a ts__ = g__ ( \ as -> s (a : as) ) ts__
  
  reduce_345__ g__ name ts__ = g__ ( Export_Variable (getRange name) name ) ts__
  
  reduce_346__ g__ name ts__ = g__ ( Export_TypeOrClass (getRange name) name MaybeNames_Nothing ) ts__
  
  reduce_347__ g__ name _ _ range_end ts__
      = g__ ( let { range = rangeFromTo (getRange name) range_end }
          in  Export_TypeOrClassComplete range name )
            ts__
  
  reduce_348__ g__ name _ range_end ts__
      = g__
                ( let { range = rangeFromTo (getRange name) range_end }
          in  Export_TypeOrClass range name (MaybeNames_Just []) )
            ts__
  
  reduce_349__ g__ name _ names range_end ts__
      = g__
                ( let { range = rangeFromTo (getRange name) range_end }
          in  Export_TypeOrClass range name (MaybeNames_Just names) )
            ts__
  
  reduce_350__ g__ range_begin name ts__
      = g__ ( let { range = rangeFromTo range_begin (getRange name) }
          in  Export_Module range name ) ts__
  
  reduce_351__ g__ s ts__ = g__ ( s [] ) ts__
  
  reduce_352__ g__ a ts__ = g__ ( \ as -> a : as ) ts__
  
  reduce_353__ g__ s _ a ts__ = g__ ( \ as -> s (a : as) ) ts__
  
  reduce_354__ g__ name ts__ = g__ ( name ) ts__
  
  reduce_355__ g__ ts__ = g__ Nothing ts__
  
  reduce_356__ g__ a ts__ = g__ (Just a) ts__
  
  reduce_357__ g__ _ name ts__ = g__ ( name ) ts__
  
  reduce_358__ g__ s ts__ = g__ (s []) ts__
  
  reduce_359__ g__ s ts__ = g__ (s []) ts__
  
  reduce_360__ g__ s ts__ = g__ (s []) ts__
  
  reduce_361__ g__ s ts__ = g__ (s []) ts__
  
  reduce_362__ g__ s ts__ = g__ (s []) ts__
  
  reduce_363__ g__ s ts__ = g__ (s []) ts__
  
  reduce_364__ g__ ts__ = g__ (\ as -> as) ts__
  
  reduce_365__ g__ s a ts__ = g__ (\ as -> s (a : as)) ts__
  
  reduce_366__ g__ ts__ = g__ (\ as -> as) ts__
  
  reduce_367__ g__ s a ts__ = g__ (\ as -> s (a : as)) ts__
  
  reduce_368__ g__ name ts__ = g__ ( DependencyBinding_Simple (getRange name) name ) ts__
  
  reduce_369__ g__ range_begin name _ as range_end ts__
      = g__ ( DependencyBinding_Rename (rangeFromTo range_begin range_end) name as ) ts__
  
  reduce_370__ g__ ts__ = g__ (\ as -> as) ts__
  
  reduce_371__ g__ s a ts__ = g__ (\ as -> s (a : as)) ts__
  
  reduce_372__ g__ name ts__ = g__ ( DependencyBinding_Simple (getRange name) name ) ts__
  
  reduce_373__ g__ range_begin name _ as range_end ts__
      = g__ ( DependencyBinding_Rename (rangeFromTo range_begin range_end) name as ) ts__
  
  reduce_374__ g__ ts__ = g__ (\ as -> as) ts__
  
  reduce_375__ g__ s a ts__ = g__ (\ as -> s (a : as)) ts__
  
  reduce_376__ g__ name ts__ = g__ ( DependencyBinding_Simple (getRange name) name ) ts__
  
  reduce_377__ g__ range_begin name _ as range_end ts__
      = g__ ( DependencyBinding_Rename (rangeFromTo range_begin range_end) name as ) ts__
  
  reduce_378__ g__ ts__ = g__ (\ as -> as) ts__
  
  reduce_379__ g__ s a ts__ = g__ (\ as -> s (a : as)) ts__
  
  reduce_380__ g__ name ts__
      = g__
                ( let { range = getRange name }
          in KindAnnotatedName_KindAnnotatedName range name MaybeKind_Nothing
        )
            ts__
  
  reduce_381__ g__ range_begin name _ kind range_end ts__
      = g__
                ( let { range = rangeFromTo range_begin range_end }
          in KindAnnotatedName_KindAnnotatedName range name (MaybeKind_Just kind)
        )
            ts__
  
  reduce_382__ g__ ts__ = g__ (\ as -> as) ts__
  
  reduce_383__ g__ s a ts__ = g__ (\ as -> s (a : as)) ts__
  
  reduce_384__ g__ a as ts__ = g__ (a : as) ts__
  
  reduce_385__ g__ range_begin guard_ _ expression_ ts__
      = ( do { guard      <- checkExpr guard_
              ; expression <- checkExpr expression_
              ; let { range = rangeFromTo range_begin (getRange expression) }
              ; return $ GuardedExpression_GuardedExpression
                            range guard expression
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_386__ g__ s ts__ = g__ (s []) ts__
  
  reduce_387__ g__ ts__ = g__ (\ as -> as) ts__
  
  reduce_388__ g__ s a ts__ = g__ (\ as -> s (a : as)) ts__
  
  reduce_389__ g__ a as ts__ = g__ (a : as) ts__
  
  reduce_390__ g__ name_ _ pattern ts__
      = ( do { name <- checkUnQual name_
              ; let { range = rangeFromTo (getRange name_) (getRange pattern) }
              ; return $ Expression_As range name_ pattern
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_391__ g__ range_begin pattern ts__
      = g__
                ( let { range = rangeFromTo range_begin (getRange pattern) }
          in  Expression_Irrefutable range pattern )
            ts__
  
  reduce_392__ g__ expression ts__ = g__ ( expression ) ts__
  
  reduce_393__ g__ expression _ range_end ts__
      = ( do { let { range = rangeFromTo (getRange expression) range_end }
              ; mkRecConstrOrUpdate range expression []
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_394__ g__ expression _ recordExpressionBindings range_end ts__
      = ( do { let { range = rangeFromTo (getRange expression) range_end }
              ; mkRecConstrOrUpdate range expression recordExpressionBindings
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_395__ g__ expression ts__ = g__ ( expression ) ts__
  
  reduce_396__ g__ s ts__ = g__ ( s [] ) ts__
  
  reduce_397__ g__ a ts__ = g__ ( \ as -> a : as ) ts__
  
  reduce_398__ g__ s _ a ts__ = g__ ( \ as -> s (a : as) ) ts__
  
  reduce_399__ g__ name ts__
      = g__
                ( let { inner = Expression_Variable (getRange name) name }
          in  if   isOperator name
              then Expression_InfixApplication (getRange name) MaybeExpression_Nothing inner MaybeExpression_Nothing
              else inner
        )
            ts__
  
  reduce_400__ g__ name range_end type_ ts__
      = g__
                ( let { range = rangeFromTo (getRange name) (getRange type_) }
          in  Expression_TypeApplication range name type_ )
            ts__
  
  reduce_401__ g__ expression ts__ = g__ ( expression ) ts__
  
  reduce_402__ g__ literal ts__ = g__ ( Expression_Literal (getRange literal) literal ) ts__
  
  reduce_403__ g__ range_begin expressions range_end ts__
      = g__
                ( let { range = rangeFromTo range_begin range_end }
          in  case expressions of
               { [expression] -> Expression_Parenthesized range expression
               ; _            -> Expression_Tuple range expressions } )
            ts__
  
  reduce_404__ g__ range_begin expression_ range_end ts__
      = g__ ( let { range = rangeFromTo range_begin range_end }
          in  expression_ range ) ts__
  
  reduce_405__ g__ range_begin leftExpression_ operator range_end ts__
      = g__
                ( let { range = rangeFromTo range_begin range_end
              ; leftExpression = MaybeExpression_Just leftExpression_
              ; rightExpression = MaybeExpression_Nothing
              }
          in  Expression_InfixApplication 
                 range leftExpression operator rightExpression )
            ts__
  
  reduce_406__ g__ range_begin operator rightExpression_ range_end ts__
      = g__
                ( let { range = rangeFromTo range_begin range_end
              ; leftExpression = MaybeExpression_Nothing
              ; rightExpression = MaybeExpression_Just rightExpression_
              }
          in  Expression_InfixApplication
                 range leftExpression operator rightExpression )
            ts__
  
  reduce_407__ g__ range ts__ = g__ ( Expression_Wildcard range ) ts__
  
  reduce_408__ g__ s ts__ = g__ ( s [] ) ts__
  
  reduce_409__ g__ a ts__ = g__ ( \ as -> a : as ) ts__
  
  reduce_410__ g__ s _ a ts__ = g__ ( \ as -> s (a : as) ) ts__
  
  reduce_411__ g__ expression ts__ = g__ ( \ range -> Expression_List range [expression] ) ts__
  
  reduce_412__ g__ expressions_ ts__ = g__ ( \ range -> Expression_List range (reverse expressions_) ) ts__
  
  reduce_413__ g__ from _ ts__
      = g__
                ( \ range -> Expression_Enum range 
                       from
                       MaybeExpression_Nothing 
                       MaybeExpression_Nothing )
            ts__
  
  reduce_414__ g__ from _ then_ _ ts__
      = g__
                ( \ range -> Expression_Enum range
                       from
                       (MaybeExpression_Just then_)
                       MaybeExpression_Nothing )
            ts__
  
  reduce_415__ g__ from _ to ts__
      = g__
                ( \ range -> Expression_Enum range
                       from
                       MaybeExpression_Nothing 
                       (MaybeExpression_Just to) )
            ts__
  
  reduce_416__ g__ from _ then_ _ to ts__
      = g__
                ( \ range -> Expression_Enum range 
                       from
                       (MaybeExpression_Just then_)
                       (MaybeExpression_Just to) )
            ts__
  
  reduce_417__ g__ expression _ qualifiers ts__
      = g__ ( \ range -> Expression_Comprehension range expression qualifiers ) ts__
  
  reduce_418__ g__ s ts__ = g__ ( s [] ) ts__
  
  reduce_419__ g__ a ts__ = g__ ( \ as -> a : as ) ts__
  
  reduce_420__ g__ s _ a ts__ = g__ ( \ as -> s (a : as) ) ts__
  
  reduce_421__ g__ xs _ x ts__ = g__ ( x : xs ) ts__
  
  reduce_422__ g__ x1 _ x2 ts__ = g__ ( [x2,x1] ) ts__
  
  reduce_423__ g__ pattern_ _ expression ts__
      = ( do { pattern <- checkPattern pattern_
              ; let { range = rangeFromTo (getRange pattern) 
                                          (getRange expression) }
              ; return $ Qualifier_Generator range pattern expression
              }
        )
            >>=
            (\ v1 -> g__ v1 ts__)
  
  reduce_424__ g__ guard ts__ = g__ ( Qualifier_Guard (getRange guard) guard ) ts__
  
  reduce_425__ g__ range_begin range_decls declarations ts__
      = g__
                ( let { range_end = if null declarations then range_begin
                                                 else range_decls
              ; range     = rangeFromTo range_begin range_end 
              }
          in  Qualifier_Let range declarations )
            ts__
  
  reduce_426__ g__ name _ expression ts__
      = g__
                ( let { range = rangeFromTo (getRange name) (getRange expression) }
            in  RecordExpressionBinding_RecordExpressionBinding range name expression
          )
            ts__
  
  reduce_427__ g__ range_begin range_end ts__
      = g__
                ( let { range = rangeFromTo range_begin range_end }
          in  Expression_Constructor range hsUnitConName
        )
            ts__
  
  reduce_428__ g__ range_begin range_end ts__
      = g__ ( let { range = rangeFromTo range_begin range_end }
          in  Expression_List range []
        ) ts__
  
  reduce_429__ g__ range_begin n range_end ts__
      = g__
                ( let { range = rangeFromTo range_begin range_end }
          in  Expression_Constructor range (hsTupleConName (n+1))
        )
            ts__
  
  reduce_430__ g__ name ts__ = g__ ( Expression_Constructor (getRange name) name ) ts__
  
  reduce_431__ g__ name ts__ = g__ ( Expression_Variable (getRange name) name ) ts__
  
  reduce_432__ g__ name ts__ = g__ ( Expression_Constructor (getRange name) name ) ts__
  
  reduce_433__ g__ name ts__ = g__ ( name ) ts__
  
  reduce_434__ g__ range_begin _ name_ range_end ts__
      = g__ ( let { range = rangeFromTo range_begin range_end }
          in name_ range
        ) ts__
  
  reduce_435__ g__ name ts__ = g__ ( name ) ts__
  
  reduce_436__ g__ range name_ ts__ = g__ ( name_ range ) ts__
  
  reduce_437__ g__ name range ts__ = g__ ( Name_Operator range [] name ) ts__
  
  reduce_438__ g__ range ts__ = g__ ( hsPlingName range [] ) ts__
  
  reduce_439__ g__ value range ts__ = g__ ( Literal_Int range value ) ts__
  
  reduce_440__ g__ value_ range ts__ = g__ ( Literal_Char range [value_] ) ts__
  
  reduce_441__ g__ value range ts__ = g__ ( Literal_Float range value ) ts__
  
  reduce_442__ g__ value range ts__ = g__ ( Literal_String range value ) ts__
  
  reduce_443__ g__ s ts__ = g__ (s []) ts__
  
  reduce_444__ g__ ts__ = g__ (\ as -> as) ts__
  
  reduce_445__ g__ s a ts__ = g__ (\ as -> s (a : as)) ts__
  
  reduce_446__ g__ a as ts__ = g__ (a : as) ts__
  
  reduce_447__ g__ range_begin guard _ expression ts__
      = g__
                ( let { range = rangeFromTo range_begin (getRange expression) }
          in  GuardedExpression_GuardedExpression range guard expression
        )
            ts__
  
  reduce_448__ g__ s ts__ = g__ (s []) ts__
  
  reduce_449__ g__ ts__ = g__ (\ as -> as) ts__
  
  reduce_450__ g__ s a ts__ = g__ (\ as -> s (a : as)) ts__
  
  reduce_451__ g__ a as ts__ = g__ (a : as) ts__
  
  reduce_452__ g__ s ts__ = g__ (s []) ts__
  
  reduce_453__ g__ ts__ = g__ (\ as -> as) ts__
  
  reduce_454__ g__ s a ts__ = g__ (\ as -> s (a : as)) ts__


{- )-: frown -}


